/* Copyright 2003 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved. */

%macro cycxrate ;

%global fmsetl_error;
%global fmsetl_errormsg;
%global fmsetl_statusId;
%global format_missing;
%global table_name_missing;

%let fmsetl_error = ;
%let fmsetl_errormsg = ;
%let fmsetl_statusId = 1 ; /* Will be changed to 0 if program runs successful */
%let format_missing = 0 ;
%let table_name_missing = 0 ;

%let entry = cycxrate;
%let putlog = Y;

%let curr_rec_dttm = '01JAN5999:00:00'DT;

/******************************************************************************************/
/*          Provide names for the necessary reference tables to be created                */
/******************************************************************************************/

%let time_member_table  = WORK._SDM_TIME_MBRS ;
%let currency_table     = WORK._CYCLE_CURRENCIES ;
%let xrate_type_table   = WORK._XRATETYPES ;
%let dim_type_table     = WORK._CYCLEDIMTYPE ;
%let dimension_table    = WORK._CYCLEDIM ;

/******************************************************************************************/
/*                 This macro will write processing errors to the log                     */
/******************************************************************************************/

%macro _chkError ;
  %global _tmpRC;
  %let _tmpRC = 0;
  %if (&sysrc ne 0) or (&syserr ne 0 and &syserr ne 4) %then %do;
    %if (&sysrc ne 0) %then
        %let _tmpRC = &sysrc;
    %else
        %let _tmpRC = &syserr;
    %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=999
            , msgTxt1=&_tmpRC, msgTxt2=&sysmsg, msgPutlog=&putlog);
  %end;
%mend _chkError ;

data null ;
   call symput( 'nStartTime', datetime()) ;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

/******************************************************************************************/
/*                       Validation of global macro variables                             */
/******************************************************************************************/

%if %symexist(fmsetl_xrateTypeIds_SDM) %then %do;
   %if %nrbquote(&fmsetl_xrateTypeIds_SDM) = %str() %then %do;
      %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=18,
               msgTxt1=<fmsetl_xrateTypeIds_SDM : &fmsetl_xrateTypeIds_SDM>, msgPutlog=&putlog);
      %goto exit;
   %end;
%end;
%else %do;
   %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=21,
            msgTxt1=<fmsetl_xrateTypeIds_SDM>, msgPutlog=&putlog);
   %goto exit;
%end;

%if %symexist(fmsetl_xrateTypeCodes) %then %do;
   %if %nrbquote(&fmsetl_xrateTypeCodes) = %str() %then %do;
      %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=18,
               msgTxt1=<fmsetl_xrateTypeCodes : &fmsetl_xrateTypeCodes>, msgPutlog=&putlog);
      %goto exit;
   %end;
%end;
%else %do;
   %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=21,
            msgTxt1=<fmsetl_xrateTypeCodes>, msgPutlog=&putlog);
   %goto exit;
%end;

%if %symexist(fmsetl_currencyids_sdm) %then %do;
   %if %nrbquote(&fmsetl_currencyids_sdm) = %str() %then %do;
      %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=18,
               msgTxt1=<fmsetl_currencyids_sdm : &fmsetl_currencyids_sdm>, msgPutlog=&putlog);
      %goto exit;
   %end;
%end;
%else %do;
   %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=21,
            msgTxt1=<fmsetl_currencyids_sdm>, msgPutlog=&putlog);
   %goto exit;
%end;

%if %symexist(fmsetl_currencycodes) %then %do;
   %if %nrbquote(&fmsetl_currencycodes) = %str() %then %do;
      %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=18,
               msgTxt1=<fmsetl_currencycodes : &fmsetl_currencycodes>, msgPutlog=&putlog);
      %goto exit;
   %end;
%end;
%else %do;
   %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=21,
            msgTxt1=<fmsetl_currencycodes>, msgPutlog=&putlog);
   %goto exit;
%end;

%if %symexist(fmsetl_baseCurrencyId_SDM) %then %do;
   %if %nrbquote(&fmsetl_baseCurrencyId_SDM) = %str() %then %do;
      %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=18,
               msgTxt1=<fmsetl_baseCurrencyId_SDM : &fmsetl_baseCurrencyId_SDM>, msgPutlog=&putlog);
      %goto exit;
   %end;
%end;
%else %do;
   %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=21,
            msgTxt1=<fmsetl_baseCurrencyId_SDM>, msgPutlog=&putlog);
   %goto exit;
%end;

%if %symexist(fmsetl_baseCurrencyCode) %then %do;
   %if %nrbquote(&fmsetl_baseCurrencyCode) = %str() %then %do;
      %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=18,
               msgTxt1=<fmsetl_baseCurrencyCode : &fmsetl_baseCurrencyCode>, msgPutlog=&putlog);
      %goto exit;
   %end;
%end;
%else %do;
   %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=21,
            msgTxt1=<fmsetl_baseCurrencyCode>, msgPutlog=&putlog);
   %goto exit;
%end;

%if %symexist(fmsetl_timePeriodIds_SDM) %then %do;
   %if %nrbquote(&fmsetl_timePeriodIds_SDM) = %str() %then %do;
      %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=18,
               msgTxt1=<fmsetl_timePeriodIds_SDM : &fmsetl_timePeriodIds_SDM>, msgPutlog=&putlog);
      %goto exit;
   %end;
%end;
%else %do;
   %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=21,
            msgTxt1=<fmsetl_timePeriodIds_SDM>, msgPutlog=&putlog);
   %goto exit;
%end;

%if %symexist(fmsetl_timePeriodCodes) %then %do;
   %if %nrbquote(&fmsetl_timePeriodCodes) = %str() %then %do;
      %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=18,
               msgTxt1=<fmsetl_timePeriodCodes : &fmsetl_timePeriodCodes>, msgPutlog=&putlog);
      %goto exit;
   %end;
%end;
%else %do;
   %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=21,
            msgTxt1=<fmsetl_timePeriodCodes>, msgPutlog=&putlog);
   %goto exit;
%end;

%if %symexist(fmsetl_xrateSetId_SDM) %then %do;
   %if %nrbquote(&fmsetl_xrateSetId_SDM) = %str() %then %do;
      %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=18,
               msgTxt1=<fmsetl_xrateSetId_SDM : &fmsetl_xrateSetId_SDM>, msgPutlog=&putlog);
      %goto exit;
   %end;
%end;
%else %do;
   %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=21,
            msgTxt1=<fmsetl_xrateSetId_SDM>, msgPutlog=&putlog);
   %goto exit;
%end;

%if %symexist(fmsetl_exchgRateSetCode_DDS) %then %do;
   %if %nrbquote(&fmsetl_exchgRateSetCode_DDS) = %str() %then %do;
      %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=18,
               msgTxt1=<fmsetl_exchgRateSetCode_DDS : &fmsetl_exchgRateSetCode_DDS>, msgPutlog=&putlog);
      %goto exit;
   %end;
%end;
%else %do;
   %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=21,
            msgTxt1=<fmsetl_exchgRateSetCode_DDS>, msgPutlog=&putlog);
   %goto exit;
%end;

%if %symexist(fmsetl_processedDTTM_From) %then %do;
   %if %datatyp(&fmsetl_processedDTTM_From) ne NUMERIC %then %do;
      %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=1,
                msgTxt1=<fmsetl_processedDTTM_From : &fmsetl_processedDTTM_From>, msgPutlog=&putlog);
      %goto exit;
   %end;
%end;
%else %do;
   %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=21,
            msgTxt1=<fmsetl_processedDTTM_From>, msgPutlog=&putlog);
   %goto exit;
%end;

%if %symexist(fmsetl_processedDTTM_To) %then %do;
   %if %datatyp(&fmsetl_processedDTTM_To) ne NUMERIC %then %do;
      %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=1,
                msgTxt1=<fmsetl_processedDTTM_To : &fmsetl_processedDTTM_To>, msgPutlog=&putlog);
      %goto exit;
   %end;
%end;
%else %do;
   %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=21,
            msgTxt1=<fmsetl_processedDTTM_To>, msgPutlog=&putlog);
   %goto exit;
%end;

%if %symexist(fmsetl_tempCreateSimpleTableNm) %then %do;
   %if %nrbquote(&fmsetl_tempCreateSimpleTableNm) = %str() %then %do;
      %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=18,
               msgTxt1=<fmsetl_tempCreateSimpleTableNm : &fmsetl_tempCreateSimpleTableNm>, msgPutlog=&putlog);
      %goto exit;
   %end;
%end;
%else %do;
   %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=21,
            msgTxt1=<fmsetl_tempCreateSimpleTableNm>, msgPutlog=&putlog);
   %goto exit;
%end;

%if %symexist(fmsetl_tempCreateComplexTableNm) %then %do;
   %if %nrbquote(&fmsetl_tempCreateComplexTableNm) = %str() %then %do;
      %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=18,
               msgTxt1=<fmsetl_tempCreateComplexTableNm : &fmsetl_tempCreateComplexTableNm>, msgPutlog=&putlog);
      %goto exit;
   %end;
%end;
%else %do;
   %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=21,
            msgTxt1=<fmsetl_tempCreateComplexTableNm>, msgPutlog=&putlog);
   %goto exit;
%end;

%if %symexist(fmsetl_tempStatusTableNm) %then %do;
   %if %nrbquote(&fmsetl_tempStatusTableNm) = %str() %then %do;
      %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=18,
               msgTxt1=<fmsetl_tempStatusTableNm : &fmsetl_tempStatusTableNm>, msgPutlog=&putlog);
      %goto exit;
   %end;
%end;
%else %do;
   %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=21,
            msgTxt1=<fmsetl_tempStatusTableNm>, msgPutlog=&putlog);
   %goto exit;
%end;

/******************************************************************************************/
/*               Assign librefs for Conform(_CONFORM) and SASSDM(_SDMLIB)                 */
/******************************************************************************************/

%bldLibNm(sesetl_sasLibRef=_CONFORM
        , sesetl_OMRSASLibraryObjName=&Soldatatier_folder./&CONFLibObj./&CONFLibObj.
        , sesetl_useType=META
        , sesetl_validateLib=Y
        ) ;

%bldLibNm(sesetl_sasLibRef=_SDMLIB
        , sesetl_OMRSASLibraryObjName=&Solsystem_folder./&SdmLibObj./&SdmLibObj.
        , sesetl_useType=BASE
        , sesetl_validateLib=Y
         ) ;

%bldLibNm(sesetl_OMRSASLibraryObjName=&Solsystem_folder./&SdmLibObj./&SdmLibObj.,
          sesetl_useType=CON
          ) ;

/******************************************************************************************/
/* Convert the macro variable values for complex exchange rate types, currency and time   */
/* members into datasets.                                                                 */
/******************************************************************************************/

data &xrate_type_table   (keep=EXCHNG_RATE_TYPE_RK EXCHNG_RATE_TYPE_CD)
     &currency_table     (keep=CURRENCY_RK CURRENCY_CD)
     &time_member_table  (keep=TIME_MBR_SDM_ID TIME_MBR_CD) ;

   length CODE1 $64 ;
   length CODE2 $64 ;
   length EXCHNG_RATE_TYPE_RK 8 ;
   length RATE_TYPE_CD $64 ;
   length CURRENCY_RK 8 ;
   length CURRENCY_CD $64 ;

   do nXX = 1 to 9999999999 ;
     CODE1 = strip( kscan("&fmsetl_xrateTypeIds_SDM", nXX, "09"x) ) ;
     CODE2 = strip( kscan("&fmsetl_xrateTypeCodes", nXX, "09"x) ) ;
     if CODE1 eq "" then leave ;
     EXCHNG_RATE_TYPE_RK = input(CODE1, 32.) ;
     EXCHNG_RATE_TYPE_CD = CODE2 ;
     output &xrate_type_table ;
   end ;

   do nXX = 1 to 9999999999 ;
     CODE1 = strip( kscan("&fmsetl_currencyids_sdm", nXX, "09"x) ) ;
     CODE2 = strip( kscan("&fmsetl_currencycodes", nXX, "09"x) ) ;
     if CODE1 eq "" then leave ;
     CURRENCY_RK = input(CODE1, 32.) ;
     CURRENCY_CD = CODE2 ;
     output &currency_table ;
   end ;

   do nXX = 1 to 9999999999 ;
     CODE1 = strip( kscan("&fmsetl_timePeriodIds_SDM", nXX, "09"x) ) ;
     CODE2 = strip( kscan("&fmsetl_timePeriodCodes", nXX, "09"x) ) ;
     if CODE1 eq "" then leave ;
     TIME_MBR_SDM_ID = input(CODE1, 32.) ;
     TIME_MBR_CD = CODE2 ;
     output &time_member_table ;
   end ;

run ;
%_chkError;
%if &_tmpRC %then %goto exit;

/******************************************************************************************/
/*                 Find the DDS RK for the DDS Exchange Rate Set Code                     */
/******************************************************************************************/

proc sql noprint ;
    select distinct EXCHANGE_RATE_SET_RK into :_tmpXrateSetRK_DDS
        from _CONFORM.CURRENCY_EXCH_RATE_SET
        where EXCHANGE_RATE_SET_ID = "&fmsetl_exchgRateSetCode_DDS";
quit ;
%_chkError;
%if &_tmpRC %then %goto exit;

/******************************************************************************************/
/*                                                                                        */
/*        CREATE FORMATS NEEDED FOR BOTH SIMPLE AND COMPLEX EXCHANGE RATE IMPORT          */
/*                                                                                        */
/******************************************************************************************/

/********************************* FMT_CURRENCY *******************************************/

/* Check if _SDMLIB.SAS_DIM_TYPE exists */
%let _tmpDSN = _SDMLIB.SAS_DIM_TYPE;
%if not %sysfunc( exist(&_tmpDSN) ) %then %do;
   %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=11
           , msgPutlog=N);
   %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=2
           , msgTxt1=dimension_type, msgTxt2=&sesgmsg_msgReturn
           , msgPutlog=&putlog);
   %goto exit ;
%end;

%let dimension_type_rk = 0;
proc sql noprint;
  select DIM_TYPE_ID into :dimension_type_rk from &_tmpDSN
  where DIM_TYPE_CD = 'CURRENCY' and VALID_TO_DTTM  >= &fmsetl_processedDTTM_To;
quit;
%_chkError;
%if &_tmpRC %then %goto exit;

%if &dimension_type_rk = 0 %then %do;
   %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=3,
            msgTxt1=<CURRENCY>, msgPutlog=&putlog);
   %goto exit;
%end ;

/* Check if _SDMLIB.SAS_DIMENSION exists */
%let _tmpDSN = _SDMLIB.SAS_DIMENSION;
%if not %sysfunc( exist(&_tmpDSN) ) %then %do;
   %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=12
           , msgPutlog=N);
   %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=2
           , msgTxt1=dimension, msgTxt2=&sesgmsg_msgReturn
           , msgPutlog=&putlog);
   %goto exit ;
%end;

%let _tmpSortVar = INTERNAL_DIMENSION_ID;
%let _tmpWhere = where=(DIM_TYPE_ID = &dimension_type_rk and
                        VALID_TO_DTTM >= &fmsetl_processedDTTM_To);

data WORK._TMPCURRDIMS;
   set &_tmpDSN(&_tmpWhere);
   keep &_tmpSortVar;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

/* Check for the existence of active currency dimensions in the dimension table */
proc sql noprint;
   select count(*) INTO :_rec_cnt
   from WORK._TMPCURRDIMS
   ;
quit;
%_chkError;
%if &_tmpRC %then %goto exit ;

%if (&_rec_cnt < 1) %then %do;
  %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=4
          , msgTxt1=<CURRENCY>, msgPutlog=&putlog);
  %goto exit ;
%end;

proc sort data = WORK._TMPCURRDIMS nodupkey;
   by &_tmpSortVar;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

/* Check if _SDMLIB.SAS_MEMBER exists */
%let _tmpDSN = _SDMLIB.SAS_MEMBER;
%if not %sysfunc( exist(&_tmpDSN) ) %then %do;
   %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=13
           , msgPutlog=N);
   %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=2
           , msgTxt1=member, msgTxt2=&sesgmsg_msgReturn
           , msgPutlog=&putlog);
   %goto exit ;
%end;

%let _tmpWhere = where=(VALID_FROM_DTTM <= &fmsetl_processedDTTM_To and
                        VALID_TO_DTTM >= &fmsetl_processedDTTM_To);
data WORK._TMPMEMS;
   set &_tmpDSN(&_tmpWhere);
   drop VALID_FROM_DTTM VALID_TO_DTTM;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

/* Check for the existence of active records in the member table */
proc sql noprint;
   select count(*) INTO :_rec_cnt
   from WORK._TMPMEMS
   ;
quit;
%_chkError;
%if &_tmpRC %then %goto exit ;

%if (&_rec_cnt < 1) %then %do;
  %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=5
          , msgPutlog=&putlog);
  %goto exit ;
%end;

proc sort data = WORK._TMPMEMS;
   by &_tmpSortVar;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

%let _tmpFormat=FMT_CURRENCY;
%let _tmpStart=MEMBER_CD;
%let _tmpLabel=MEMBER_ID;

data work.&_tmpFormat;
   length START $ 64;
   length LABEL 8;
   retain FMTNAME "&_tmpFormat";
   retain TYPE "I";
   retain HLO "";
   merge WORK._TMPCURRDIMS (in=inA)
         WORK._TMPMEMS (in=inB);
      by &_tmpSortVar;
   START = strip(&_tmpStart);
   LABEL = &_tmpLabel;
   keep START LABEL FMTNAME TYPE HLO;
   if (inA and inB) then
      output;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

/* Check for the existence of active currency records in the member table */
proc sql noprint;
   select count(*) INTO :_rec_cnt
   from WORK.&_TMPFORMAT
   ;
quit;
%_chkError;
%if &_tmpRC %then %goto exit ;

%if (&_rec_cnt < 1) %then %do;
  %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=6
          , msgTxt1=<CURRENCY>, msgPutlog=&putlog);
  %goto exit ;
%end;

proc sort data = WORK.&_tmpformat nodupkey;
   by START;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

proc format cntlin=&_tmpFormat library=WORK;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

%if not %sysfunc( cexist(work.formats.&_tmpFormat..infmt) ) %then %do;
   %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=7,
            msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
   %goto exit;
%end;

/********************************* FMT_TIMEMBR ********************************************/

proc sql noprint ;
    create table WORK._DDSTMPTIME as
    select distinct TIME_PERIOD_RK as TIME_MBR_DDS_RK,
                    TIME_PERIOD_ID as TIME_MBR_CD
        from _CONFORM.TIME_PERIOD
        ;
quit ;
%_chkError;
%if &_tmpRC %then %goto exit;

proc sort in=&time_member_table ;
  by TIME_MBR_CD ;
run ;
%_chkError;
%if &_tmpRC %then %goto exit;

proc sort in=WORK._DDSTMPTIME;
  by TIME_MBR_CD ;
run ;
%_chkError;
%if &_tmpRC %then %goto exit;

data WORK._TIME_DDS_SDM WORK._ERRTMPTIME ;
  merge &time_member_table(in=SDM)
        WORK._DDSTMPTIME(in=DDS) ;
  by TIME_MBR_CD ;
  if SDM and DDS then do ;
    output WORK._TIME_DDS_SDM ;
  end ;
  if SDM and not DDS then do ;
    output WORK._ERRTMPTIME ;
  end ;
run ;
%_chkError;
%if &_tmpRC %then %goto exit;

%let _tmpFormat=FMT_TIMEMBR;
%let _tmpStart=TIME_MBR_DDS_RK;
%let _tmpLabel=TIME_MBR_SDM_ID;

data work.&_tmpFormat;
   length START 8;
   length LABEL 8;
   retain FMTNAME "&_tmpFormat";
   retain HLO "";
   set WORK._TIME_DDS_SDM end=nEnd;
   START = &_tmpStart;
   LABEL = &_tmpLabel;
   keep START LABEL FMTNAME HLO;
   output ;
   if nEnd then do ;
      START = .;
      LABEL = -1;
      HLO = "O";
      output;
   end ;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

proc sort data = WORK.&_tmpformat nodupkey;
   by START;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

proc format cntlin=&_tmpFormat library=WORK;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

%if not %sysfunc( cexist(work.formats.&_tmpFormat..format) ) %then %do;
   %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=7,
            msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
   %goto exit;
%end;

/********************************* FMT_XRATE_TYPE *******************************************/

%let _tmpDSN = _SDMLIB.SAS_DIM_TYPE;
%let xrate_type_int_rk = 0;
proc sql noprint;
  select INTERNAL_DIM_TYPE_ID into :xrate_type_int_rk from &_tmpDSN
  where DIM_TYPE_CD = 'XRATE_TYPE' and VALID_TO_DTTM  >= &fmsetl_processedDTTM_To;
quit;
%_chkError;
%if &_tmpRC %then %goto exit;

%if &xrate_type_int_rk = 0 %then %do;
   %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=3,
            msgTxt1=<XRATE_TYPE>, msgPutlog=&putlog);
   %goto exit;
%end ;

%let _tmpDSN = _SDMLIB.SAS_MEMBER;
%let _tmpFormat=FMT_XRATE_TYPE;
%let _tmpStart=MEMBER_CD;
%let _tmpLabel=MEMBER_ID;
%let _tmpWhere=where=(INTERNAL_DIM_TYPE_ID = &xrate_type_int_rk and
                      VALID_TO_DTTM >= &fmsetl_processedDTTM_To);

data work.&_tmpFormat;
   length START $ 64;
   length LABEL 8;
   retain FMTNAME "&_tmpFormat";
   retain TYPE "I";
   retain HLO "";
   set &_tmpDSN(&_tmpWhere);
   START = strip(&_tmpStart);
   LABEL = &_tmpLabel;
   keep START LABEL FMTNAME TYPE HLO;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

/* Check for the existence of active xrate_type records in the member table */
proc sql noprint;
   select count(*) INTO :_rec_cnt
   from WORK.&_tmpFormat
   ;
quit;
%_chkError;
%if &_tmpRC %then %goto exit ;

%if (&_rec_cnt < 1) %then %do;
  %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=6
          , msgTxt1=<XRATE_TYPE>, msgPutlog=&putlog);
  %goto exit ;
%end;

proc sort data = WORK.&_tmpFormat NODUPKEY;
   by START;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

proc format cntlin=&_tmpFormat library=WORK;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

%if not %sysfunc( cexist(work.formats.&_tmpFormat..infmt) ) %then %do;
   %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=7,
            msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
   %goto exit;
%end;

/******************************************************************************************/
/*                                                                                        */
/*                      EXTRACTION OF SIMPLE EXCHANGE RATES                               */
/*                                                                                        */
/******************************************************************************************/

/* Check if _CONFORM.CURRENCY_EXCH_RATE exists */
%let _tmpDSN = _CONFORM.CURRENCY_EXCH_RATE;
%if not %sysfunc( exist(&_tmpDSN) ) %then %do;
   %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=15
           , msgPutlog=N);
   %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=8
           , msgTxt1=currency_exch_rate, msgTxt2=&sesgmsg_msgReturn
           , msgPutlog=&putlog);
   %goto exit;
%end;

%let _tmpWhere = where=(PROCESSED_DTTM > &fmsetl_processedDTTM_From and
                        PROCESSED_DTTM <= &fmsetl_processedDTTM_To and
                        EXCHANGE_RATE_SET_RK = &_tmpXrateSetRK_DDS and
                        TO_CURRENCY_CD = "&fmsetl_baseCurrencyCode." and
                        EXCHANGE_RT > 0);

data WORK._TMPSRATEV/VIEW=WORK._TMPSRATEV;
   keep EXCHNG_RATE_TYPE_RK FROM_CURNC_MEMB_RK TO_CURNC_MEMB_RK TIME_MEMB_RK
        CONCAT_MEMBER_IDS RATE PROCESSED_DTTM;
   length EXCHNG_RATE_TYPE_RK 8;
   length FROM_CURNC_MEMB_RK 8;
   length TO_CURNC_MEMB_RK 8;
   length TIME_MEMB_RK 8;
   length CONCAT_MEMBER_IDS $255 ;
   length RATE 8;
   format RATE best32.;
   set &_tmpDSN(&_tmpWhere);
   EXCHNG_RATE_TYPE_RK = input(EXCHANGE_RATE_TYPE_CD, FMT_XRATE_TYPE.);
   FROM_CURNC_MEMB_RK = input(strip(FROM_CURRENCY_CD), FMT_CURRENCY.);
   TO_CURNC_MEMB_RK = &fmsetl_baseCurrencyId_SDM;
   CONCAT_MEMBER_IDS = "";
   RATE = EXCHANGE_RT;
   TIME_MEMB_RK = input(put(TIME_PERIOD_RK, FMT_TIMEMBR.), BEST.) ;
   if TIME_MEMB_RK ne -1 then do ;
      output ;
   end ;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

proc sort data=WORK._TMPSRATEV out=_TMPSRATESort;
   by FROM_CURNC_MEMB_RK;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

/* Check if cycle's currency table exists */
%let _tmpDSN = &currency_table;
%if not %sysfunc( exist(&_tmpDSN) ) %then %do;
   %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=16
           , msgPutlog=N);
   %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=10
          , msgTxt1=&_tmpDSN, msgTxt2=&sesgmsg_msgReturn
          , msgPutlog=&putlog);
   %goto exit;
%end;

data WORK._TMPCURRSUBSET;
   set &_tmpDSN;
   FROM_CURNC_MEMB_RK = CURRENCY_RK;
   keep FROM_CURNC_MEMB_RK;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

proc sort data=WORK._TMPCURRSUBSET;
   by FROM_CURNC_MEMB_RK;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

data WORK._TMPSRATE;
   merge WORK._TMPSRATESORT (in = inA)
         WORK._TMPCURRSUBSET (in = inB);
      by FROM_CURNC_MEMB_RK;
   if (inA) and (inB) then
      output;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

%let _tmpSortVar = EXCHNG_RATE_TYPE_RK;
proc sort data=WORK._TMPSRATE;
   by &_tmpSortVar;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

/* Check if cycle's exchange rate type table exists */
%let _tmpDSN = &xrate_type_table;
%if not %sysfunc( exist(&_tmpDSN) ) %then %do;
   %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=17
           , msgPutlog=N);
   %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=10
           , msgTxt1=&_tmpDSN, msgTxt2=&sesgmsg_msgReturn
           , msgPutlog=&putlog);
   %goto exit;
%end;

proc sort data=&_tmpDSN;
   by &_tmpSortVar;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

data WORK._TMPSRATE;
   merge WORK._TMPSRATE (in = inA)
         &_tmpDSN (in = inB);
      by &_tmpSortVar;
   drop EXCHNG_RATE_TYPE_CD;
   if (inA) and (inB) then
      output;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

proc sort data=WORK._TMPSRATE;
   BY DESCENDING PROCESSED_DTTM;
%_chkError;
%if &_tmpRC %then %goto exit;

proc sort data=WORK._TMPSRATE nodupkey;
   by EXCHNG_RATE_TYPE_RK FROM_CURNC_MEMB_RK TO_CURNC_MEMB_RK
      TIME_MEMB_RK;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

data WORK._TMPSRATE;
   set WORK._TMPSRATE;
   drop PROCESSED_DTTM;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

/******************************************************************************************/
/* Filter rates down by type.                                                             */
/******************************************************************************************/

%let _tmpSortVar = EXCHNG_RATE_TYPE_RK;
proc sort data=WORK._TMPSRATE;
   by &_tmpSortVar;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

/* Check if cycle's exchange rate type table exists */
%let _tmpDSN = WORK._XRATETYPES;
%if not %sysfunc( exist(&_tmpDSN) ) %then %do;
   %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=15
           , msgPutlog=N);
   %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=10
           , msgTxt1=&_tmpDSN, msgTxt2=&sesgmsg_msgReturn
           , msgPutlog=&putlog);
   %goto exit;
%end;

proc sort data=&_tmpDSN;
   by &_tmpSortVar;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

data WORK.ETL_SIMPRATE;
   merge WORK._TMPSRATE (in = inA)
         &_tmpDSN (in = inB);
      by &_tmpSortVar;
   if (inA) and (inB) then
      output;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

/******************************************************************************************/

proc sql noprint;
   connect to &sesetl_EngineType ( &sesetl_BestLogin &sesetl_ConnectionOptions );
   execute(create table &fmsetl_tempCreateSimpleTableNm (
           XRATE_IDX INT(11),
           XRATE_TYPE_MEMBER_ID int(11) NOT NULL default 0,
           FROM_CURRENCY_MEMBER_ID int(11) NOT NULL default 0,
           TO_CURRENCY_MEMBER_ID int(11) NOT NULL default 0,
           TIME_MEMBER_ID int(11) NOT NULL default 0,
           CONCAT_MEMBER_IDS varchar(255) NOT NULL default '',
           XRATE double NOT NULL default 0
/*
           XRATE_IDX INT(11),
           EXCHNG_RATE_TYPE_RK int(11) NOT NULL default 0,
           FROM_CURNC_MEMB_RK int(11) NOT NULL default 0,
           TO_CURNC_MEMB_RK int(11) NOT NULL default 0,
           TIME_MEMB_RK int(11) NOT NULL default 0,
           CONCAT_MEMBER_IDS varchar(255) NOT NULL default '',
           RATE double NOT NULL default 0
*/
           )ENGINE=MyISAM
           ) by &sesetl_EngineType ;
   disconnect from &sesetl_EngineType ;
quit;
%_chkError;
%if &_tmpRC %then %goto exit;

proc sql noprint;
   connect to &sesetl_EngineType ( &sesetl_BestLogin &sesetl_ConnectionOptions );
   execute(create table &fmsetl_tempCreateComplexTableNm (
           XRATE_IDX INT(11),
           XRATE_TYPE_MEMBER_ID int(11) NOT NULL default 0,
           FROM_CURRENCY_MEMBER_ID int(11) NOT NULL default 0,
           TO_CURRENCY_MEMBER_ID int(11) NOT NULL default 0,
           TIME_MEMBER_ID int(11) NOT NULL default 0,
           CONCAT_MEMBER_IDS varchar(255) NOT NULL default '',
           XRATE double NOT NULL default 0
/*
           XRATE_IDX INT(11),
           EXCHNG_RATE_TYPE_RK int(11) NOT NULL default 0,
           FROM_CURNC_MEMB_RK int(11) NOT NULL default 0,
           TO_CURNC_MEMB_RK int(11) NOT NULL default 0,
           TIME_MEMB_RK int(11) NOT NULL default 0,
           CONCAT_MEMBER_IDS varchar(255) NOT NULL default '',
           RATE double NOT NULL default 0
*/
           )ENGINE=MyISAM
           ) by &sesetl_EngineType ;
   disconnect from &sesetl_EngineType ;
quit;
%_chkError;
%if &_tmpRC %then %goto exit;

data WORK._TMPSRATECONCAT(keep=XRATE_IDX EXCHNG_RATE_TYPE_RK FROM_CURNC_MEMB_RK TO_CURNC_MEMB_RK
                              TIME_MEMB_RK CONCAT_MEMBER_IDS RATE
                         drop=IDXCNT) ;
  length XRATE_IDX 8 ;
  length EXCHNG_RATE_TYPE_RK 8 ;
  length FROM_CURNC_MEMB_RK 8 ;
  length TO_CURNC_MEMB_RK 8 ;
  length TIME_MEMB_RK 8 ;
  length CONCAT_MEMBER_IDS $255 ;
  length RATE 8 ;
  format RATE best32.;
  retain IDXCNT 0 ;
  set WORK.ETL_SIMPRATE;
  IDXCNT = IDXCNT + 1 ;
  XRATE_IDX = IDXCNT ;
run ;
%_chkError;
%if &_tmpRC %then %goto exit;

/*******************************************************************************/

%sesFMS_uploadDataToSDM(cSrcNm=WORK._TMPSRATECONCAT, cTrgNm=&fmsetl_tempCreateSimpleTableNm);

/******************************************************************************************/
/*                                                                                        */
/*                      EXTRACTION OF COMPLEX EXCHANGE RATES                              */
/*                                                                                        */
/******************************************************************************************/

%if %symexist(fmsetl_dimensionTypeCodes) %then %do;
   %if %nrbquote(&fmsetl_dimensionTypeCodes) = %str() %then %do;
      %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=18,
               msgTxt1=<fmsetl_dimensionTypeCodes : &fmsetl_dimensionTypeCodes>, msgPutlog=&putlog);
      %goto exit;
   %end;
   %if %symexist(fmsetl_dimensionCodes) %then %do;
      %if %nrbquote(&fmsetl_dimensionCodes) = %str() %then %do;
         %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=18,
                  msgTxt1=<fmsetl_dimensionCodes : &fmsetl_dimensionCodes>, msgPutlog=&putlog);
         %goto exit;
      %end;
   %end;
   %else %do;
      %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=21,
               msgTxt1=<fmsetl_dimensionCodes>, msgPutlog=&putlog);
      %goto exit;
   %end;
%end;
%else %do;
   data WORK.ETL_XRATE;
      set WORK.ETL_SIMPRATE;
   run;
   %goto complex_exit;
%end;

/******************************************************************************************/
/* Convert the macro variable values for dimension types and dimensions into datasets.    */
/******************************************************************************************/

data &dim_type_table     (keep=CODE)
     &dimension_table    (keep=CODE) ;

   length CODE $64 ;

   do nXX = 1 to 9999999999 ;
     CODE = strip( kscan("&fmsetl_dimensionTypeCodes", nXX, "09"x) ) ;
     if CODE eq "" then leave ;
     output &dim_type_table ;
   end ;

   do nXX = 1 to 9999999999 ;
     CODE = strip( kscan("&fmsetl_dimensionCodes", nXX, "09"x) ) ;
     if CODE eq "" then leave ;
     output &dimension_table ;
   end ;

run ;
%_chkError;
%if &_tmpRC %then %goto exit;

/******************************************************************************************/
/* This code loads the dimension type codes from the work area data set and places them   */
/* into a concatenated list of dimension types that will later be used to concatenate the */
/* the members belonging to a complex exchange rate.                                      */
/******************************************************************************************/
%macro mbrConcat ;

   %global fmsetl_concatMbrStmt ;
   %let fmsetl_concatMbrStmt= ;

   data _null_ ;
     length CONCATSTMT $32767 ;
     retain CONCATSTMT ;
     set &dim_type_table end=nEnd ;
     if _n_ = 1 then do ;
       CONCATSTMT = strip(CONCATSTMT) ||
                    "strip(put(" || quote(strip(CODE)) || "n, 32.))"  ;
     end ;
     else do ;
       CONCATSTMT = strip(CONCATSTMT) || " || ',' || " ||
                    "strip(put(" || quote(strip(CODE)) || "n, 32.))"  ;
     end ;
     if nEnd then do ;
       call symput("fmsetl_concatMbrStmt", CONCATSTMT) ;
     end ;
   run ;

%mend ;

%mbrConcat ;

/******************************************************************************************/
/* This portion of code will take the _CONFORM.CURRENCY_COMPLEX_EXCH_RATE table and       */
/* extract the rates based on dimension type values.  Also, all of the member rks for the */
/* members will be converted to codes.  No _CONFORM to SDM conversion will be performed   */
/* at this time.                                                                          */
/******************************************************************************************/

/* Extract the dimension types from dimension type work area data set and place them into */
/* a delimited list: ACCOUNT|INTORG|PRODUCT,... */

%macro dtypeConcat ;
   %global fmsetl_concatDtypeStmt ;
   %let fmsetl_concatDtypeStmt= ;

   %global fmsetl_dTypeCheck ;
   %let fmsetl_dTypeCheck= ;

   data _null_ ;
     length CONCATSTMT $32767 ;
     length CONCATDTYPE $32767 ;
     retain CONCATSTMT ;
     retain CONCATDTYPE ;

     set &dim_type_table end=nEnd ;
     if _n_ = 1 then do ;
       CONCATSTMT = strip(CODE);
       CONCATDTYPE = quote(strip(CODE)) ;
     end ;
     else do ;
       CONCATSTMT = strip(CONCATSTMT) || "|" || strip(CODE) ;
       CONCATDTYPE = strip(CONCATDTYPE) || "," || quote(strip(CODE)) ;
     end ;
     if nEnd then do ;
       call symput("fmsetl_concatDtypeStmt", CONCATSTMT) ;
       call symput("fmsetl_dTypeCheck", CONCATDTYPE) ;
     end ;
   run ;

%mend dtypeConcat;

%dtypeConcat ;

/******************************************************************************************/
/* Create RK ==> Code formats for all of the dimension types in fmsetl_concatDtypeStmt.   */
/******************************************************************************************/
proc sql noprint;
   select count(*) INTO :_dtype_cnt
   from &dim_type_table
   ;
quit;
%_chkError;
%if &_tmpRC %then %goto exit ;

%macro fmtdtype/parmbuff;

%let _tmpDSN=_CONFORM.DIMENSION_TYPE;
%let fmt=fmt_dds_dtype;
%let x=_x;
%let dtypeString=%sysfunc(ksubstr(&syspbuff, 2), %eval(%sysfunc(klength(&syspbuff)) -2 ));

data work.fmt_dtype_fmtname;
   length FMTNAME $32;
   length TYPE $1;
   length START $32;
   length LABEL $32;
   stop;
run;

%do i = 1 %to &_dtype_cnt;
   %let dimtype=%sysfunc(kscan(%bquote(&dtypeString),&i,|));
   %let _tmpWhere = where=(DIMENSION_TYPE_CD = "&dimtype" and
                           VALID_FROM_DTTM <= &fmsetl_processedDTTM_To and
                           VALID_TO_DTTM >= &fmsetl_processedDTTM_To);
   data _null_;
      set _CONFORM.DIMENSION_TYPE(&_tmpWhere);
      call symput('prefix', %unquote(strip(TABLE_NM)));
   run;
   %_chkError;
   %if &_tmpRC %then %goto fmtdtype_exit;

   %if &dimtype ne ITEMCAT %then %do;
      %if (&dimtype eq ACCOUNT) or (&dimtype eq ANALYSIS) or (&dimtype eq COSTCTR) or
          (&dimtype eq EXTORG) or (&dimtype eq INTORG) or (&dimtype eq PROFITCTR) or
          (&dimtype eq TIME) %then %do;
         %let dtypeid=&prefix._ID;
         %let dtypeRK=&prefix._RK;
      %end;
      %else %do;
         %let dtypeid=MEMBER_ID;
         %let dtypeRK=MEMBER_RK;
      %end;

      %let _tmpWhere = where=(VALID_FROM_DTTM <= &fmsetl_processedDTTM_To and
                           VALID_TO_DTTM >= &fmsetl_processedDTTM_To);
      %let _tmpFormat=&fmt&i&x;

      data work._tmp_dtype_fmtname;
         length FMTNAME $32;
         length TYPE $1;
         length START $32;
         length LABEL $32;
         FMTNAME="FMT_DTYPE_FMTNAME";
         TYPE="C";
         START="&dimtype";
         LABEL="&_tmpFormat";
         output;
      run;
      %_chkError;
      %if &_tmpRC %then %goto fmtdtype_exit;

      proc append base=work.fmt_dtype_fmtname data=work._tmp_dtype_fmtname;
      run;
      %_chkError;
      %if &_tmpRC %then %goto fmtdtype_exit;

      data work.&_tmpFormat;
         length START 8;
         length LABEL $32;
         retain FMTNAME "&_tmpFormat";
         retain TYPE "N";
         retain HLO "";
         set _CONFORM.&prefix(&_tmpWhere) end=last;
         START = &dtypeRK;
         LABEL = &dtypeid;
         output;
         if (last) then do;
            START = .;
            LABEL = '';
            output;
            HLO = 'O';
            START = .;
            LABEL = '*** ERROR ***';
            output;
         end;
         keep START LABEL FMTNAME TYPE HLO;
      run;
      %_chkError;
      %if &_tmpRC %then %goto fmtdtype_exit;

      proc format cntlin=&_tmpFormat library=WORK;
      run;
      %_chkError;
      %if &_tmpRC %then %goto fmtdtype_exit;

      %let format_missing = 0;

      %if not %sysfunc( cexist(work.formats.&_tmpFormat..format) ) %then %do;
         %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=7,
                  msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
         %let format_missing = 1;
         %goto fmtdtype_exit;
      %end;
   %end;
%end;

proc format cntlin=fmt_dtype_fmtname library=WORK;
run;
%_chkError;
%if &_tmpRC %then %goto fmtdtype_exit;

%let format_missing = 0;

%if not %sysfunc( cexist(work.formats.fmt_dtype_fmtname.formatc) ) %then %do;
   %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=7,
            msgTxt1=<FMT_DTYPE_FMTNAME>, msgTxt2=&syserr, msgPutlog=&putlog);
   %let format_missing = 1;
   %goto fmtdtype_exit;
%end;

%fmtdtype_exit:

%mend fmtdtype;

%fmtdtype(&fmsetl_concatDtypeStmt);

%if &_tmpRC or &format_missing %then
    %goto exit;

/**********************************************************************************************/
/* FMT_COL_TO_DTYPE: Derives the dimension type from the column name in the                   */
/* CURRENCY_COMPLEX_EXCH_RATE table                                                           */
/**********************************************************************************************/

%let _tmpDSN=_CONFORM.DIMENSION_TYPE;
%let _tmpFormat=FMT_COL_TO_DTYPE;
%let _tmpStart=MISC_COLUMN_NM;
%let _tmpLabel=DIMENSION_TYPE_CD;

data WORK.&_tmpFormat;
   length START $ 50;
   length LABEL $ 32;
   retain FMTNAME "&_tmpFormat";
   retain TYPE "C";
   retain HLO "";
   set &_tmpDSN;
   if DIMENSION_TYPE_CD = "CURRENCY" then do;
      START = "FROM_CURRENCY_CD";
      LABEL = "CURRENCY";
      output;
      START = "TO_CURRENCY_CD";
      LABEL = "CURRENCY";
      output;
   end;
   else do;
      START = &_tmpStart;
      LABEL = &_tmpLabel;
      output;
   end;
   keep START LABEL FMTNAME TYPE HLO;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

proc sort data=&_tmpFormat nodupkeys;
  by start;
run;
proc format cntlin=&_tmpFormat library=WORK;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

%if not %sysfunc( cexist(work.formats.&_tmpFormat..formatc) ) %then %do;
   %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=7,
            msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
   %goto exit;
%end;

/******************************************************************************************/
/* Create rateCols macro that contains list of all columns existing in the                */
/* CURRENCY_COMPLEX_EXCH_RATE table of the dds.                                           */
/******************************************************************************************/

proc sql noprint;
  create table work.xrate_cols as
  select name
  from dictionary.columns
  where upcase(libname) = "_CONFORM" and upcase(memname) = "CURRENCY_COMPLEX_EXCH_RATE"
  ;
quit;
%_chkError;
%if &_tmpRC %then %goto exit;

proc sql noprint;
   select count(*) INTO :_col_cnt
   from work.xrate_cols
   ;
quit;
%_chkError;
%if &_tmpRC %then %goto exit;

%macro xrateCols ;

   data _null_ ;
     set work.xrate_cols end=nEnd ;
     if _n_ = 1 then
       call symput("rateCols", name);
     else
       call symput("rateCols", strip(symget("rateCols")) || "," || name);
   run ;

%mend xrateCols;

%xrateCols;

/******************************************************************************************/
/* Filter out the rates based on exchange rate set and base currency.                     */
/******************************************************************************************/

%let _tmpDSN = _CONFORM.CURRENCY_COMPLEX_EXCH_RATE;
%if not %sysfunc( exist(&_tmpDSN) ) %then %do;
   %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=15
          ,  msgPutlog=N);
   %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=8
          , msgTxt1=&_tmpDSN, msgTxt2=&sesgmsg_msgReturn
          , msgPutlog=&putlog);
   %goto exit;
%end ;

%macro filterRates/parmbuff;

%global if_condition ;
%let if_condition= ;

%let _tmpDSN = _CONFORM.CURRENCY_COMPLEX_EXCH_RATE;
%let _tmpWhere = where=(PROCESSED_DTTM > &fmsetl_processedDTTM_From and
                        PROCESSED_DTTM <= &fmsetl_processedDTTM_To and
                        EXCHANGE_RATE_SET_RK = &_tmpXrateSetRK_DDS and
                        TO_CURRENCY_CD = "&fmsetl_baseCurrencyCode." and
                        EXCHANGE_RT > 0);

%let cd=_CD;

data WORK._TMPCRATE ;
   length no_output 8;
   set &_tmpDSN(&_tmpWhere);
   no_output = 0;
   %let colString=%sysfunc(ksubstr(&syspbuff, 2), %eval(%sysfunc(klength(&syspbuff)) -2 ));
   %do i = 1 %to &_col_cnt;
      %let colName=%sysfunc(kscan(%bquote(&colString),&i));
      %if &colName ne EXCHANGE_RATE_SET_RK and &colName ne EXCHANGE_RATE_TYPE_CD and
          &colName ne EXCHANGE_RATE_SOURCE_CD and &colName ne FROM_CURRENCY_CD and
          &colname ne TO_CURRENCY_CD and &colname ne TIME_PERIOD_RK and
          &colName ne EXCHANGE_RT and &colName ne EFFECTIVE_FROM_DT and
          &colName ne EFFECTIVE_TO_DT and &colName ne PROCESSED_DTTM %then %do;
         %let dimtype=%sysfunc(putc(&colName, FMT_COL_TO_DTYPE.));
         %let _tmp_fmtname=%sysfunc(putc(&dimtype, FMT_DTYPE_FMTNAME.));
         %let dimtypecd=%sysfunc(strip(&dimtype))&cd;
         format &dimtypecd $char32.;
         %do n = 1 %to &_dtype_cnt;
            %let dtype_temp=%sysfunc(kscan(%bquote(&fmsetl_dTypeCheck),&n));
            %if &n = 1 %then
               %let if_condition = "&dimtype" = &dtype_temp ;
            %else
               %let if_condition = &if_condition or "&dimtype" = &dtype_temp;
         %end;
         %if &if_condition %then %do;
            %if "&dimtype" = "ITEMCAT" %then %do;
               &dimtypecd = &colName;
            %end;
            %else %do;
               &dimtypecd = put(&colName., &_tmp_fmtname..);
            %end;
         %end;
         %else %do;
            if "&dimtype" = "ITEMCAT" then do;
               if &colName ne "" then do;
                  no_output = 1;
               end;
            end;
            else if &colName ne . then do;
               no_output = 1;
            end;
            else do;
               &dimtypecd = "";
            end;
         %end;
         keep &dimtypecd ;
      %end;
   %end;
   keep EXCHANGE_RATE_SET_RK EXCHANGE_RATE_TYPE_CD EXCHANGE_RATE_SOURCE_CD FROM_CURRENCY_CD
        TO_CURRENCY_CD TIME_PERIOD_RK EXCHANGE_RT EFFECTIVE_FROM_DT EFFECTIVE_TO_DT
        PROCESSED_DTTM;
   if no_output = 0 then
      output;
run;
%_chkError;

%mend filterRates;

%filterRates(&rateCols);

%_chkError;
%if &_tmpRC %then %goto exit;

/* Check if complex exchange rate's dimension table exists */
%let _tmpDSN = &dimension_table;
%if not %sysfunc( exist(&_tmpDSN) ) %then %do;
   %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=19
           , msgPutlog=N);
   %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=10
          , msgTxt1=&_tmpDSN, msgTxt2=&sesgmsg_msgReturn
          , msgPutlog=&putlog);
   %goto exit;
%end;

proc sort data=&_tmpDSN;
   by CODE;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

/******************************************************************************************/
/* build lists of the complex exchange rate dimension types in macro vars                 */
/*                                                                                        */
/* Name: xrdtypes & dtype_ids                                                             */
/* Contents: list of dimension types codes and ids                                        */
/*                                                                                        */
/* Name: dimcnt                                                                           */
/* Contents: number of items in above lists                                               */
/******************************************************************************************/

%let _tmpSortVar = DIMENSION_CD;
data WORK._TMPALLDIMS;
   set &dimension_table;
   &_tmpSortVar = CODE;
   keep &_tmpSortVar;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

proc sort data = WORK._TMPALLDIMS;
   by &_tmpSortVar;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

%let _tmpWhere = where=(VALID_FROM_DTTM <= &fmsetl_processedDTTM_To and
                        VALID_TO_DTTM >= &fmsetl_processedDTTM_To);

data WORK._TMPSDMDIM;
   set _SDMLIB.SAS_DIMENSION(&_tmpWhere);
run;
%_chkError;
%if &_tmpRC %then %goto exit;

proc sort data = WORK._TMPSDMDIM;
   by &_tmpSortVar;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

data WORK._TMPALLDIMS
     WORK._TMPMISSINGDIMS;
   merge WORK._TMPALLDIMS (in = inA)
         WORK._TMPSDMDIM (in = inB);
      by &_tmpSortVar;
   keep DIMENSION_CD INTERNAL_DIMENSION_ID DIM_TYPE_ID;
   if (inA and ^inB) then
      output WORK._TMPMISSINGDIMS;
   else if (inA) then
      output WORK._TMPALLDIMS;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

proc sql noprint;
   select count(*) INTO :missingCnt FROM WORK._TMPMISSINGDIMS;
quit;
%_chkError;
%if &_tmpRC %then %goto exit;

%if (&missingCnt > 0) %then %do;
   data _null_;
      set WORK._TMPMISSINGDIMS (obs=1);
      call symput("missing_dim", strip(DIMENSION_CD));
   run;
   %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=20
           , msgTxt1=<&missing_dim>, msgPutlog=&putlog);
   %goto exit ;
%end;

%let _tmpSortVar = DIM_TYPE_ID;
proc sort data = WORK._TMPALLDIMS;
   by &_tmpSortVar;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

%let _tmpWhere = where=(VALID_FROM_DTTM <= &fmsetl_processedDTTM_To and
                        VALID_TO_DTTM >= &fmsetl_processedDTTM_To);

data WORK._TMPSDMDIMTYPE;
   set _SDMLIB.SAS_DIM_TYPE (&_tmpWhere);
run;
%_chkError;
%if &_tmpRC %then %goto exit;

proc sort data = WORK._TMPSDMDIMTYPE;
   by &_tmpSortVar;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

data WORK._TMPALLDIMS;
   merge WORK._TMPALLDIMS (in = inA)
         WORK._TMPSDMDIMTYPE;
      by &_tmpSortVar;
   keep DIMENSION_CD INTERNAL_DIMENSION_ID DIM_TYPE_ID DIM_TYPE_CD;
   if (inA) then
      output;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

proc sql noprint;
   select count(*) INTO :dimcnt FROM WORK._TMPALLDIMS;
quit;
%_chkError;
%if &_tmpRC %then %goto exit;

data _null_;
   set WORK._TMPALLDIMS;
   if (_n_ = 1) then do;
      call symput("xrdtypes",DIM_TYPE_CD);
      call symput("dtype_ids",DIM_TYPE_ID);
   end;
   else do;
      call symput("xrdtypes",strip(symget("xrdtypes"))||"|"||DIM_TYPE_CD);
      call symput("dtype_ids",strip(symget("dtype_ids"))||","||DIM_TYPE_ID);
   end;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

/******************************************************************************************/
/* Create formats needed for complex exchange rates                                       */
/******************************************************************************************/

/* FMT_TO_DIM */
%let _tmpDSN=WORK._TMPALLDIMS;
%let _tmpFormat=FMT_TO_DIM;
%let _tmpStart=DIM_TYPE_CD;
%let _tmpLabel=DIMENSION_CD;

data WORK.&_tmpFormat;
   length START $ 64;
   length LABEL $ 32;
   retain FMTNAME "&_tmpFormat";
   retain TYPE "C";
   retain HLO "";
   set &_tmpDSN;
   START = &_tmpStart;
   LABEL = &_tmpLabel;
   keep START LABEL FMTNAME TYPE HLO;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

proc format cntlin=&_tmpFormat library=WORK;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

%if not %sysfunc( cexist(work.formats.&_tmpFormat..formatc) ) %then %do;
   %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=7,
            msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
   %goto exit;
%end;

/* FMT_DIMS */
%let _tmpDSN=WORK._TMPALLDIMS;
%let _tmpFormat=FMT_DIMS;
%let _tmpStart=DIM_TYPE_ID;
%let _tmpLabel=INTERNAL_DIMENSION_ID;
data work.&_tmpFormat;
   length START 8;
   length LABEL 8;
   retain FMTNAME "&_tmpFormat";
   retain TYPE "I";
   retain HLO "";
   set &_tmpDSN;
   START = &_tmpStart;
   LABEL = &_tmpLabel;
   keep START LABEL FMTNAME TYPE HLO;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

proc format cntlin=&_tmpFormat library=WORK;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

%if not %sysfunc( cexist(work.formats.&_tmpFormat..infmt) ) %then %do;
   %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=7,
            msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
   %goto exit;
%end;

/* FMT_DTYPE_RK */
%let _tmpDSN=WORK._TMPALLDIMS;
%let _tmpFormat=FMT_DTYPE_RK;
%let _tmpStart=DIMENSION_CD;
%let _tmpLabel=DIM_TYPE_ID;
data work.&_tmpFormat;
   length START $ 64;
   length LABEL 8;
   retain FMTNAME "&_tmpFormat";
   retain TYPE "I";
   retain HLO "";
   set &_tmpDSN;
   START = &_tmpStart;
   LABEL = &_tmpLabel;
   keep START LABEL FMTNAME TYPE HLO;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

proc format cntlin=&_tmpFormat library=WORK;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

%if not %sysfunc( cexist(work.formats.&_tmpFormat..infmt) ) %then %do;
   %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=7,
            msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
   %goto exit;
%end;

/* FMT_MEMTYPE -- format to be used for data validation */
%let _tmpFormat=FMT_MEMTYPE;
%let _tmpStart=MEMBER_ID;
%let _tmpLabel=DIM_TYPE_CD;
%let _tmpWhere=where=(VALID_FROM_DTTM <= &fmsetl_processedDTTM_To and
                      VALID_TO_DTTM >= &fmsetl_processedDTTM_To);

%let _tmpDSN=WORK._TMPALLDIMS;
%let _tmpSortVar=INTERNAL_DIMENSION_ID;
proc sort data = &_tmpDSN nodupkey;
   by &_tmpSortVar;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

%let _tmpDSN=_SDMLIB.SAS_MEMBER;

data WORK._TMPMEMBER;
   set &_tmpDSN(&_tmpWhere);
run;
%_chkError;
%if &_tmpRC %then %goto exit;

proc sort data = WORK._TMPMEMBER;
   by &_tmpSortVar;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

data work.&_tmpFormat;
   length START 8;
   length LABEL $ 32;
   retain FMTNAME "&_tmpFormat";
   retain HLO "";
   retain TYPE "N";
   merge WORK._TMPALLDIMS (in=inA)
         WORK._TMPMEMBER (in=inB);
      by &_tmpSortVar;
   START = &_tmpStart;
   LABEL = &_tmpLabel;
   keep START LABEL FMTNAME TYPE HLO;
   if (inA and inB) and (DIM_TYPE_CD ^= 'TRADER') then
      output;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

data WORK.&_tmpFormat;
   set WORK.&_tmpFormat end=last;
   output;
   if (last) then do;
      HLO = 'O';
      START = .;
      LABEL = '*** ERROR ***';
      FMTNAME = "&_tmpFormat";
      TYPE = "N";
      output;
   end;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

proc format cntlin=&_tmpFormat library=WORK;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

%if not %sysfunc( cexist(work.formats.&_tmpFormat..format) ) %then %do;
   %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=7,
            msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
   %goto exit;
%end;

/* Create the formats for all of the dimensions */

%macro fmtdims/parmbuff;

%let _tmpDSN=_SDMLIB.SAS_MEMBER;
%let _tmpStart=MEMBER_CD;
%let _tmpLabel=MEMBER_ID;

%let fmt=fmt_dtype;
%let x=_x;

%do i = 1 %to &dimcnt;
   %let dtype_rk=%sysfunc(kscan(&syspbuff,&i,'(,)'));
   %let _tmpWhere=where=(INTERNAL_DIMENSION_ID = input(symget("dtype_rk"), FMT_DIMS.) and
                         VALID_FROM_DTTM <= &fmsetl_processedDTTM_To and
                         VALID_TO_DTTM >= &fmsetl_processedDTTM_To);
   proc sort data = &_tmpDSN(&_tmpWhere
                    keep= &_tmpStart &_tmpLabel INTERNAL_DIMENSION_ID VALID_FROM_DTTM VALID_TO_DTTM)
              out = work._tmpSort(drop= INTERNAL_DIMENSION_ID VALID_FROM_DTTM VALID_TO_DTTM);
      by &_tmpStart;
   run;
   %_chkError;
   %if &_tmpRC %then %goto fmtdims_exit;

   %let _tmpFormat=&fmt&dtype_rk&x;

   data work.&_tmpFormat;
      length START $ 64;
      length LABEL 8;
      retain FMTNAME "&_tmpFormat";
      retain TYPE "I";
      retain HLO "";
      set WORK._TMPSORT end=last;
      START = strip(&_tmpStart);
      LABEL = &_tmpLabel;
      output;
      if (last) then do;
         START = '';
         LABEL = .;
         output;
         HLO = 'O';
         START = .;
         LABEL = .;
         output;
      end;
      keep START LABEL FMTNAME TYPE HLO;
   run;
   %_chkError;
   %if &_tmpRC %then %goto fmtdims_exit;

   proc format cntlin=&_tmpFormat library=WORK;
   run;
   %_chkError;
   %if &_tmpRC %then %goto fmtdims_exit;

   %let format_missing = 0;
   %if not %sysfunc( cexist(work.formats.&_tmpFormat..infmt) ) %then %do;
      %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=7,
               msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
      %let format_missing = 1;
      %goto fmtdims_exit;
   %end;

%end;

%fmtdims_exit:

%mend fmtdims;

%fmtdims(&dtype_ids);

%if &_tmpRC or &format_missing %then
    %goto exit;

/******************************************************************************************/
/* Convert dds member codes to sassdm member rks and concatenate these members together.  */
/* Also, validate the members based on dimension type.                                    */
/******************************************************************************************/

%macro ratework/parmbuff;

%let err=_ERROR;
%let fmt=fmt_dtype;
%let x=_x.;
%let cd=_CD;
%let code=_CODE;

%let _tmpDSN = WORK._TMPCRATE;

data WORK._TMPCRATE (keep=EXCHNG_RATE_TYPE_RK FROM_CURNC_MEMB_RK TO_CURNC_MEMB_RK
                     TIME_MEMB_RK CONCAT_MEMBER_IDS RATE)
     WORK.BADRECS /* (keep=EXCHNG_RATE_TYPE_RK FROM_CURNC_MEMB_RK TO_CURNC_MEMB_RK
                   TIME_MEMB_RK CONCAT_MEMBER_IDS RATE) */
     ;
   length EXCHNG_RATE_TYPE_RK 8;
   length FROM_CURNC_MEMB_RK 8;
   length TO_CURNC_MEMB_RK 8;
   length TIME_MEMB_RK 8;
   length RATE 8;
   format RATE best32.;
   length BAD_RECORD 8;
   length MEM_DIMTYPE $ 8;
   length DIMTYPE_HOLD $ 8;
   length CONCAT_MEMBER_IDS $ 255;
   length no_output 8;
   set &_tmpDSN;
   EXCHNG_RATE_TYPE_RK = input(EXCHANGE_RATE_TYPE_CD, FMT_XRATE_TYPE.) ;
   FROM_CURNC_MEMB_RK = input(strip(FROM_CURRENCY_CD), FMT_CURRENCY.);
   TO_CURNC_MEMB_RK = &fmsetl_baseCurrencyId_SDM;
   RATE = EXCHANGE_RT ;
   TIME_MEMB_RK = input(put(TIME_PERIOD_RK, FMT_TIMEMBR.), BEST.) ;
   BAD_RECORD = 0;
   /* Validate the members of the dimension types existing in the complex exchange rate table */
   %let dtypeString=%sysfunc(ksubstr(&syspbuff, 2), %eval(%sysfunc(klength(&syspbuff)) -2 ));
   no_output = 0;
   if TIME_MEMB_RK not in (&fmsetl_timePeriodIds_SDM) then
     no_output = 1 ;
   if FROM_CURNC_MEMB_RK not in (&fmsetl_currencyids_sdm) then
     no_output = 1 ;
   %do i = 1 %to &dimcnt;
       %let dimtype=%sysfunc(kscan(%bquote(&dtypeString),&i,|));
       %let dimname=%sysfunc(putc(&dimtype, $FMT_TO_DIM.));
       %let dimtype_rk=%sysfunc(inputn(&dimname, FMT_DTYPE_RK.));
       /* %let dimtype1=%sysfunc(putc(%sysfunc(upcase(%sysfunc(strip(&dimtype)))), $DTPCOL.)); */
       %let dimtypecd=%sysfunc(strip(&dimtype))&cd;
       %let dimtypecode=%sysfunc(strip(&dimtype))&code;
       %let dimtypeerr=%sysfunc(strip(&dimtype))&err;
       DIMTYPE_HOLD = "&dimtype";
       "&dimtype"n=input(strip("&dimtypecd"n), &fmt&dimtype_rk.&x);
       checkId = "&dimtype"n ;
       if (checkID eq .) then
         "&dimtype"n = 0 ;
       MEM_DIMTYPE = strip(put(checkId, FMT_MEMTYPE.));
       if checkID ne . then do;
          if (DIMTYPE_HOLD = 'TRADER') then do;
             if (MEM_DIMTYPE = 'INTORG') then do;
                "&dimtypeerr"n = 0;
             end;
             else do;
                bad_record = 1;
                "&dimtypeerr"n = 1;
             end;
          end;
          else do;
             if (MEM_DIMTYPE = DIMTYPE_HOLD) then do;
                "&dimtypeerr"n = 0;
             end;
             else do;
                BAD_RECORD = 1;
                "&dimtypeerr"n = 1;
             end;
          end;
       end;
       else if &dimtypecd ne '' then
          no_output = 1;
       "&dimtypecode"n = "&dimtypecd"n;
   %end;

   if not no_output then do;
      if (BAD_RECORD = 1) then do;
/*         keep EXCHNG_RATE_TYPE_RK FROM_CURNC_MEMB_RK TO_CURNC_MEMB_RK TIME_MEMB_RK
              CONCAT_MEMBER_IDS RATE */
              %do i = 1 %to &dimcnt;
                  %let dimtype=%sysfunc(kscan(%bquote(&dtypeString),&i,|));
                  /* %let dimtype1=%sysfunc(putc(%sysfunc(upcase(%sysfunc(strip(&dimtype)))), $DTPCOL.)); */
                  %let dimtypecode=%sysfunc(strip(&dimtype))&code;
                  %let dimtypeerr=%sysfunc(strip(&dimtype))&err;
/*                  "&dimtype"n "&dimtypecode"n "&dimtypeerr"n */
              %end;
              ;

         output WORK.BADRECS;
      end;
      else do;
/*         keep EXCHNG_RATE_TYPE_RK FROM_CURNC_MEMB_RK TO_CURNC_MEMB_RK TIME_MEMB_RK
              CONCAT_MEMBER_IDS RATE */
              %do i = 1 %to &dimcnt;
                  %let dimtype=%sysfunc(kscan(%bquote(&dtypeString),&i,|));
                  /* %let dimtype1=%sysfunc(putc(%sysfunc(upcase(%sysfunc(strip(&dimtype)))), $DTPCOL.)); */
                  %let dimtypecode=%sysfunc(strip(&dimtype))&code;
/*                  "&dimtype"n "&dimtypecode"n */
              %end;
              ;
         CONCAT_MEMBER_IDS = &fmsetl_concatMbrStmt ;
         output WORK._TMPCRATE;
      end;
   end;
run;
%_chkError;
%if &_tmpRC %then %goto ratework_exit;

/******************************************************************************************/
/* Filter rates down by type.                                                             */
/******************************************************************************************/

%let _tmpSortVar = EXCHNG_RATE_TYPE_RK;
proc sort data=WORK._TMPCRATE;
   by &_tmpSortVar;
run;
%_chkError;
%if &_tmpRC %then %goto ratework_exit;

/* Check if cycle's exchange rate type table exists */
%let _tmpDSN = &xrate_type_table;
%if not %sysfunc( exist(&_tmpDSN) ) %then %do;
   %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=15
           , msgPutlog=N);
   %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=10
           , msgTxt1=&_tmpDSN, msgTxt2=&sesgmsg_msgReturn
           , msgPutlog=&putlog);
   %goto ratework_exit;
%end;

proc sort data=&_tmpDSN;
   by &_tmpSortVar;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

%ratework_exit:

%mend ratework;

%ratework(&xrdtypes);

%if &_tmpRC %then %goto exit;

/******************************************************************************************/
/* Filter rates down by type.                                                             */
/******************************************************************************************/

%let _tmpSortVar = EXCHNG_RATE_TYPE_RK;
proc sort data=WORK._TMPCRATE;
   by &_tmpSortVar;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

/* Check if cycle's exchange rate type table exists */
%let _tmpDSN = WORK._XRATETYPES;
%if not %sysfunc( exist(&_tmpDSN) ) %then %do;
   %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=15
           , msgPutlog=N);
   %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=10
           , msgTxt1=&_tmpDSN, msgTxt2=&sesgmsg_msgReturn
           , msgPutlog=&putlog);
   %goto exit;
%end;

proc sort data=&_tmpDSN;
   by &_tmpSortVar;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

data WORK.ETL_COMPRATE;
   merge WORK._TMPCRATE (in = inA)
         &_tmpDSN (in = inB);
      by &_tmpSortVar;
   if (inA) and (inB) then
      output;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

/******************************************************************************************/

data WORK._TMPCRATECONCAT(keep=XRATE_IDX EXCHNG_RATE_TYPE_RK FROM_CURNC_MEMB_RK TO_CURNC_MEMB_RK
                              TIME_MEMB_RK CONCAT_MEMBER_IDS RATE
                         drop=IDXCNT) ;
  length XRATE_IDX 8 ;
  length EXCHNG_RATE_TYPE_RK 8 ;
  length FROM_CURNC_MEMB_RK 8 ;
  length TO_CURNC_MEMB_RK 8 ;
  length TIME_MEMB_RK 8 ;
  length CONCAT_MEMBER_IDS $255 ;
  length RATE 8 ;
  format RATE best32.;
  retain IDXCNT 0 ;
  set WORK.ETL_COMPRATE;
  IDXCNT = IDXCNT + 1 ;
  XRATE_IDX = IDXCNT ;
run ;
%_chkError;
%if &_tmpRC %then %goto exit;

/*******************************************************************************/

%sesFMS_uploadDataToSDM(cSrcNm=WORK._TMPCRATECONCAT, cTrgNm=&fmsetl_tempCreateComplexTableNm);

data WORK.ETL_XRATE;
   set WORK.ETL_SIMPRATE
       WORK.ETL_COMPRATE;
run;

%complex_exit:

proc sort in=WORK.ETL_XRATE out=WORK._countRates1 ;
  /*by EXCHNG_RATE_TYPE_RK ; */
  by EXCHNG_RATE_TYPE_RK TIME_MEMB_RK;
run ;
%_chkError;
%if &_tmpRC %then %goto exit;

proc summary data=WORK._countRates1 ;
  /*by EXCHNG_RATE_TYPE_RK ;   */
  by EXCHNG_RATE_TYPE_RK TIME_MEMB_RK;
  /*output out=WORK._countRates1 (drop=_TYPE_
                   rename=(_FREQ_=RATE_CNT));*/
  output out=work._countRates1
     (drop=_TYPE_ rename=(_FREQ_=RATE_CNT EXCHNG_RATE_TYPE_RK=XRATE_TYPE_MEMBER_ID TIME_MEMB_RK=TIME_MEMBER_ID));
run ;
%_chkError;
%if &_tmpRC %then %goto exit;

*%sesFMS_uploadDataToSDM(cSrcNm=WORK._countRates1, cTrgNm=&fmsetl_tempStatusTableNm);

data _SDMLIB.&fmsetl_tempStatusTableNm;
  set WORK._countRates1;
run ;
%_chkError;
%if &_tmpRC %then %goto exit;

data null ;
   call symput('nElapsedTime', datetime() - symget('nStartTime')) ;
run;
%_chkError;
%if &_tmpRC %then %goto exit;

%put TIMER: Total time to import Exchange Rates: &nElapsedTime ;

%let fmsetl_statusId = 0; /* Program ran successful */

%exit:

%if &fmsetl_statusID ne 0 %then %do;
   %let fmsetl_error = &sesgmsg_msgCodeReturn;
   %let fmsetl_errormsg = &sesgmsg_msgReturn;
%end;

%mend cycxrate;
