
%MACRO sesfms;


/**********************************************************
 * sesFMS_addSourceCode.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_addSourceCode.sas                                                       */
/*  Purpose: Adds a column of type String with a specific Code value.                        */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */
%macro sesFMS_addSourceCode(cDS, cColNm, cSourceCode);
  %let cMethodNm = sesFMS_addSourceCode;
  %let nTrackTime = %qsysfunc(datetime());

  /*  Make sure we have a source data set  */
  %if not %sysfunc(exist(&cDS)) %then %do;
    %let error_number=27;
    %let errObj=&cDS;
    %goto ERRORRETURN;
  %end ;

  /*  Add the column to the data set   */
  data &cDS ;
       length &cColNm $32 ;
       set &cDS ;
       &cColNm = "&cSourceCode" ;
       if TRANSACTION_AMT not in(.) ;
  run;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesSOL_logError( YTDConv, L, &error_number, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_addSourceCode;


/**********************************************************
 * sesFMS_appendDS.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_appendDS.sas                                                            */
/*  Purpose: Appends non-derived Period activity trans. to newly derived Period activity     */
/*           trans.                                                                          */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */
%macro sesFMS_appendDS(cBaseDS, cDataDS);
  %let cMethodNm = sesFMS_appendDS;
  %let  nTrackTime = %qsysfunc(datetime()) ;

  /*  Make sure we have both data sets  */
  %if not %sysfunc(exist(&cBaseDS, DATA)) %then %do;
    %let error_number=22;
    %let errObj=&cBaseDS;
    %goto ERRORRETURN;
  %end ;

  %if not %sysfunc(exist(&cDataDS, DATA)) %then %do;
    %let error_number=23;
    %let errObj=&cDataDS;
    %goto ERRORRETURN;
  %end ;

  /*  Run the Append  */
  proc append base=&cBaseDS data=&cDataDS FORCE ;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesSOL_logError( YTDConv, L, &error_number, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_appendDS;


/**********************************************************
 * sesFMS_buildIsFirstPrdFormatDS.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_buildIsFirstPrdFormatDS.sas                                             */
/*  Purpose: Build a temporary data set that maps IsFirstPrd status with DDS Time Mbr RK.    */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */
%macro sesFMS_buildIsFirstPrdFormatDS( cCurrTimeMbrCode_Map_PrdId_DS,
                                        cCurrTimeMbrDDS_RKs_DS,
                                        cIsFirstPrd_Map_PrdId_DS,
                                        cFormatDS,
                                        cDDSLib);

  %let cMethodNm = sesFMS_buildIsFirstPrdFormatDS;
  %let cTempDS1 = work._fmsetlTemp01 ;
  %let nowDTTM = %qsysfunc(datetime()) ;
  %let nTrackTime = %qsysfunc(datetime()) ;


  /*  Clean up  */
  %if %sysfunc(exist(&cTempDS1)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cTempDS1, 2, '.');
    quit;
  %end;
  %if %sysfunc(exist(&cFormatDS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cFormatDS, 2, '.');
    quit;
  %end;

  /*Sort all three input */
  proc sort in=&cCurrTimeMbrCode_Map_PrdId_DS ;
        by VALUE;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  proc sort in=&cCurrTimeMbrDDS_RKs_DS ;
     by VALUE;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  proc sort in=&cIsFirstPrd_Map_PrdId_DS ;
    by CODE;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;


  data &cTempDS1 ;
    merge &cCurrTimeMbrCode_Map_PrdId_DS(in=a)
    &cCurrTimeMbrDDS_RKs_DS(in=b rename=(CODE=DDS_RK)) ;
    by VALUE ;
    if a and b then do;
      output;
    end;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  proc sort in=&cTempDS1 ;
    by CODE;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  data &cFormatDS(drop=CODE rename=(DDS_RK=CODE)) ;
    merge &cTempDS1(in=a)
    &cIsFirstPrd_Map_PrdId_DS(in=b) ;
    by CODE ;
    if a and b then do;
      output;
    end;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );

  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_buildIsFirstPrdFormatDS;


/**********************************************************
 * sesFMS_buildRunYTDToPAFormatDS.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_buildRunYTDToPAFormatDS.sas                                             */
/*  Purpose: Build a temporary data set that maps YTD to PA status with DDS Time Mbr RK.     */
/*  Usage  :                                                                                 */
/*           Input :                                                                         */
/*             cCurrTimeMbrCode_Map_PrdId_DS                                                 */
/*             CODE = Period Id                                                              */
/*             VALUE = SDM Time Mbr Codes                                                    */
/*                                                                                           */
/*             cCurrTimeMbrDDS_RKs_DS                                                        */
/*             CODE = DDS RK                                                                 */
/*             VALUE = SDM Time Mbr Codes                                                    */
/*                                                                                           */
/*             cRunYTDToPA_Map_PrdId_DS                                                      */
/*             CODE = Period Id                                                              */
/*             VALUE = Y or N                                                                */
/*                                                                                           */
/*           Output:                                                                         */
/*             cFormatDS                                                                     */
/*             CODE = DDS RK                                                                 */
/*             VALUE = Y or N                                                                */
/*                                                                                           */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */
%macro sesFMS_buildRunYTDToPAFormatDS(cCurrTimeMbrCode_Map_PrdId_DS,
                                        cCurrTimeMbrDDS_RKs_DS,
                                        cRunYTDToPA_Map_PrdId_DS,
                                        cFormatDS,
                                        cDDSLib);

  %let cMethodNm = sesFMS_buildRunYTDToPAFormatDS;
  %let cTempDS1 = work._fmsetlTemp01;
  %let nowDTTM = %qsysfunc(datetime()) ;
  %let nTrackTime = %qsysfunc(datetime()) ;

  /*  Clean up  */
  %if %sysfunc(exist(&cTempDS1)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cTempDS1, 2, '.');
    quit;
  %end;
  %if %sysfunc(exist(&cFormatDS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cFormatDS, 2, '.');
    quit;
  %end;


  /* Sort all three input */
  proc sort in=&cCurrTimeMbrCode_Map_PrdId_DS ;
            by VALUE;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;


  proc sort in=&cCurrTimeMbrDDS_RKs_DS ;
      by VALUE;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  proc sort in=&cRunYTDToPA_Map_PrdId_DS ;
       by CODE;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;


  data &cTempDS1 ;
       merge &cCurrTimeMbrCode_Map_PrdId_DS(in=a)
             &cCurrTimeMbrDDS_RKs_DS(in=b rename=(CODE=DDS_RK)) ;
       by VALUE ;
       if a and b then do;
          output;
       end;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;


  proc sort in=&cTempDS1 ;
       by CODE;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  data &cFormatDS(drop=CODE rename=(DDS_RK=CODE)) ;
        merge &cTempDS1(in=a) &cRunYTDToPA_Map_PrdId_DS(in=b) ;
    by CODE ;
    if a and b then do;
      output;
    end;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;


  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );

  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend  sesFMS_buildRunYTDToPAFormatDS;



/**********************************************************
 * sesFMS_buildStatusDS.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_buildStatusDS.sas                                                       */
/*  Purpose: Build a temporary data set to hold status of this import.                       */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */
%macro sesFMS_buildStatusDS( cTimeDS, cAnalysisDS, cStatusDS, statusOverride);

  %let cMethodNm = sesFMS_buildStatusDS;
  %let cTempStatusDS = WORK._FMSETLTEMP01;
  %let nowDTTM = %qsysfunc(datetime()) ;
  %let nTrackTime = %qsysfunc(datetime()) ;

  %if %sysfunc(exist(&cStatusDS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cStatusDS, 2, '.');
    quit;
  %end;

  %if %sysfunc(exist(&cTempStatusDS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cTempStatusDS, 2, '.');
    quit;
  %end;

  data &cTempStatusDS(keep=TIME_ID ANALYSIS_ID PERIOD_ID STATUS_ID
                           IMPORTED_TRANS_CNT ZERO_TRANS_CNT INVALID_ERROR_CNT
                           ORG_EQ_TRADER_CNT IC_ERROR_CNT NON_IC_ERROR_CNT REMOVE_CNT) ;
    length TIME_ID 8;
    length ANALYSIS_ID 8;
      length PERIOD_ID 8;
      TIME_ID = -999999;
      ANALYSIS_ID = -999999;
      PERIOD_ID = -999999;
      STATUS_ID = &statusOverride;
      IMPORTED_TRANS_CNT = 0 ;
      ZERO_TRANS_CNT = 0 ;
      INVALID_ERROR_CNT = 0 ;
      ORG_EQ_TRADER_CNT = 0 ;
      IC_ERROR_CNT = 0 ;
      NON_IC_ERROR_CNT = 0 ;
      REMOVE_CNT = 0 ;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  proc append base=&cStatusDS data=&cTempStatusDS;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_buildStatusDS;


/**********************************************************
 * sesFMS_calcPeriodActivity.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_calcPeriodActivity.sas                                                  */
/*  Purpose: Calculate period activity transaction values.                                   */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */
%macro sesFMS_calcPeriodActivity(cCurrentPrdDS, cPriorPrdDS, lTranCols, cCurrentAmtColNm, cPriorAmtColNm, cOutputDS, cPrdIdColNm);
  %let cMethodNm = sesFMS_calcPeriodActivity;
  %let nTrackTime = %qsysfunc(datetime());

  /*  Clean up, in case we had a previously aborted attempt  */
  %if %sysfunc(exist(&cOutputDS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cOutputDS, 2, '.');
    quit;
  %end;

  /*  Make sure we have a source transaction tables in the DDS  */
  %if not %sysfunc(exist(&cCurrentPrdDS, DATA))  %then %do;
    %let error_number=19;
    %let errObj=&cCurrentPrdDS;
    %goto ERRORRETURN;
  %end ;
  %if not %sysfunc(exist(&cPriorPrdDS, DATA))  %then %do;
    %let error_number=20;
    %let errObj=&cPriorPrdDS;
    %goto ERRORRETURN;
  %end ;

  /*  Build the column merge by list  */
  filename _col temp;
  data _null_;
      file _col;
    set &lTranCols end=eof;
    put '"' value + (-1) '"n ';
      if eof then put ';';
  run;

  /*  Do the math and build a resulting converted data set  */
  proc sort in=&cCurrentPrdDS out=&cCurrentPrdDS;
    by
         %inc _col;
  run;

  %if "&cProduct" ne "OP" %then %do;
    %let _col=AFFECTED_TIME_PERIOD_RK;
  %end;
  %else %do;
    %let _col=TIME_PERIOD_RK;
  %end;

  data &cOutputDS (drop=&cCurrentAmtColNm &cPriorAmtColNm &cPrdIdColNm) ;
       length TRANSACTION_DT 8 ;
       merge &cCurrentPrdDS (in=nCurrentObs) &cPriorPrdDS(drop=&_col) ;

       by
            %inc _col ;
       if nCurrentObs then do;
        if &cPriorAmtColNm = . then &cPriorAmtColNm = 0 ;
        TRANSACTION_AMT = &cCurrentAmtColNm - &cPriorAmtColNm ;
        TRANSACTION_DT  = . ;
        if TRANSACTION_AMT not in(.) then output ;
       end ;
  run ;


  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %sesSOL_logError( YTDConv, L, 21, &cMethodNm) ;
    %goto RETURNZERO;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesSOL_logError( YTDConv, L, &error_number, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_calcPeriodActivity;


/**********************************************************
 * sesFMS_convertDdsRksToCodes.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_convertDdsRksToCodes.sas                                                */
/*  Purpose: Build Required Formats and Convert DDS RKs to Codes.                            */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_convertDdsRksToCodes( cDS, cDDSLib, lRKtoCode, lDimTypeRKCol
                        , cCurrTimePrdDDSRK_to_Code_FMT, lCycleDimTypesRKs);
%let cMethodNm = sesFMS_convertDdsRksToCodes;
%let nTrackTime = %qsysfunc(datetime()) ;
%let nHashExpMax = 20;   /* This should be decided in mid-tier or parent caller or system */

/*  Make sure we have a source data set  */
%if not %sysfunc(exist(&cDS)) %then %do;
      %goto RETURNZERO;
%end;

/* Clean up and create lFormats */
%if %sysfunc(exist(lFormats)) %then  %do;
  proc datasets library=work nolist;
    delete lFormats;
  quit;
%end;
proc sql noprint;
   create table lFormats (FormatNm varchar(100), DimTypeCode varchar(100));
quit;

/*  Build all the formats that we need  */
%let cTempTableNm = WORK._tempFormat;

proc sql noprint;
  select count(*)
    into :num_data
    from &lCycleDimTypesRKs;
quit;
%let num_data=%eval(&num_data);

%if &num_data > 0 %then %do;
  proc sql noprint;
    select code
        into :_code1 - :_code&num_data
      from &lCycleDimTypesRKs;
  quit;
%end;
%else %do;
  %goto RETURNZERO;
%end;

%do nXX=1 %to &num_data;
  %let  nBuildFormat = 0;
  %let num_match=0;

  proc sql noprint;
    select count(*)
      into :num_match
      from &lRKtoCode
     where %trim(code)="&&_code&nXX";
  quit;

  %if &num_match < 1 %then %do;
    %let error_number=32;
    %let errObj=&&_code&nXX;
    %goto ERRORRETURN;
  %end;
  %else %do;
    proc sql noprint;
      select code, value
          into :cDimTypeCode, :cTableNm
          from &lRKtoCode
       where code="&&_code&nXX";
    quit;
    %let cDimTypeCode=%trim(&cDimTypeCode);
    %let cTableNm=%trim(&cTableNm);
  %end;
  %let cFormatNm = F&nXX.A;

  %if (%nrbquote(&cDimTypeCode) eq %bquote(TIME)) or (%nrbquote(&cDimTypeCode) eq %bquote(TRADER))
     or (%nrbquote(&cDimTypeCode) eq %bquote(CURRENCY)) or (%nrbquote(&cDimTypeCode) eq %bquote(ITEMCAT))
     or (%nrbquote(&cDimTypeCode) eq %bquote(ADJTYPE)) or (%nrbquote(&cDimTypeCode) eq %bquote(SOURCE)) %then %do;
    /* do nothing */
  %end;
  %else %do;
            proc sql noprint;
                select count(*)
                into :nFound
                from &lDimTypeRKCol
               where code="&cDimTypeCode";
            quit;

            %if &nFound < 1 %then %do;
                %sesSOL_logError( YTDConv, L, 30, &cFormatNm, &cDimTypeCode, &cMethodNm);
                %goto RETURNZERO;
            %end;

            proc sql noprint;
                select value
                  into :cRK
                  from &lDimTypeRKCol
                 where code="&cDimTypeCode";
             quit;

             %if &cRK eq  or not %symexist(cRK) %then %do;
                 %sesSOL_logError( YTDConv, L, 30, &cFormatNm, &cDimTypeCode, &cMethodNm);
                 %goto RETURNZERO;
             %end;

             %let cID =;
             %let solsetl_tempCode1 = ;

             proc sql noprint;
                select case
                   when strip(KEY_COLUMN_NM) = 'MEMBER_RK' then 'MEMBER_ID'
                   else BUSINESS_ID_COLUMN_NM
                   end as BUSINESS_ID_COLUMN_NM
                   into :solsetl_tempCode1
                   from &cDDSLib..DIMENSION_TYPE
                   where DIMENSION_TYPE_CD eq "&cDimTypeCode" and
                         (VALID_FROM_DTTM < &fmsetl_processedDTTM_To and
                          VALID_TO_DTTM > &fmsetl_processedDTTM_To)
                   ;
             quit ;

             %let cID = &solsetl_tempCode1;
             proc sql noprint ;
                 create table &cTempTableNm as
                     select &cRK as CODE, &cID as VALUE
                       from &cDDSLib..&cTableNm;
             quit ;

             %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
                 %let error_number=28;
                 %let errObj=&cDimTypeCode;
                 %goto ERRORRETURN;
             %end ;

             %let nBuildFormat = 1 ;
  %end;


  %if &nBuildFormat %then %do;
    %if %sysfunc(exist(lTemp)) %then  %do;
      proc datasets library=work nolist;
        delete %kscan(&cOutputYtdDS, 2, '.');
      quit;
    %end;

    %let temp_ltemp = &cDimTypeCode._list;
    %sesFMS_createFormat( &cTempTableNm, &cFormatNm, 1, &temp_ltemp, 1);
  /* %sesFMS_createFormat( &cTempTableNm, &cFormatNm, 1, lTemp, 0); */

    %if _FMErrorCode = 0 %then %do;
        %sesSOL_logError( &cLogEntry, L, 29, &cFormatNm, &cDimTypeCode, &cMethodNm);
        %goto RETURNZERO;
    %end ;

    %if %sysfunc(exist(lTemp)) %then  %do;
      proc datasets library=work nolist;
        delete %kscan(&cOutputYtdDS, 2, '.');
      quit;
    %end;

    proc sql noprint;
      insert into lFormats values("&cFormatNm", "&cDimTypeCode");
    quit;

    %if %nrbquote(&cDimTypeCode) eq %bquote(INTORG) %then %do;
      proc sql noprint;
        insert into lFormats values("&cFormatNm", "TRADER");
      quit;
    %end;
  %end ;
%end ;

/*  Convert RK(s) to Code(s)  */
/*  drop cols not CURRENCY_CD*/

filename _drop temp;
data _null_;
  file _drop;
  set &lCycleDimTypesRKs(where=(value NE "CURRENCY_CD")) end=eof;
  put '"' value +(-1) '"n ' ;
  if eof then
    put ';';
run;

filename _length temp;
data _null_;
  file _length;
  set &lCycleDimTypesRKs(where=(value NE "CURRENCY_CD")) end=eof;
  code = strip(putc(code, '$DTPCOL.')) !! "_CD";
  put '"' code +(-1) '"n $32 ' ;
  if eof then
    put ';';
run;


proc sql noprint;
    select count(*)
      into :nNumRow
      from lCycleDimTypesRKs
     where value NE "CURRENCY_CD";
quit;
%let nNumRow = %eval(&nNumRow);
%let nHashSize=;

filename _stmts temp;
data _null_;
  file _stmts;
  length dimtypecode formatnm $100 ;
  if _N_ = 1 then do;
    nRow=symgetn('nNumRow');
    call symputx( 'nHashSize', ceil(log2(nRow)));
    nHash=symgetn('nHashSize');
    nHashMax = symgetn('nHashExpMax');
    if nHash > nHashMax then do;
        call symputx('nHashSize', nHashMax);
    end;
    nHash=symgetn('nHashSize');

    declare hash h0(dataset: "lformats", hashexp: nHash);
    h0.defineKey("DimTypeCode");
    h0.defineData("FormatNm");
    h0.defineDone();
    call missing(DimTypeCode, FormatNm);
  end;

  set lCycleDimTypesRKs(where=(value NE "CURRENCY_CD"));

  if code = 'TIME' then
    FormatNm = symget('cCurrTimePrdDDSRK_to_Code_FMT');
  else
    h0.find(key: code);

  code = strip(putc(code, '$DTPCOL.')) !! "_CD";

  put '"' code +(-1) '"n = put("' value +(-1) '"n, $' +(-1) FormatNm +(-1) '.); ' ;
  put 'if "' code +(-1) '"n = "*" then do; ' ;
  put '   "' code +(-1) '"n = "*DDS RK* " !! strip(put("' value +(-1) '"n, best26.)) ;' ;
  put 'end;' ;

run;

data &cDS ;
   drop
    %inc _drop;
  length
    %inc _length;

  set &cDS ;

  %inc _stmts;

  if TRANSACTION_AMT not in(.) ;
run ;


%if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
  %let error_number=31;
  %let errObj=&cDS;
  %goto ERRORRETURN;
%end;

/*  Dump the Time  */
%sesSOL_dumpTimer( &nTrackTime, &cMethodNm );

%goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

%ERRORRETURN:
  %sesSOL_logError( YTDConv, L, &error_number, &errObj, &cMethodNm);

%RETURNZERO:
  %let _FMErrorCode=0;

%THEEND:

%mend sesFMS_convertDdsRksToCodes;


/**********************************************************
 * sesFMS_createDimTypeFactSetOrd.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_createDimTypeFactSetOrd.sas                                             */
/* Create Dim Type List in Fact Set Order.                                                   */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_createDimTypeFactSetOrd(cWorkAreaDS, lDim);
  %let cMethodNm = sesFMS_createDimTypeFactSetOrd;
  %let nowDTTM =    %qsysfunc(datetime());
  %let nTrackTime = %qsysfunc(datetime());

  /*  Turn(copy) the table into a named table  */
  %if (%sysfunc(exist(&cWorkAreaDS))) %then %do;
    data &lDim;
      set &cWorkAreaDS;
      run;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );

  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_createDimTypeFactSetOrd;


/**********************************************************
 * sesFMS_createErrorReport.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : YTDConv_createErrorReport.sas                                                   */
/*  Purpose: Print error report to Dav.                                                      */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */
%macro sesFMS_createErrorReport(lCycleDimTypes,
                                  cInvalidDS,
                                  nInvalidCnt,
                                  cOrgEqTraderDS,
                                  nOrgEqTraderCnt,
                                  cIcErrDS,
                                  nIcErrCnt,
                                  cNonIcErrDS,
                                  nNonIcErrCnt,
                                  cDataLoadId);

  %let cMethodNm = sesFMS.createErrorReport;
  %let cInvalidTitle = %sesSOL_getMessage( YTDConv, R, 1, "", "", "", "", "") ;
  %let cInvalidTitle2 = %sesSOL_getMessage( YTDConv, R, 9, "", "", "", "", "") ;

  %if "&cProduct" ne "OP" %then
    %let cInvalidTitle3 = %sesSOL_getMessage( YTDConv, R, 10, "", "", "", "", "") ;
  %else
    %let cInvalidTitle3 = %sesSOL_getMessage( YTDConv, R, 12, "", "", "", "", "") ;

  %let cInvalidTitle4 = %sesSOL_getMessage( YTDConv, R, 11, "", "", "", "", "") ;
  %let cICErrorTitle = %sesSOL_getMessage( YTDConv, R, 2, "", "", "", "", "") ;
  %let cNonICErrorTitle = %sesSOL_getMessage( YTDConv, R, 3, "", "", "", "", "") ;
  %let cOrgEqTraderTitle = %sesSOL_getMessage( YTDConv, R, 4, "", "", "", "", "") ;
  %let cValueFormat = %sesSOL_getMessage( YTDConv, R, 5, "", "", "", "", "") ;
  %let cDateFormat = %sesSOL_getMessage( YTDConv, R, 6, "", "", "", "", "") ;
  %let cTimeFormat = %sesSOL_getMessage( YTDConv, R, 7, "", "", "", "", "") ;
  %let displayMaxObs = 1000 ; /* Max display rows in error report */
  %let nTrackTime = %sysfunc(datetime());

  /*  Set HTML files names  */
  /* %let cDataLoadId  =&fmsetl_tempCheck1; */

  %let cIndexHTML   = index_&cDataLoadId..html;
  %let cBodyHTML    = body_&cDataLoadId..html;
  %let cContentHTML = contents_&cDataLoadId..html;

  /*  Source dim type is not part of the list, but we need it for this operation   */

  /*  Clean up  first */
  %if %sysfunc(exist(lDimTypes)) %then  %do;
    proc datasets library=work nolist;
      delete lDimTypes;
    quit;
  %end;

  /* create lDimTypes */
  proc sql noprint;
    create table lDimTypes as
    select *
      from &lCycleDimTypes;
  quit;

  footnote %sysfunc(today(), &cDateFormat), %sysfunc(time(), &cTimeFormat) ;
  ods html file="&cBodyHTML" frame="&cIndexHTML" contents="&cContentHTML" ;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  /*--------------------------------------*/
  /*  Create Invalid Transactions Report  */
  /*--------------------------------------*/
  %let displayObs = &nInvalidCnt;
  %let cSubTitle = %sesSOL_getMessage(YTDConv, R, 8, %qsysfunc(strip(&displayObs)), "", "", "", "") ;

  %if &displayObs gt &displayMaxObs %then %do;
    %let cSubTitle = &cSubTitle - %sesSOL_getMessage( YTDConv, R, 9,
                                        %qsysfunc(strip(&displayMaxObs)), "", "", "", "") ;
    %let displayObs = &displayMaxObs;
  %end;

  title "&cInvalidTitle";
  title2 "&cInvalidTitle2";
  title3 "&cInvalidTitle3";
  title4 "&cInvalidTitle4";
  ods proclabel="&cInvalidTitle";


  filename _col_st1 temp;
  data _null_;
      file _col_st1;
    set lDimTypes end=eof;
    code = strip(putc(code, '$DTPCOL.')) !! "_ERROR";
    put '"' code +(-1) '"n ' ;
  run;

  data _null_;
    file _col_st1 mod;
    set lDimTypes end=eof;
    code = strip(putc(code, '$DTPCOL.')) !! "_CODE";
    put '"' code +(-1) '"n ' ;
    if eof then
      put ' MEASURE;';
  run;

  filename _col_st3 temp;
  data _null_;
    file _col_st3;
    set lDimTypes end=eof;
    code = strip(putc(code, '$DTPCOL.')) !! "_ERROR";
      put 'define "' code +(-1) '"n / noprint; ' ;
  run;

  data _null_;
    file _col_st3 mod;
    set lDimTypes end=eof;
    code = strip(putc(code, '$DTPCOL.')) !! "_CODE";
    cDimTypeLabel = strip(Propcase(code)) ;
    put 'define "' code +(-1) '"n / display "' cDimTypeLabel +(-1) '";';
    if eof then
      put 'define MEASURE / display "Value" format=&cValueFormat;';
  run;

  filename _col_st5 temp;
  data _null_;
    file _col_st5;
    set lDimTypes end=eof;
    code1 = strip(putc(code, '$DTPCOL.')) !! "_CODE";
    code2 = strip(putc(code, '$DTPCOL.'));
    put 'compute "' code1 +(-1) '"n;';
    put 'if _C' _N_ +(-1) '_=1 then do;';
    put 'call define(_COL_, "STYLE", "style(CALLDEF)={background=#f0f0f0 foreground=red font_weight=bold}" );';
    put 'end;';
    put 'endcomp;';
  run;

  proc report data=&cInvalidDS(obs=&displayObs) nowd split=' ' contents="&cSubTitle";
    column
      %inc _col_st1;
      %inc _col_st3;
      %inc _col_st5;
      ;
  run;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  /*--------------------------------------*/
  /*    Create Org=Trader Error Report    */
  /*--------------------------------------*/
  filename _col_st2 temp;
  data _null_;
    file _col_st2 mod;
    set lDimTypes end=eof;
    code = strip(putc(code, '$DTPCOL.')) !! "_CODE";
    put '"' code +(-1) '"n ' ;
    if eof then
      put ' MEASURE;';
  run;

  filename _col_st4 temp;
  data _null_;
    file _col_st4 mod;
    set lDimTypes end=eof;
    code = strip(putc(code, '$DTPCOL.')) !! "_CODE";
    cDimTypeLabel = strip(Propcase(code)) ;
    put 'define "' code +(-1) '"n / display "' cDimTypeLabel +(-1) '";';
    if eof then
      put 'define MEASURE / display "Value" format=&cValueFormat;';
  run;

  %let displayObs = &nOrgEqTraderCnt ;
  %let cSubTitle = %sesSOL_getMessage( YTDConv, R, 8, %qsysfunc(strip(&displayObs)), "", "", "", "") ;
  %if &displayObs gt &displayMaxObs %then %do;
    %let cSubTitle = &cSubTitle - %sesSOL_getMessage( YTDConv, R, 9,
                                          %qsysfunc(strip(&displayMaxObs)), "", "", "", "") ;
    %let displayObs = &displayMaxObs;
  %end;

  title "&cOrgEqTraderTitle";
  ods proclabel="&cOrgEqTraderTitle";
  proc report data=&cOrgEqTraderDS(obs=&displayObs) nowd split=' ' contents="&cSubTitle";
     column
       %inc _col_st2;
       %inc _col_st4;
  run;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  /*--------------------------------------*/
  /*        Create IC Error Report        */
  /*--------------------------------------*/
  %let displayObs = &nIcErrCnt ;
  %let cSubTitle = %sesSOL_getMessage( YTDConv, R, 8, %qsysfunc(strip(&displayObs)), "", "", "", "") ;
  %if &displayObs gt &displayMaxObs %then %do;
    %let cSubTitle = &cSubTitle %sesSOL_getMessage( YTDConv, R, 9,
                                          %qsysfunc(strip(&displayMaxObs)), "", "", "", "") ;
    %let displayObs = &displayMaxObs;
  %end;

  title "&cICErrorTitle";
  ods proclabel="&cICErrorTitle";

  proc report data=&cIcErrDS(obs=&displayObs) nowd split=' ' contents="&cSubTitle";
     column
       %inc _col_st2;
       %inc _col_st4;
  run;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  /*--------------------------------------*/
  /*      Create Non IC Error Report      */
  /*--------------------------------------*/
  %let displayObs = &nNonIcErrCnt ;
  %let cSubTitle = %sesSOL_getMessage( YTDConv, R, 8, %qsysfunc(strip(&displayObs)), "", "", "", "") ;
  %if &displayObs gt &displayMaxObs %then %do;
    %let cSubTitle = &cSubTitle  -  %sesSOL_getMessage( YTDConv, R, 9,
                                          %qsysfunc(strip(&displayMaxObs)), "", "", "", "") ;
    %let displayObs = &displayMaxObs;
  %end;

  title "&cNonICErrorTitle";
  ods proclabel="&cNonICErrorTitle";
  proc report data=&cNonIcErrDS(obs=&displayObs) nowd split=' ' contents="&cSubTitle";
     column
       %inc _col_st2;
       %inc _col_st4;
  run;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;


  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_createErrorReport;


/**********************************************************
 * sesFMS_createFormat.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_createFormat.sas                                                        */
/*  Purpose: Create a SAS Format from work area data set and return a named list in a table  */
/*           It sets 1 to global _FMErrorCode, if succeeds. Otherwise, 0 and jump to the end.*/
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_createFormat(cWorkAreaDS,cFormatNm,codeIsNum,lDim,nBuildList);

  %let _FMErrorCode=1;  /* it is a global, should be defined in the parent programs
                           0 : error, 1: successful for all processes in this macro */
  %let cMethodNm = sesFMS_createFormat;
  %let cFormatDS = _dsformatYTD;
  %let nZeroObs = 0;
  %let nCodeFld=0;
  %let nValueFld=0;
  %let nTrackTime=0;
  %let dsID=0;
  %let nRC=0;
  %let returnNum=0;
  %let nTrackTime = %sysfunc(datetime());
  %let zero=0;

  %if %sysfunc(exist(&cWorkAreaDS)) %then %do;
    %let dsID=%sysfunc(open(&cWorkAreaDS));
    %if %sysfunc(attrn(&dsID, NLOBS)) EQ 0 %then
      %let nZeroObs = 1;
    %if &dsID %then
      %let dsID=%sysfunc(close(&dsID));  /* 0 if successuful */
  %end;
  %else %do;
    %let error_number=4;
    %let errObj=&cWorkAreaDS;
    %goto ERRORRETURN;
  %end;

  /*  Clean up, in case we had a previously aborted attempt  */
  %if %sysfunc(exist(&cFormatDS)) %then  %do;
    proc datasets library=work nolist;
      delete &cFormatDS;
    run;
  %end;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %let error_number=5;
    %let errObj=&cFormatNm;
    %goto ERRORRETURN;
  %end;

  /* Build format */
  /*----------------------------------------*/
  /*----------------------------------------*/
  /* Check if cWorkAreaDS has FMTNAME field */
  /*----------------------------------------*/
  /*----------------------------------------*/

  proc sql noprint;
      select count(*)
        into :moreThan2fmt
        from dictionary.columns
       where memname=upper("&cWorkAreaDS")
         and name="FMTNAME";
  quit;
  %let moreThan2fmt=%eval(&moreThan2fmt);
  %if &moreThan2fmt > 0 %then %do;
    %let fmtname_stmt = FMTNAME=FMTNAME;
    %let _dup_option =;
    %let _sort_key = FMTNAME CODE;
  %end;
  %else %do;
    %let fmtname_stmt = FMTNAME="&cFormatNm";
    %let _dup_option = nodupkey;
    %let _sort_key = CODE;
  %end;

  proc sort data=&cWorkAreaDS out=&cFormatDS &_dup_option;
          by &_sort_key;
  run;

  %if (&SYSERR NE 0) AND (&SYSERR NE 4) %then %do;
        %let error_number=5;
      %let errObj=&cFormatNm;
        %goto ERRORRETURN;
  %end;

  /*  Turn(copy) the table into a named table  */
  %if &nBuildList %then %do;

    %let _var=&lDim;
    %let _wrong=%sysfunc( kcompress( %sysfunc( ksubstr(&_var, 1, 1))
                      , '_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'));
    %let _wrongrest=%sysfunc( kcompress( %sysfunc( ksubstr(&_var, 2))
                      , '0123456789_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'));

    %put _wrong=&_wrong;
    %put _wrongrest=&_wrongrest;
    %if %nrbquote(&_wrong) = %str() and %nrbquote(&_wrongrest) = %str() %then %do;
      data &lDim;
      set &cFormatDS;
      run;
    %end;
  %end;

  %if &codeIsNum %then %do;
    %if &nZeroObs %then %do;
      data &cFormatDS;
              length START 8;
              length LABEL $64;
              length FMTNAME $64;
              length HLO $1;
              keep START LABEL FMTNAME HLO;
              &fmtname_stmt;
              START   = .;
              LABEL   = "*";
              HLO     = "O";
              output;
      run;
    %end;
    %else %do;
      data &cFormatDS;
        set &cFormatDS end=nEnd;
        length START 8;
        length LABEL $64;
        length FMTNAME $64;
        length HLO $1;
        keep START LABEL FMTNAME HLO;
      &fmtname_stmt;
        START   = CODE;
        LABEL   = VALUE;
        HLO     = "";
        output;
        if nEnd then do;
          &fmtname_stmt;
          START   = .;
          LABEL   = "*";
          HLO     = "O";
          output;
        end;
      run;
    %end;
  %end;
  %else %do;
    %if &nZeroObs %then %do;
      data &cFormatDS;
              length START $64;
              length LABEL $64;
              length FMTNAME $64;
              length HLO $1;
              keep START LABEL FMTNAME HLO;
              &fmtname_stmt;
              START   = "";
              LABEL   = "*";
              HLO     = "O";
              output;
      run;
    %end;
    %else %do;
      data &cFormatDS;
              set &cFormatDS end=nEnd;
              length START $64;
              length LABEL $64;
              length FMTNAME $64;
              length HLO $1;
              keep START LABEL FMTNAME HLO;
              &fmtname_stmt;
              START   = CODE;
              LABEL   = VALUE;
              HLO     = "";
              output;
              if nEnd then do;
                FMTNAME = "&cFormatNm";
                START   = "";
                LABEL   = "*";
                HLO     = "O";
                output;
                end;
            run;
    %end;
  %end;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %let error_number=5;
    %let errObj=&cFormatNm;
    %goto ERRORRETURN;
  %end;

  proc format cntlin=&cFormatDS library=WORK;
  run;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %let error_number=5;
    %let errObj=&cFormatNm;
    %goto ERRORRETURN;
  %end;


  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );

  /*  Clean up  */
  /*
  %if %sysfunc(exist(&cFormatDS)) %then  %do;
    proc datasets library=work nolist;
      delete &cFormatDS;
    run;
  %end;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %let error_number=5;
    %let errObj=&cFormatNm;
    %goto ERRORRETURN;
  %end;

  */
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %let _FMErrorCode=0;
    %sesSOL_logError( YTDConv, L, &error_number, &errObj, &cMethodNm);

  %THEEND:

%mend sesFMS_createFormat;




/**********************************************************
 * sesFMS_createFormatNew.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_createFormat.sas                                                        */
/*  Purpose: Create a SAS Format from work area data set and return a named list in a table  */
/*           It sets 1 to global _FMErrorCode, if succeeds. Otherwise, 0 and jump to the end.*/
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_createFormatNew(cWorkAreaDS,cFormatNm,codeIsNum,lDim,nBuildList);
%let _FMErrorCode=1;  /* it is a global, should be defined in the parent programs
                         0 : error, 1: successful for all processes in this macro */
%let cMethodNm = sesFMS_createFormat;
%let nZeroObs = 0;
%let nCodeFld=0;
%let nValueFld=0;
%let nTrackTime=0;
%let dsID=0;
%let nRC=0;
%let returnNum=0;
%let nTrackTime = %sysfunc(datetime());
%let zero=0;

%if %sysfunc(exist(&cWorkAreaDS)) %then %do;
  %let dsID=%sysfunc(open(&cWorkAreaDS));
  %if %sysfunc(attrn(&dsID, NLOBS)) EQ 0 %then
    %let nZeroObs = 1;
  %if &dsID %then
    %let dsID=%sysfunc(close(&dsID));  /* 0 if successuful */
%end;
%else %do;
  %let error_number=4;
  %let errObj=&cWorkAreaDS;
  %goto ERRORRETURN;
%end;

/* Build format */

/*  Turn(copy) the table into a named table  */
%if &nBuildList %then %do;

  %let _var=&lDim;
  %let _wrong=%sysfunc( kcompress( %sysfunc( ksubstr(&_var, 1, 1))
                    , '_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'));
  %let _wrongrest=%sysfunc( kcompress( %sysfunc( ksubstr(&_var, 2))
                    , '0123456789_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'));

  %put _wrong=&_wrong;
  %put _wrongrest=&_wrongrest;
  %if %nrbquote(&_wrong) = %str() and %nrbquote(&_wrongrest) = %str() %then %do;
    data &lDim;
    set &cFormatDS;
    run;
  %end;
%end;
%if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
  %let error_number=5;
  %let errObj=&cFormatNm;
  %goto ERRORRETURN;
%end;

proc format cntlin=&cWorkAreaDS library=WORK;
run;

%if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
  %let error_number=5;
  %let errObj=&cFormatNm;
  %goto ERRORRETURN;
%end;


/*  Dump the Time  */
%sesSOL_dumpTimer( &nTrackTime, &cMethodNm );

/*  Clean up  */
/*
%if %sysfunc(exist(&cFormatDS)) %then  %do;
  proc datasets library=work nolist;
    delete &cFormatDS;
  run;
%end;

%if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
  %let error_number=5;
  %let errObj=&cFormatNm;
  %goto ERRORRETURN;
%end;

*/
%goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

%ERRORRETURN:
  %let _FMErrorCode=0;
  %sesSOL_logError( YTDConv, L, &error_number, &errObj, &cMethodNm);

%THEEND:
%mend sesFMS_createFormatNew;




/**********************************************************
 * sesFMS_generateErrorReports.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_generateErrorReports.sas                                                */
/*  Purpose: Print error report to Dav.                                                      */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_generateErrorReports( cStatusDS,
                                  lCycleDimTypes,
                                  cInvalidDS,
                                  cOrgEqTraderDS,
                                  cIcErrDS,
                                  cNonIcErrDS,
                                  cDataLoadId);

  %let cMethodNm = sesFMS_generateErrorReports;
  /* %let nowDTTM = %sysfunc(datetime()); */
  %let nTrackTime = %sysfunc(datetime());

  /*  Prep and submit the STPBegin macro  */

  %global _odsstyle _ODSOPTIONS rptDate rptTime ;
  %let _odsstyle=SASWeb;


  %STPBEGIN;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  %let dsID = %sysfunc(open( &cStatusDS ));
  %if &dsID %then %do;
    %let nStatusFld      = %sysfunc(varnum(&dsID, STATUS_ID)) ;
    %let nInvalidFld     = %sysfunc(varnum(&dsID, INVALID_ERROR_CNT));
    %let nOrgEqTraderFld = %sysfunc(varnum(&dsID, ORG_EQ_TRADER_CNT));
    %let nIcErrFld       = %sysfunc(varnum(&dsID, IC_ERROR_CNT));
    %let nNonIcErrFld    = %sysfunc(varnum(&dsID, NON_IC_ERROR_CNT));
    %let nInvalidCnt     = 0;
    %let nOrgEqTraderCnt = 0;
    %let nIcErrCnt       = 0;
    %let nNonIcErrCnt    = 0;

    %let nRC = %sysfunc(rewind(&dsID));
    %do %while(%sysfunc(fetch(&dsID)) EQ 0);  /*(0, %sysrc(_SWNOUPD), %sysrc(_SENOLCK)));  */
      %if %sysfunc(getvarn(&dsID, &nStatusFld)) ne . %then %do;
        %let nStatusId = %sysfunc(getvarn(&dsID, &nStatusFld));
      %end;

      %if %sysfunc(getvarn(&dsID, &nInvalidFld)) ne . %then %do;
        %let nInvalidCnt = %eval(&nInvalidCnt + %sysfunc(getvarn(&dsID, &nInvalidFld))) ;
      %end;
      %if %sysfunc(getvarn(&dsID, &nOrgEqTraderFld)) ne . %then %do;
        %let nOrgEqTraderCnt = %eval(&nOrgEqTraderCnt + %sysfunc(getvarn(&dsID, &nOrgEqTraderFld))) ;
      %end;
      %if %sysfunc(getvarn(&dsID, &nIcErrFld)) ne . %then %do;
        %let nIcErrCnt = %eval(&nIcErrCnt + %sysfunc(getvarn(&dsID, &nIcErrFld))) ;
      %end;
      %if %sysfunc(getvarn(&dsID, &nNonIcErrFld)) ne . %then %do;
        %let nNonIcErrCnt = %eval(&nNonIcErrCnt + %sysfunc(getvarn(&dsID, &nNonIcErrFld))) ;
      %end;

    %end;

    %if &nStatusID eq 2 %then %do ;
      %sesFMS_createErrorReport( lCycleDimTypes, &cInvalidDS, &nInvalidCnt,
                                      &cOrgEqTraderDS, &nOrgEqTraderCnt, &cIcErrDS, &nIcErrCnt,
                                      &cNonIcErrDS, &nNonIcErrCnt, &cDataLoadId );

      %if &_FMErrorCode EQ 0 %then %do;
        %if &dsID %then %do;
           %let dsID = %sysfunc(close(&dsID));
           %goto RETURNZERO;
        %end ;
      %end ;
    %end;

    %if &dsID %then
      %let dsID = %sysfunc(close(&dsID));
  %end ;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;
  %THEEND:

  /*  End the report output  */
  ods html close ;
  %STPEND;

%mend sesFMS_generateErrorReports;


/**********************************************************
 * sesFMS_getAccountTypes.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_getAccountTypes.sas                                                     */
/*  Purpose: Build a temporary data set of account types.                                    */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_getAccountTypes(cAccountDS, cSASLib, nAccountDimId, nAcctTypePropertyId, cDDSLib, nNowDTTM);

  %let cMethodNm = sesFMS_getAccountTypes;
  %let cTempDS1 = work._fmsetlTemp01;
  %let cTempDS2 = work._fmsetlTemp02;
  %let nowDTTM =    %qsysfunc(datetime());
  %let nTrackTime = %qsysfunc(datetime());

  /*  Clean up  */
  %if %sysfunc(exist(&cTempDS1)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cTempDS1, 2, '.');
    quit;
  %end;
  %if %sysfunc(exist(&cTempDS2)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cTempDS2, 2, '.');
    quit;
  %end;
  %if %sysfunc(exist(&cAccountDS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cAccountDS, 2, '.');
    quit;
  %end;


  /*  Get a list of accounts  */
  proc sql noprint ;
    create table &cTempDS1 as
        select distinct SAS_MEMBER.MEMBER_CD as CODE, SAS_MEMBER_PROPERTY.PROPERTY_VALUE as VALUE , SAS_MEMBER_PROPERTY.PROPERTY_ID
            from &cSASlib..SAS_MEMBER, &cSASlib..SAS_DIMENSION, &cSASlib..SAS_MEMBER_PROPERTY
           where SAS_MEMBER.INTERNAL_DIMENSION_ID=SAS_DIMENSION.INTERNAL_DIMENSION_ID and
                 SAS_MEMBER.MEMBER_ID=SAS_MEMBER_PROPERTY.MEMBER_ID  and
           SAS_DIMENSION.DIMENSION_ID=&nAccountDimId and
                 SAS_MEMBER_PROPERTY.PROPERTY_ID= &nAcctTypePropertyId and
                 (SAS_MEMBER.VALID_FROM_DTTM < &nNowDTTM and
                 SAS_MEMBER.VALID_TO_DTTM > &nNowDTTM) and
                 (SAS_MEMBER_PROPERTY.VALID_FROM_DTTM < &nNowDTTM and
                 SAS_MEMBER_PROPERTY.VALID_TO_DTTM > &nNowDTTM) ;
  quit;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  proc sort in=&cTempDS1 ;
       by code ;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  proc sql noprint ;
    create table &cTempDS2 as
          select distinct GL_ACCOUNT_ID as code, GL_ACCOUNT_RK
                from &cDDSLib..GL_ACCOUNT;
  quit ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  proc sort in=&cTempDS2 ;
       by CODE;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  data &cAccountDS(rename=(GL_ACCOUNT_RK=CODE)) ;
       drop CODE ;
       merge &cTempDS1(in=a) &cTempDS2(in=b) ;
       by CODE ;
       if a then output ;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;


  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );

  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_getAccountTypes;


/**********************************************************
 * sesFMS_getAllDDSRKsFmt.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_getAllDDSRKsFmt.sas                                                     */
/*  Purpose: Get all DDS RKs and then create format                                          */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_getAllDDSRKsFmt(cFactDimMbr, cDDSLib, cDimDimTypeDS);
  /* cFactDimMbr : (Input)should be a subset of sas_etl_fact_dim_mbr by ETL_instanceID*/

  %let cMethodNm = sesFMS_getAllDDSRKs ;
  %let nowDTTM =    %qsysfunc(datetime());
  %let nTrackTime = %qsysfunc(datetime());

  /* get all MEMBER_CD(dimension type) from X and write them into macros */
  %let num_data = 0;
  proc sql noprint;
    select count(*)
      into :num_data
      from &cDimDimTypeDS a
      where exists( select b.dimension_cd
                      from &cFactDimMbr b
                      where a.dim_CD = b.dimension_cd
                  );
      ;
  quit;
  %let num_data = %eval(&num_data);
  %if &num_data <= 0 %then %do;
     %let error_number = 56;
     %let errObj=&cFactDimMbr;
     %sesSOL_logError( YTDConv, L, &error_number, &errObj, &cMethodNm);
     %goto RETURNZERO;
  %end;

  /* Check if active record exists */
  /* Clean up */
  %if %sysfunc(exist(dimValError)) %then  %do;
    proc datasets library=work nolist;
      delete dimValError;
    quit;
  %end;

  %macro dimValidateError(dim_cd, table_nm, Col, cFactDimMbr);

  proc sql noprint;
    create table dimValError as
    select *
    from &cFactDimMbr a
    where a.dimension_cd eq "&dim_cd"
      and not exists( select b.&Col
                      from &table_nm b
                      where a.member_CD = b.&Col
                        and valid_from_dttm le &fmsetl_processeddttm_to
                        and valid_to_dttm ge &fmsetl_processeddttm_to
               );
  quit;


  %mend;

  proc sql noprint;
    select d.dim_cd, t.table_nm,
      case
         when strip(t.KEY_COLUMN_NM) = 'MEMBER_RK' then 'MEMBER_ID'
         else t.BUSINESS_ID_COLUMN_NM
      end,
      t.key_column_nm
      into :dimCD1-:dimCD&num_data, :dimTblNm1-:dimTblNm&num_data, :idCol1-:idCol&num_data, :keyCol1-:keyCol&num_data
      from &cdimDimTypeDS d,
           &cDDSLib..dimension_type t

      where d.dim_type_cd=t.dimension_type_cd
       and (t.VALID_FROM_DTTM < &fmsetl_processedDTTM_To and
            t.VALID_TO_DTTM > &fmsetl_processedDTTM_To)
       and exists( select b.dimension_cd
                      from &cFactDimMbr b
                      where d.dim_CD = b.dimension_cd
                  )
     ;
  quit;

  proc sql noprint;
    create table dimVal (
        FMTNAME varchar(64),
        LABEL   varchar(256),
        START   varchar(256),
        TYPE    varchar(1),
        HLO     varchar(1)
     );
  quit;

  %do i=1 %to &num_data;   /* # unique dimensions (from dimdimtype) */
      /* Check if active records */
      %dimValidateError( &&dimCD&i, &cDDSLib..&&dimTblNm&i, &&idCol&i, &cFactDimMbr);

      /* Check dimValError is empty */
      proc sql noprint;
          select count(*), dimension_cd, member_cd
          into :num_error, :_dim_cd, :_mem_cd
          from dimValError;
      quit;
      %let num_error = %eval(&num_error);

      %if &num_error > 0 %then %do;
          %let error_number = 57;
          %let errObj1=&_mem_cd;
          %let errObj2=&_dim_cd;
          %goto ERRORRETURN;
      %end;

      /* moving dim's fmtname, code and value */
      proc sql noprint;
          insert into dimVal
          select strip(d.dim_type_cd) || "_FMT", trim(a.member_cd),
          %if %index(&&keyCol&i, _RK) %then %do;
            strip(put(b.&&keyCol&i, best32.)), 'N', ''
          %end;
          %else %do;
            b.&&keyCol&i, 'C', ''
          %end;

          from &cFactDimMbr a, &cDDSLib..&&dimTblNm&i b, &cdimDimTypeDS d
          where a.dimension_cd= "&&dimCD&i"
             and b.&&idCol&i = a.member_cd
             and b.VALID_FROM_DTTM <= &fmsetl_processedDTTM_To
             and b.VALID_TO_DTTM >= &fmsetl_processeddttm_to
             and d.dim_cd = a.dimension_cd
          ;
      quit;
  %end;

  /* Insert Other value (*) for each format */
  proc sort data=dimVal;
    by fmtname start;
  run;
  data dimVal;
    set dimVal;
    by fmtname start;
    output;
    if last.fmtname then do;
      HLO = 'O';
      label = '*';
      output;
    end;
  run;

  /* create formats out of one big table */
  %sesFMS_createFormatNew(dimVal, dimVal_fmt, 1, ldimVal, 0 );

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesSOL_logError( YTDConv, L, &error_number, &errObj1, &errObj2, &cMethodNm);

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_getAllDDSRKsFmt;


/**********************************************************
 * sesFMS_getAllDimFormat.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_getAllDimFormat.sas                                                     */
/*  Purpose: Create all formats of all dimensions                                            */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_getAllDimFormat(cFactDimMbr);

  %let cMethodNm = sesFMS_getAllDimFormat ;
  %let nowDTTM =    %qsysfunc(datetime());
  %let nTrackTime = %qsysfunc(datetime());


  /* get all MEMBER_CD(dimension type) from X and write them into macros */
  %let num_data = 0;
  proc sql noprint;
    select count(*)
      into :num_data
      from &cFactDimMbr;
  quit;
  %let num_data = %eval(&num_data);

  %if &num_data > 0 %then %do;
    /* get dim type cd, rk, table name, key col name in DDS */
    proc sql noprint;
      select d.member_cd, d.dimension_id, t.dimension_type_rk, t.table_nm, t.business_id_column_nm, t.key_column_nm
        into :mbrCD1-:mbrCD&num_data, :dimID1-:dimID&num_data, :RKDS1-:RKDS&num_data, :dimTblNm1-:dimTblNm&num_data, :idCol1-:idCol&num_data, :keyCol1-:keyCol&num_data
      from &cFactDimMbr d, &cDDSLib..dimension_type t
     where d.dimension_cd = t.dimension_type_cd
         and (t.VALID_FROM_DTTM < &fmsetl_processedDTTM_To and
              t.VALID_TO_DTTM > &fmsetl_processedDTTM_To);
    quit;

    /* For each Dimension Type */
    /* Call sesFMS_getDDSRKs */
    /* Time and IntOrg are handled differently */
    %do i=1 %to &num_data;
        %let dimTypeCd =;
        proc sql noprint;
             select t.dim_type_cd
               into :dimTypeCd
               from &cSDMLib..sas_dimension d, &cSDMLib..sas_dim_type t
              where &&dimID&i= d.dimension_id
                and d.dim_type_id = t.dim_type_id
                and (VALID_FROM_DTTM < &fmsetl_processedDTTM_To and
                     VALID_TO_DTTM > &fmsetl_processedDTTM_To);
        quit;
        /* Define output tables, one for success and another for failure */
        %let cRKsDS       = WORK._&dimTypeCd;
        %let cRKsDS_error = WORK._&dimTypeCd_error;

        %if &&dimTypeCD&i EQ TIME %then %do;
             /* Current Time, Prior Time,
              Map between DDS Time Mbr RK and Run YTDtoPA, and
              Build Map between DDS Time Mbr RK and IsFirstPrd
            */
        %end;
        %else %if &&dimTypeCD&i EQ ANALYSIS %then %do;
          /* Do nothing */
        %end;
        %else %if &&dimTypeCD&i EQ INTORG %then %do;
            /* Do nothing */
        %end;
        %else %do;
            %sesFMS_getDDSRKs (&&mbrCD&i, &cDDSLib, &&dimTblNm&i, &&idCol&i, &&keyCol&i, &cRKsDS, &cRKsDS_error);
            %if &_FMErrorCode = 0 %then %do;
                %let error_number = 58;
                %let errObj=&&mbrCD&i.;
                %goto ERRORRETURN;
            %end ;
        %end;
    %end;
  %end;
  %else %do;
     %let error_number = 56;
     %let errObj=&cFactDimMbr;
     %goto ERRORRETURN;
  %end;


  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );

  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesSOL_logError( YTDConv, L, &error_number, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_getAllDimFormat;


/**********************************************************
 * sesFMS_getAnalysisDDSRKs.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_getAnalysisDDSRKs.sas                                                   */
/*  Purpose: Build a temporary data set of analysis rks from the DDS.                        */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_getAnalysisDDSRKs(cAnalysisCodesDS, cAnalysisRKsDS, cDDSLib);

  %let cMethodNm = sesFMS_getAnalysisDDSRKs ;
  %let cTempDS1 = work._fmsetlTemp01 ;
  %let cTempDS2 = work._fmsetlTemp02 ;
  %let nowDTTM =    %qsysfunc(datetime());
  %let nTrackTime = %qsysfunc(datetime());

  /*  Clean up  */

  %if %sysfunc(exist(&cTempDS1)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cTempDS1, 2, '.');
    quit;
  %end;
  %if %sysfunc(exist(&cTempDS2)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cTempDS2, 2, '.');
    quit;
  %end;
  %if %sysfunc(exist(&cAnalysisRKsDS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cAnalysisRKsDS, 2, '.');
    quit;
  %end;

  proc sort in=&cAnalysisCodesDS ;
           by CODE;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  proc sql noprint ;
    create table &cTempDS1 as
    select distinct ANALYSIS_ID as CODE, ANALYSIS_RK
        from &cDDSLib..ANALYSIS
     order by CODE;
  quit ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  data &cAnalysisRKsDS(rename=(ANALYSIS_RK=CODE) drop=CODE)
               &cTempDS2(keep=CODE) ;
            merge &cAnalysisCodesDS(in=a) &cTempDS1(in=b) ;
            by CODE ;
            if a and not b then do;
              /*  Error, mbr code exists in SDM, but not the DDS  */
              output &cTempDS2;
            end;
            else if a then do ;
              VALUE = CODE ;
              output &cAnalysisRKsDS;
            end ;
          run ;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  /*  Check if any errors where found  */
  %if %sysfunc(exist(&cTempDS2)) %then %do;
    proc sql noprint;
      select count(*)
        into :numrow
        from &cTempDS2;
        quit;
    %let numrow=%eval(&numrow);
    %if &numrow > 0 %then %do;
      proc sql noprint;
        select  code
          into :cCode1 - :cCode&numrow
          from &cTempDS2;
      quit;
      %do ii=1 %to &numrow;
        %sesSOL_logError(YTDConv, L, 54, &&cCode&ii, &cMethodNm);
      %end;
      %goto RETURNZERO;
    %end;
  %end;
  %else %do;
    %let error_number=52;
    %let errObj=&cTempDS2;
    %goto ERRORRETURN;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesSOL_logError( YTDConv, L, &error_number, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_getAnalysisDDSRKs;


/**********************************************************
 * sesFMS_getDDSDimensionTypes.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_getDDSDimensionTypes.sas                                                */
/*  Purpose: Get active records from DDS Dimension_type table and load into work area        */
/*           data set.                                                                       */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_getDDSDimensionTypes(cDDSLibRef, cDDSTableNm, cWorkAreaDS, cRKtoCodeLinkDS,
                                                              cDimTypeRKColDS, cBFColNm);

  %let cMethodNm = sesFMS_getDDSDimensionTypes;
  %let cDDSLibRef = %qsysfunc(strip(&cDDSLibRef));
  %let cDDSTableNm = %qsysfunc(strip(&cDDSTableNm));
  %let cDDSDimTypeDS =&cDDSLibRef..&cDDSTableNm;
  %if &cBFColNm eq %str() %then %do;
    %let cBFColNm = BASE_FACT_COLUMN_NM;
  %end;

  %let cKeyColNm = KEY_COLUMN_NM ;
  %let nTrackTime = %sysfunc(datetime());

  /*  Make sure DDS Dimension Type table exists  */
  %if %sysfunc(exist(&cDDSDimTypeDS)) EQ 0 %then %do;
      %let error_number=6;
      %let errObj=&cDDSDimTypeDS;
        %goto ERRORRETURN;
  %end ;

  /*  Get a list of the active records and subset down to 2 columns of information (code, value)  */
  data &cWorkAreaDS (keep=CODE VALUE);
          length CODE $64 ;
          length VALUE $64 ;
          set &cDDSDimTypeDS ;
          where VALID_TO_DTTM GT datetime() and &cBFColNm NE "" and &cKeyColNm NE "" ;
          CODE = DIMENSION_TYPE_CD;
          VALUE = &cBFColNm;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
      %let error_number=7;
      %let errObj=&cDDSDimTypeDS;
        %goto ERRORRETURN;
  %end ;

  /*
   *  Because the DDS model supports multiple languages for name/description, we could have
   *  duplicate dimension_type_cd values.  Run a proc sort nodupkey to get rid of the dupes.
   */
  proc sort in=&cWorkAreaDS out=&cWorkAreaDS nodupkey ;
          by CODE;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
      %sesSOL_logError( YTDConv, L, 8,  &cMethodNm);
      %goto RETURNZERO;
  %end ;

  /*
   *  Build a table of Dimension_type_cd to table_nm
   */

  proc sql noprint ;
      create table &cRKtoCodeLinkDS as
    select distinct DIMENSION_TYPE_CD as CODE, TABLE_NM as VALUE
                 from &cDDSDimTypeDS
                where &cBFColNm NE ""
                  and KEY_COLUMN_NM NE ""
          and (VALID_FROM_DTTM < &fmsetl_processedDTTM_To and
                             VALID_TO_DTTM > &fmsetl_processedDTTM_To);
  quit ;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
      %let error_number=9;
      %let errObj=&cRKtoCodeLinkDS;
        %goto ERRORRETURN;
  %end;


  /*
   *  Build a table of Dimension_type_cd to Key_Column_Nm
   */

  proc sql noprint ;
    create table &cDimTypeRKColDS as
              select distinct DIMENSION_TYPE_CD as CODE, KEY_COLUMN_NM as VALUE
                from &cDDSDimTypeDS
                where &cBFColNm NE ""
                  and KEY_COLUMN_NM NE ""
            and (VALID_FROM_DTTM < &fmsetl_processedDTTM_To and
                             VALID_TO_DTTM > &fmsetl_processedDTTM_To);
  quit;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
      %let error_number=9;
      %let errObj=&cDimTypeRKColDS;
        %goto ERRORRETURN;
  %end;


  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );

  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesSOL_logError( YTDConv, L, &error_number, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_getDDSDimensionTypes;


/**********************************************************
 * sesFMS_getDDSRKs.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_getDDSRKs.sas                                                           */
/*  Purpose: Build a temporary data set of RKs from the DDS                                  */
/*  Usage  : cCodes :  (Input)  Dim member code such as ACTUAL, BUDGET of ANALYSIS           */
/*           cRKsDS  :  (Output) (temp) table to store RKs                                   */
/*           cRKsDS_error : (output) error table if cCode is not found in DDS                */
/*                                                                                           */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */
%macro sesFMS_getDDSRKs (cCode, cDDSLib, dimTblNm, idCol, keyCol, cRKsDS, cRKsDS_error);

  %let cMethodNm = sesFMS_getDDSRKs ;
  %let nowDTTM =    %qsysfunc(datetime());
  %let nTrackTime = %qsysfunc(datetime());

  /* select RKs from DDS table */
  proc sql noprint;
    select &keyCol
      into :_temp_value
      from &cDDSLib..&dimTblNm a
     where a.&idCol = "&cCode";
  quit;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  /* Insert result in the target table */
  %if &_temp_value > 0 %then %do;
      %if not (%qsysfunc(exist( &cRKsDS) ))%then %do;
          /* create a table if not exists */
        proc sql noprint;
              create table  &cRKsDS (
                value varchar(256),
                  code integer);
          quit;
    %end;
      /* write result in target table */
      proc sql noprint;
         insert into &cRKsDS values ( "&cCode", &_temp_value);
      quit;
  %end;
  %else %do;
      /* Error, mbr code exists in SDM, but not the DDS  */
      /* error table that would contain dimension types not found in DDS */
      %if not %qsysfunc( exist(&temp_errorDS)) %then %do;
          proc sql noprint;
              create table temp_errorDS (code varchar(256));
          quit;
      %end;

      %if %sysfunc(exist(&cRKsDS_error)) %then %do;
          proc sql noprint;
              insert into &cRKsDS_error values ("&cCode");
          quit;
      %end;
    %sesSOL_logError(YTDConv, L, 54, &cCode , &cMethodNm);
  %end;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );

  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_getDDSRKs;


/**********************************************************
 * sesFMS_getDimMbrSubset.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_getDimMbrSubset.sas                                                     */
/*  Purpose: subsets ETL temp fact dim member table by given ETL instance ID                 */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_getDimMbrSubset(ETL_instanceID, TempFactDimMbr, subTempFactDimMbr);
  /* ETL_InstanceID : (Input) ETL instance ID from mid-tier parameter */
  /* TempFactDimMbr : (Input) table name includes all dimension type information */
  /* subTempFActDimMbr : (output) table name subsetting TempFactDimMbr by ETL_instanceID */

  %let cMethodNm = sesFMS_getDimMbrSubset;
  %let nTrackTime = %qsysfunc(datetime());

  %if %sysfunc(exist(&TempFactDimMbr)) %then %do;
      /* Subsetting by ETL_instanceID */
      data &subTempFactDimMbr;
        set &TempFactDimMbr(where=(ETL_INSTANCE_ID = &ETL_instanceID));
      run;
  %end;
  %else %do;
      %let error_number = 52;
      %let errObj=&TempFactDimMbr;
      %goto ERRORRETURN;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesSOL_logError( YTDConv, L, &error_number, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_getDimMbrSubset;


/**********************************************************
 * sesFMS_getJournsFromDDS.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_getJournsFromDDS.sas                                                    */
/*  Purpose: Get a subset of Journal trans. from the DDS and dump into a work area data set. */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */
%macro sesFMS_getJournsFromDDS(cDDSLibRef,
                                 nFrom_DTTM,
                                 nTo_DTTM,
                                 cCurrTimeMbr_Fmt,
                                 cAnalysisFormat,
                                 cCurrencyFormat,
                                 cOutputJrnlDS,
                                 cAccountFormat,
                                 lKeepCols,
                                 cDDSJrnlDS,
                                 cDDSJrnlDetailDS,
                                 cSourceColNm,
                                 cSourceCode,
                                 cSubSetOrg_Fmt,
                                 nIsSubSettingOrg
                                 );

  %let cMethodNm = sesFMS_getJournsFromDDS;
  %let cDDSLibRef = %qsysfunc(strip(&cDDSLibRef));
  %let cDDSJrnlDS = %qsysfunc(strip(&cDDSJrnlDS));
  %let cInputDS =&cDDSLibRef..&cDDSJrnlDS;
  %let cDDSJrnlDetailDS = %qsysfunc(strip(&cDDSJrnlDetailDS));
  %let cInputDetailDS =&cDDSLibRef..&cDDSJrnlDetailDS;
  %let cDDSDimTypeTableNm =  &cDDSLibRef..dimension_type;
  %let nTrackTime = %qsysfunc(datetime());
  %let nJrnlCnt = 0;

  /*  Clean up, in case we had a previously aborted attempt  */
  %if %sysfunc(exist(&cOutputJrnlDS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cOutputJrnlDS, 2, '.');
    quit;
  %end;

  /*  Make sure we have a source transaction table in the DDS  */
  %if not %sysfunc(exist(&cInputDS, DATA))%then %do;
    %let error_number=37;
    %let errObj=&cInputDS;
    %goto ERRORRETURN;
  %end ;
  %if not %sysfunc(exist(&cInputDetailDS, DATA))%then %do;
    %let error_number=37;
    %let errObj=&cInputDetailDS;
    %goto ERRORRETURN;
  %end ;

  /*------------------------------------------------*/
  /*  Build the subsetted transaction output table  */
  /*------------------------------------------------*/

  /* Statements for All dimensional operations should be written down on a file*/
  /*
  proc sql noprint;
     create table _dim_type_rk_col as
     select dimension_type_cd as dim_type, base_fact_column_nm  as dim_rk_col
       from &lKeepCols a, &cDDSDimTypeTableNm b
      where a.code = b.dimension_type_cd
        and a.code NE 'TIME'
        and (VALID_FROM_DTTM < &fmsetl_processedDTTM_To and
             VALID_TO_DTTM > &fmsetl_processedDTTM_To);
  quit;
  */

  /* Write a dynamic SQL statement in a file */
  filename _sqlstmt temp;
  data _null_;
      file _sqlstmt;
      set _dim_type_rk_col end=eof;

    if _N_ = 1 then do;
          put "create table &cOutputJrnlDS as ";
          put "select &cDDSJrnlDS..AFFECTED_TIME_PERIOD_RK, &cDDSJrnlDetailDS..*";
          put "  from &cInputDS, &cInputDetailDS";
          put " where &cDDSJrnlDS..GL_JRNL_ID = &cDDSJrnlDetailDS..GL_JRNL_ID and";
          put "       put( &cDDSJrnlDS..AFFECTED_TIME_PERIOD_RK, &cCurrTimeMbr_Fmt..) NE ""*"" and";
      end;
      dim_type_fmt = strip(dim_type) !! "_FMT.";
      dim_rk_col_nm   = strip(dim_rk_col);

      if index(dim_rk_col_nm, '_CD') then
         dim_type_fmt = '$' !! strip(dim_type_fmt);

      put "       put(&cDDSJrnlDetailDS.." dim_rk_col_nm ", " dim_type_fmt ") NE ""*"" and" ;

      if eof then do;
          put "       &cDDSJrnlDetailDS..TRANSACTION_AMT not in(.) and ";
          put "       ( &cDDSJrnlDS..PROCESSED_DTTM GT &nFrom_DTTM and ";
          put "          &cDDSJrnlDS..PROCESSED_DTTM LE &nTo_DTTM );";
      end;

      /* example of output :
      put '       put( &cDDSJrnlDetailDS..ANALYSIS_RK, &cAnalysisFormat..) NE "*" and
      put '       put( &cDDSJrnlDetailDS..CURRENCY_CD, &cCurrencyFormat..) NE "*" and
      put '       put( &cDDSJrnlDetailDS..GL_ACCOUNT_RK, &cAccountFormat..) NE "*" and
      */
  run;

  /* run the dynamic SQL statement */
  proc sql noprint;
      %inc _sqlstmt;
  quit;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %sesSOL_logError( YTDConv, L, 38, &cInputDS, &cInputDetailDS, &cMethodNm);
    %goto ERRORRETURN;
  %end;

  /*  Did we find any Journals  */
  %let fmsetl_tempCheck1 = 0;
  proc sql noprint ;
    select count(*)
        into :fmsetl_tempCheck1
        from &cOutputJrnlDS ;
  quit ;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
       %sesSOL_logError( YTDConv, L, 39, &cMethodNm) ;
    %goto ERRORRETURN;
  %end;

  %let nJrnlCnt = %eval(&fmsetl_tempCheck1);
  %if &nJrnlCnt eq 0 %then %let nJrnlCnt = 0 ;
  %if &nJrnlCnt EQ 0 %then %do ;
        /*  No journals found, skip the rest  */
        %goto THEEND;
  %end ;


  /*
   *  Does the resulting Journal table have the required Cycle Dimension Type columns
   */

  /* put col names of cOutputJrnlDS into _jrnlTranCols */
  %let libNm=%kscan(&cOutputJrnlDS, 1, '.');
  %let tblNm=%kscan(&cOutputJrnlDS, 2, '.');

  proc sql noprint;
    create table _jrnlTranCols as
    select upcase(name) as name
      from dictionary.columns
     where libname="&libNm" and memname="&tblNm";
  quit;

  /*
   *  Keep just the columns we need
   */

  proc sort in=&lKeepCols out=&lKeepCols;
    by value;
  run;

  proc sql noprint;
    create table _temp_diff as
     /* lKeepCol-_jrnlTranCols */
     select value
       from &lKeepCols
     except
     select name
       from _jrnlTranCols;
  quit;

  %let num_data=0;
  proc sql noprint;
    select count(*)
      into :num_data
      from _temp_diff;
  quit;
  %let num_data=%eval(&num_data);


  %if &num_data > 0 %then %do;
    proc sql noprint;
  /*    select code */
      select value
        into :_col1 - :_col&num_data
        from _temp_diff;
    quit;

    %do ii=1 %to &num_data;
      %sesSOL_logError(YTDConv, L, 40, &cInputDS, &cInputDetailDS, &&_col&ii, &cMethodNm) ;
    %end;
    %goto RETURNZERO;
  %end;

  /* move all column names into variables, col1 to colxx */
  filename _col temp;
  data _null_;
      file _col;
    set &lKeepCols end=eof;
    put '"' value + (-1) '"n ';
  run;

  data &cOutputJrnlDS  ;
  keep %inc _col;
           TRANSACTION_AMT
           TRANSACTION_DT
           &cSourceColNm;
          length "SOURCE_CD"n $32 ;
          set &cOutputJrnlDS ;
          &cSourceColNm = "&cSourceCode" ;
  run ;

  %goto THEEND;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %let error_number=11;
    %let errObj=&cInputDS;
    %goto ERRORRETURN;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesSOL_logError( YTDConv, L, &error_number, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_getJournsFromDDS;


/**********************************************************
 * sesFMS_getPriorTransFromDDS.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_getPriorTransFromDDS.sas                                                */
/*  Purpose:                                                                                 */
/*  Get a subset of the prior period GL transactions from the DDS and dump into a work area  */
/*  data set. We only want those transactions where the YTD amount flag is true.             */
/*                                                                                           */
/*  Subset by:                                                                               */
/*     > prior time period                                                                   */
/*     > transaction_amt_ytd_flg eq "Y"                                                      */
/*     > specific analysis RK                                                                */
/*     > currency hierarchy leaf nodes                                                       */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */
%macro sesFMS_getPriorTransFromDDS(cDDSLibRef,
                                     cDDSTransTableNm,
                                     cPriorTimeMbr_Fmt,
                                     cAnalysisFormat,
                                     cCurrencyFormat,
                                     cOutputYtdDS,
                                     cNewAmtColNm,
                                     cPeriod_Fmt,
                                     cTempPrdColNm);

  %let cMethodNm = sesFMS_getPriorTransFromDDS;
  %let cDDSLibRef = %qsysfunc(strip(&cDDSLibRef));
  %let cDDSTransTableNm = %qsysfunc(strip(&cDDSTransTableNm));
  %let cInputDS =&cDDSLibRef..&cDDSTransTableNm;
  %let nTrackTime = %qsysfunc(datetime());

  /*  Clean up, in case we had a previously aborted attempt  */
  %if %sysfunc(exist(&cOutputYtdDS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cOutputYtdDS, 2, '.');
    quit;
  %end;

  /*  Make sure we have a source transaction table in the DDS  */
  %if not %sysfunc(exist(&cInputDS, DATA)) and not %sysfunc(exist(&cInputDS, VIEW)) %then %do;
    %let error_number=10;
    %let errObj=&cInputDS;
    %goto ERRORRETURN;
  %end ;

  data &cOutputYtdDS (rename=(TRANSACTION_AMT=&cNewAmtColNm)) ;
    length "&cTempPrdColNm"n 8;
    set &cInputDS ;
      if put(AFFECTED_TIME_PERIOD_RK, &cPriorTimeMbr_Fmt..) NE "*" ;
      if upcase(TRANSACTION_AMT_YTD_FLG) eq "Y" ;
      %inc _fstmt;
      if TRANSACTION_AMT NE . ;
      &cTempPrdColNm = put(AFFECTED_TIME_PERIOD_RK, &cPeriod_Fmt..);
  run ;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %let error_number=11;
    %let errObj=&cInputDS;
    %goto ERRORRETURN;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesSOL_logError( YTDConv, L, &error_number, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_getPriorTransFromDDS;


/**********************************************************
 * sesFMS_getTimeDDSRKs.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_getTimeDDSRKs.sas                                                       */
/*  Purpose: Build a temporary data set of TIME_PERIOD rks from the DDS.                     */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_getTimeDDSRKs(cTimeMbrCodesDS, cTimeMbr_DDSRKs_DS, cTimeMbr_DDSRKs_Map_PrdId_DS, cDDSLib);

  %let cMethodNm = sesFMS_getTimeDDSRKs ;
  %let  cTempDS1 = work._fmsetlTemp01 ;
  %let  cTempDS2 = work._fmsetlTemp02 ;
  %let  cTempDS3 = work._fmsetlTemp03 ;

  %let  nowDTTM = %qsysfunc(datetime()) ;
  %let  nTrackTime = %qsysfunc(datetime()) ;


  /*  Clean up  */
  %if %sysfunc(exist(&cTempDS1)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cTempDS1, 2, '.');
    quit;
  %end;
  %if %sysfunc(exist(&cTempDS2)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cTempDS2, 2, '.');
    quit;
  %end;
  %if %sysfunc(exist(&cTempDS3)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cTempDS3, 2, '.');
    quit;
  %end;
  %if %sysfunc(exist(&cTimeMbr_DDSRKs_DS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cTimeMbr_DDSRKs_DS, 2, '.');
    quit;
  %end;

  /*
   *  It's possible for Prior Time Mbrs to be blank.  We'll exclude them from the list
   */

  data &cTempDS3(drop=VALUE);
      length PERIOD_ID 8;
      set &cTimeMbrCodesDS;
      if strip(VALUE) NE "" ;
      PERIOD_ID = input(strip(CODE), 32.);
      CODE = VALUE;
  run ;

  proc sort in=&cTempDS3;
       by CODE;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;


  proc sql noprint ;
    create table &cTempDS1 as
      select distinct TIME_PERIOD_ID as CODE, TIME_PERIOD_RK
        from &cDDSLib..TIME_PERIOD
     order by CODE;
  quit ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;


  /**************************************
   *  Match Time Prd DDS RKs to Codes
   **************************************/
  data &cTimeMbr_DDSRKs_DS(rename=(TIME_PERIOD_RK=CODE) drop=CODE PERIOD_ID)
               &cTempDS2(keep=CODE) ;
        merge &cTempDS3(in=a) &cTempDS1(in=b) ;
        by CODE ;
        if a and not b then do;
            /*  Error, mbr code exists in SDM, but not the DDS  */
            output &cTempDS2;
        end;
        else if a then do ;
            VALUE = CODE ;
            output &cTimeMbr_DDSRKs_DS;
        end ;
  run ;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;


  /*  Check if any errors where found  */
  %if %sysfunc(exist(&cTempDS2)) %then %do;
    proc sql noprint;
      select count(*)
        into :numrow
        from &cTempDS2;
        quit;
    %let numrow=%eval(&numrow);
    %if &numrow %then %do;
      proc sql noprint;
        select  code
            into :cCode1 - :cCode&numrow
            from &cTempDS2;
      quit;

      %do ii=1 %to &numrow;
        %sesSOL_logError(YTDConv, L, 53, &&cCode&ii, cMethodNm);
      %end;
      %goto RETURNZERO;
    %end;
  %end;
  %else %do;
    %let error_number=52;
    %let errObj=&cTempDS2;
    %goto ERRORRETURN;
  %end;

  /******************************************
   *  Match Time Mbr DDS RKs to Period Ids
   ******************************************/
  data &cTimeMbr_DDSRKs_Map_PrdId_DS(rename=(TIME_PERIOD_RK=CODE) drop=CODE PERIOD_ID)
       &cTempDS2(keep=CODE) ;
    length VALUE 8;
    merge &cTempDS3(in=a) &cTempDS1(in=b) ;
      by CODE ;
    if a and not b then do;
    /*  Error, mbr code exists in SDM, but not the DDS  */
      output &cTempDS2;
    end;
    else if a then do ;
      VALUE = PERIOD_ID;
      output &cTimeMbr_DDSRKs_Map_PrdId_DS;
    end ;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  /*  Check if any errors where found  */
  %if %sysfunc(exist(&cTempDS2)) %then %do;
    proc sql noprint;
      select count(*)
        into :numrow
        from &cTempDS2;
    quit;
    %let numrow=%eval(&numrow);
    %if &numrow > 0 %then %do;
      proc sql noprint;
        select  code
            into :cCode1 - :cCode&numrow
            from &cTempDS2;
      quit;

      %do ii=1 %to &numrow;
        %sesSOL_logError(YTDConv, L, 53, &&cCode&ii, cMethodNm);
      %end;
      %goto RETURNZERO;
    %end;
  %end;
  %else %do;
    %let error_number=52;
    %let errObj=&cTempDS2;
    %goto ERRORRETURN;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );

  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesSOL_logError( YTDConv, L, &error_number, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_getTimeDDSRKs;


/**********************************************************
 * sesFMS_getTimer.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_getTimer.sas                                                            */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */
%macro sesFMS_getTimer;
  %sysevalf(&nTimer);
%mend sesFMS_getTimer;


/**********************************************************
 * sesFMS_getTransFromDDS.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_getTransFromDDS.sas                                                    */
/*  Purpose: Get a subset of transaction from the DDS and dump into two work area data set.  */
/*           One data set with transactions flagged as YTD amounts, other flagged as either  */
/*           (Period activity amounts or Statisticala ccounts).                              */
/*  Subset by:                                                                               */
/*     > range of Process_dttm                                                               */
/*     > list of current time period(s)                                                      */
/*     > list of analysis(s)                                                                 */
/*     > currency hierarchy leaf nodes                                                       */
/*  Where the following is true, Transactions are dumped to a work area YTD data set:        */
/*     > transaction_amt_ytd_flg is 0 (false)                                                */
/*     > account type is statistical                                                         */
/*     > current period is the first period of the fiscal year and account type is I/S       */
/*  Otherwise transactions are dumped to a work area Period Activity data set                */
/*     (i.e. transactions are imported but not converted)                                    */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */
%macro sesFMS_getTransFromDDS(cDDSLibRef,
                                cDDSTransTableNm,
                                nFrom_DTTM,
                                nTo_DTTM,
                                cCurrTimeMbr_Fmt,
                                cAnalysisFormat,
                                cCurrencyFormat,
                                cOutputYtdDS,
                                cOutputPaDS,
                                cOutputYtdColBlankDS,
                                cAccountFormat,
                                cNewAmtColNm,
                                cIsFirstPrd_Fmt,
                                lKeepCols,
                                cAcctTypeRevenue,
                                cAcctTypeExpense,
                                cAcctTypeStatFlow,
                                cAcctTypeNonFrequency,
                                cRunYtdPA_Fmt,
                                cPeriod_Fmt,
                                cTempPrdColNm);

  %let cMethodNm = sesFMS_getCurrentTransFromDDS;
  %let cDDSLibRef = %qsysfunc(strip(&cDDSLibRef));
  %let cDDSTransTableNm = %qsysfunc(strip(&cDDSTransTableNm));
  %let cInputDS =&cDDSLibRef..&cDDSTransTableNm;
  %let cDDSDimTypeTableNm =  &cDDSLibRef..dimension_type;
  %let nTrackTime = %qsysfunc(datetime());

  /*  Clean up, in case we had a previously aborted attempt  */
  %if %sysfunc(exist(&cOutputYtdDS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cOutputYtdDS, 2, '.');
    quit;
  %end;
  %if %sysfunc(exist(&cOutputPaDS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cOutputPaDS, 2, '.');
    quit;
  %end;


  /*  Make sure we have a source transaction table in the DDS  */
  %if not %sysfunc(exist(&cInputDS, DATA)) and not %sysfunc(exist(&cInputDS, VIEW)) %then %do;
    %let error_number=10;
    %let errObj=&cInputDS;
    %goto ERRORRETURN;
  %end ;

  /*  Build the subsetted transaction output table  */
  /* extract all rows in the list, , into col_name */

  filename _keep temp;
  data _null_;
    file _keep;
    if _N_ = 1 then do;
       put '"&cTempPrdColNm"n "TRANSACTION_AMT"n  "TRANSACTION_DT"n ';
    end;
    set &lKeepCols end=eof;
    value = strip(value);

    put '"' value +(-1) '"n ' ;
    if eof then
      put ';';
  run;


  /* Statements for All dimensional operations should be written to a file */
  proc sql noprint;
    create table _dim_type_rk_col as
    select distinct(a.dimension_type_cd) as dim_type, base_fact_column_nm  as dim_rk_col
    from &cDDSDimTypeTableNm a
    where a.dimension_type_cd NE 'TIME'
      and a.valid_from_dttm le &fmsetl_processeddttm_to
      and a.valid_to_dttm ge &fmsetl_processeddttm_to
      and exists (select dimension_id
                  from &cDimDimTypeDS b, &csubTempFactDimMbr c
                  where b.dim_cd = c.dimension_cd
                    and a.dimension_type_cd = b.dim_type_cd
                  )
    ;
  quit;

  filename _fstmt temp;
  data _null_;
    file _fstmt;
    set _dim_type_rk_col end=eof;
    dim_type_fmt = strip(dim_type) !! "_FMT";
    dim_rk_col   = strip(dim_rk_col);

    if index(dim_rk_col, '_CD') then
      dim_type_fmt = '$' !! strip(dim_type_fmt);

    put 'if put("' dim_rk_col +(-1) '"n , ' dim_type_fmt +(-1) '.) NE "*";' ;
  run;

  data &cOutputYtdDS (rename=(TRANSACTION_AMT=&cNewAmtColNm))
       &cOutputPaDS(drop=&cTempPrdColNm)
       &cOutputYtdColBlankDS(drop=&cTempPrdColNm);
          keep
            %inc _keep;
       length "&cTempPrdColNm"n 8;
       set &cInputDS ;
     if put(AFFECTED_TIME_PERIOD_RK, &cCurrTimeMbr_Fmt..) NE "*" ;
       if PROCESSED_DTTM GT &nFrom_DTTM and PROCESSED_DTTM LE &nTo_DTTM ;
     %inc _fstmt;
       if TRANSACTION_AMT eq . then TRANSACTION_AMT = 0 ;

      &cTempPrdColNm = put(AFFECTED_TIME_PERIOD_RK, &cPeriod_Fmt..);

      /*  If RunYtdToPA = N, then dump all facts to cOutputPaDs  */
      if put(AFFECTED_TIME_PERIOD_RK, &cRunYtdPA_Fmt..) eq "N" then do;
        output &cOutputPaDs ;
      end;
      else do;
        if put(AFFECTED_TIME_PERIOD_RK, &cIsFirstPrd_Fmt..) eq "Y" then do;
            if (upcase(TRANSACTION_AMT_YTD_FLG) NE "Y") and (upcase(TRANSACTION_AMT_YTD_FLG) NE "N") then do;
              output &cOutputYtdColBlankDS;
            end;
            else if (upcase(TRANSACTION_AMT_YTD_FLG) eq "N" or
                 put(GL_ACCOUNT_RK, &cAccountFormat..) eq "&cAcctTypeStatFlow" or
                       put(GL_ACCOUNT_RK, &cAccountFormat..) eq "&cAcctTypeNonFrequency" or
                       put(GL_ACCOUNT_RK, &cAccountFormat..) eq "&cAcctTypeRevenue" or
                       put(GL_ACCOUNT_RK, &cAccountFormat..) eq "&cAcctTypeExpense" ) then do ;
                  output &cOutputPaDs ;
                end ;
                else output &cOutputYtdDs ;
        end;
        else do;
          if (upcase(TRANSACTION_AMT_YTD_FLG) NE "Y") and (upcase(TRANSACTION_AMT_YTD_FLG) NE "N") then do;
              output &cOutputYtdColBlankDS;
            end;
            else if (upcase(TRANSACTION_AMT_YTD_FLG) eq "N") or
                ( put(GL_ACCOUNT_RK, &cAccountFormat..) EQ "&cAcctTypeNonFrequency" ) then do ;
                output &cOutputPaDs ;
               end ;
              else output &cOutputYtdDs ;
        end;
      end;
  run ;

  %goto THEEND;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %let error_number=11;
    %let errObj=&cInputDS;
    %goto ERRORRETURN;
  %end;

  /*  Check for empty YTD Flag errors  */
  %let fmsetl_tempCheck1 = ;
  proc sql noprint ;
      select count(*)
        into :fmsetl_tempCheck1
        from &cOutputYtdColBlankDS ;
  quit ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %sesSOL_logError( YTDConv, L, 35, &cMethodNm) ;
    %goto RETURNZERO;
  %end;


  %if &fmsetl_tempCheck1 GT 0 %then %do;
    %let error_number=36;
    %let errObj=&fmsetl_tempCheck1;
    %goto ERRORRETURN;
  %end;


  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesSOL_logError( YTDConv, L, &error_number, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_getTransFromDDS;


/**********************************************************
 * sesFMS_imptdataWithoutTr.sas
 **********************************************************/

/* Copyright 2003 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved. */

%macro sesFMS_imptdataWithoutTr (gl_trans_sum_table=, update_tablename=,
                 cycle_dim_table=, fmsetl_processedDTTM_To=,
                 dataLoadObjTypeId=, dataLoadObjId=,
                 dimTypeWorkAreaDS= );

  %let fmsetl_error = ;
  %let fmsetl_errormsg = ;
  %let fmsetl_statusId = 1 ; /* Will be changed to 0 if program runs successful */
  %let format_missing = 0;

  %let entry = imptdata;
  %let putlog = Y;

  %let curr_rec_dttm = '01JAN5999:00:00'DT;

  /* options metarepository="&SlnRepos"; */

  %macro _chkError;
    %global _tmpRC;
    %let _tmpRC = 0;
    %if (&sysrc ne 0) or (&syserr ne 0 and &syserr ne 4) %then %do;
      %if (&sysrc ne 0) %then
          %let _tmpRC = &sysrc;
      %else
          %let _tmpRC = &syserr;
      %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=999
              , msgTxt1=&_tmpRC, msgTxt2=&sysmsg, msgPutlog=&putlog);
    %end;
  %mend _chkError;


  %macro mbrConcat ;
     %global fmsetl_concatMbrStmt ;
     %let fmsetl_concatMbrStmt= ;
     data _null_ ;
       length CONCATSTMT $32767 ;
       retain CONCATSTMT ;
       set &dimTypeWorkAreaDS end=nEnd ;
       if _n_ = 1 then do ;
         CONCATSTMT = strip(CONCATSTMT) ||
                      "strip(put(" || quote(strip(CODE)) || "n, 32.))"  ;
       end ;
       else do ;
         CONCATSTMT = strip(CONCATSTMT) || " || ',' || " ||
                      "strip(put(" || quote(strip(CODE)) || "n, 32.))"  ;
       end ;
       if nEnd then do ;
         call symput("fmsetl_concatMbrStmt", CONCATSTMT) ;
       end ;
     run ;
  %mend ;

  %mbrConcat ;

  /* Check if datetime passed in is valid */
  %if %datatyp(&fmsetl_processedDTTM_To) ne NUMERIC %then %do;
    %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=1,
              msgTxt1=<fmsetl_processedDTTM_To : &fmsetl_processedDTTM_To>, msgPutlog=&putlog);
    %goto exit;
  %end;


  /* Check if update_tablename macro variable is passed */

  %if (&update_tablename=.) %then %do;
     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=2,
              msgTxt1=<update_tablename : &update_tablename>, msgPutlog=&putlog);
     %goto exit;
  %end;

  /* Check if &cSDMLib..SAS_DIM_TYPE exists */
  %let _tmpDSN = &cSDMLib..SAS_DIM_TYPE;
  %if not %sysfunc( exist(&_tmpDSN) ) %then %do;
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=13
             , msgPutlog=N);
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=3
             , msgTxt1=dimension_type, msgTxt2=&sesgmsg_msgReturn
             , msgPutlog=&putlog);
     %goto exit ;
  %end;


  /* Check if &cSDMLib..SAS_DIMENSION exists */
  %let _tmpDSN = &cSDMLib..SAS_DIMENSION;
  %if not %sysfunc( exist(&_tmpDSN) ) %then %do;
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=14
             , msgPutlog=N);
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=3
             , msgTxt1=dimension, msgTxt2=&sesgmsg_msgReturn
             , msgPutlog=&putlog);
     %goto exit ;
  %end;

  /* Check for the existence of active records in the dimension table */
  proc sql noprint;
     select count(*) INTO :_rec_cnt
     from &_tmpDSN
     ;
  quit;
  %_chkError;
  %if &_tmpRC %then %goto exit ;

  %if (&_rec_cnt < 1) %then %do;
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=15
             , msgPutlog=N);
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=5
             , msgTxt1=dimension, msgTxt2=&sesgmsg_msgReturn
             , msgPutlog=&putlog);
     %goto exit ;
  %end;


  /* Check if cycle's dimension table exists */
  %let _tmpDSN = &cycle_dim_table;
  %if not %sysfunc( exist(&_tmpDSN) ) %then %do;
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=16
             , msgPutlog=N);
     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=7
            , msgTxt1=&_tmpDSN, msgTxt2=&sesgmsg_msgReturn
            , msgPutlog=&putlog);
     %goto exit;
  %end;

  proc sort data=&_tmpDSN;
     by CODE;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;


  /* Check if &cSDMLib..SAS_MEMBER exists */
  %let _tmpDSN = &cSDMLib..SAS_MEMBER;
  %if not %sysfunc( exist(&_tmpDSN) ) %then %do;
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=17
             , msgPutlog=N);
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=3
             , msgTxt1=member, msgTxt2=&sesgmsg_msgReturn
             , msgPutlog=&putlog);
     %goto exit ;
  %end ;

  /* Check for the existence of active records in the member table */
  proc sql noprint;
     select count(*) INTO :_rec_cnt
     from &_tmpDSN
     ;
  quit;
  %_chkError;
  %if &_tmpRC %then %goto exit ;

  %if (&_rec_cnt < 1) %then %do;
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=18
             , msgPutlog=N);
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=5
             , msgTxt1=dimension, msgTxt2=&sesgmsg_msgReturn
             , msgPutlog=&putlog);
     %goto exit ;
  %end;

  /*
   * build a list of the basefact dimension types in a macro var
   *
   * Name: bfdtypes & dtype_rks
   * Contents: list of dimensions and dimension types separated by commas
   * Example: TIME, INTORG, ANALYSIS, CURRENCY, ....
   *
   * Name: dimcnt
   * Contents: number of items in above lists
   */

  %let _tmpSortVar = DIMENSION_CD;
  data WORK._TMPALLDIMS;
     set &cycle_dim_table;
     &_tmpSortVar = CODE;
     keep &_tmpSortVar;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  proc sort data = WORK._TMPALLDIMS;
     by &_tmpSortVar;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  %let _tmpWhere = where=(VALID_FROM_DTTM <= &fmsetl_processedDTTM_To and
                          VALID_TO_DTTM >= &fmsetl_processedDTTM_To);

  data WORK._TMPSDMDIM;
     set &cSDMLib..SAS_DIMENSION(&_tmpWhere);
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  proc sort data = WORK._TMPSDMDIM;
     by &_tmpSortVar;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  data WORK._TMPALLDIMS
       WORK._TMPMISSINGDIMS;
     merge WORK._TMPALLDIMS (in = inA)
           WORK._TMPSDMDIM (in = inB);
        by &_tmpSortVar;
     keep DIMENSION_CD INTERNAL_DIMENSION_ID DIM_TYPE_ID;
     if (inA and ^inB) then
        output WORK._TMPMISSINGDIMS;
     else if (inA) then
        output WORK._TMPALLDIMS;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  proc sql noprint;
     select count(*) INTO :missingCnt FROM WORK._TMPMISSINGDIMS;
  quit;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  %if (&missingCnt > 0) %then %do;
     data _null_;
        set WORK._TMPMISSINGDIMS (obs=1);
        call symput("missing_dim", strip(DIMENSION_CD));
     run;
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=6
             , msgTxt1=<&missing_dim>, msgPutlog=&putlog);
     %goto exit ;
  %end;

  %let _tmpSortVar = DIM_TYPE_ID;
  proc sort data = WORK._TMPALLDIMS;
     by &_tmpSortVar;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  %let _tmpWhere = where=(VALID_FROM_DTTM <= &fmsetl_processedDTTM_To and
                          VALID_TO_DTTM >= &fmsetl_processedDTTM_To);

  data WORK._TMPSDMDIMTYPE;
     set &cSDMLib..SAS_DIM_TYPE (&_tmpWhere);
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  proc sort data = WORK._TMPSDMDIMTYPE;
     by &_tmpSortVar;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  data WORK._TMPALLDIMS;
     merge WORK._TMPALLDIMS (in = inA)
           WORK._TMPSDMDIMTYPE;
        by &_tmpSortVar;
     keep DIMENSION_CD INTERNAL_DIMENSION_ID DIM_TYPE_ID DIM_TYPE_CD;
     if (inA) then
        output;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  proc sql noprint;
     select count(*) INTO :dimcnt FROM WORK._TMPALLDIMS;
  quit;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  data _null_;
     set WORK._TMPALLDIMS;
     if (_n_ = 1) then do;
        call symput("bfdtypes",DIM_TYPE_CD);
        call symput("dtype_rks",DIM_TYPE_ID);
     end;
     else do;
        call symput("bfdtypes",strip(symget("bfdtypes"))||"|"||DIM_TYPE_CD);
        call symput("dtype_rks",strip(symget("dtype_rks"))||","||DIM_TYPE_ID);
     end;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  /* Check if &cSDMLib..SAS_PROPERTY exists */
  %let _tmpDSN = &cSDMLib..SAS_PROPERTY;
  %if not %sysfunc( exist(&_tmpDSN) ) %then %do;
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=19
             , msgPutlog=N);
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=3
             , msgTxt1=property, msgTxt2=&sesgmsg_msgReturn
             , msgPutlog=&putlog);
     %goto exit ;
  %end ;


  /* Retrieve INTERNAL_DIMENSION_ID for the ACCOUNT dimension in the cycle_dim_table */
  %let _tmpDSN = &cSDMLib..SAS_DIM_TYPE;
  %let acct_dim_type_rk = 0;
  proc sql noprint;
    select DIM_TYPE_ID into :acct_dim_type_rk from &_tmpDSN
    where DIM_TYPE_CD = 'ACCOUNT' and VALID_TO_DTTM  >= &fmsetl_processedDTTM_To;
  quit;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  %if &acct_dim_type_rk = 0 %then %do;
     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=4,
              msgTxt1=<ACCOUNT>, msgPutlog=&putlog);
     %goto exit;
  %end;

  /* Create table containing codes and dimension internal rk's for the account dimension */
  %let _tmpDSN = &cSDMLib..SAS_DIMENSION;
  %let _tmpSortVar = DIMENSION_CD;
  %let _tmpWhere=where=(DIM_TYPE_ID = &acct_dim_type_rk and
                        VALID_TO_DTTM >= &fmsetl_processedDTTM_To);

  data WORK._TMPSORT;
     set &_tmpDSN(&_tmpWhere);
     keep &_tmpSortVar INTERNAL_DIMENSION_ID;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  proc sort data=WORK._TMPSORT;
     by &_tmpSortVar;
  run;

  data WORK._TMPACCTDIMS;
     set WORK._TMPSORT;
     CODE = &_tmpSortVar;
     keep CODE INTERNAL_DIMENSION_ID;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  /* Check for the existence of active account dimensions in the dimension table */
  proc sql noprint;
     select count(*) INTO :_rec_cnt
     from WORK._TMPACCTDIMS
     ;
  quit;
  %_chkError;
  %if &_tmpRC %then %goto exit ;

  %if (&_rec_cnt < 1) %then %do;
    %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=6
            , msgTxt1=<ACCOUNT>, msgPutlog=&putlog);
    %goto exit ;
  %end;

  proc sort data=WORK._TMPACCTDIMS;
     by CODE;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  /* Retrieve the Dimension Internal Id for the account dimension in cycle dimension table */
  %let acct_int_rk = 0;
  data _null_;
     merge WORK._TMPACCTDIMS (in = inA)
           &cycle_dim_table (in = inB);
        by CODE;
     if (inA and inB) then
        call symput("acct_int_rk", INTERNAL_DIMENSION_ID);
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  %if &acct_int_rk = 0 %then %do;
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=8
             , msgTxt1=<ACCOUNT>, msgPutlog=&putlog);
     %goto exit ;
  %end ;


  /* FMT_TO_DIM */
  %let _tmpDSN=WORK._TMPALLDIMS;
  %let _tmpFormat=FMT_TO_DIM;
  %let _tmpStart=DIM_TYPE_CD;
  %let _tmpLabel=DIMENSION_CD;
  data WORK.&_tmpFormat;
     length START $ 64;
     length LABEL $ 32;
     retain FMTNAME "&_tmpFormat";
     retain TYPE "C";
     retain HLO "";
     set &_tmpDSN;
     START = &_tmpStart;
     LABEL = &_tmpLabel;
     keep START LABEL FMTNAME TYPE HLO;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  proc format cntlin=&_tmpFormat library=WORK;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  %if not %sysfunc( cexist(work.formats.&_tmpFormat..formatc) ) %then %do;
     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=11,
              msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
     %goto exit;
  %end;

  /* FMT_DIMS */
  %let _tmpDSN=WORK._TMPALLDIMS;
  %let _tmpFormat=FMT_DIMS;
  %let _tmpStart=DIM_TYPE_ID;
  %let _tmpLabel=INTERNAL_DIMENSION_ID;
  data work.&_tmpFormat;
     length START 8;
     length LABEL 8;
     retain FMTNAME "&_tmpFormat";
     retain TYPE "I";
     retain HLO "";
     set &_tmpDSN;
     START = &_tmpStart;
     LABEL = &_tmpLabel;
     keep START LABEL FMTNAME TYPE HLO;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  proc format cntlin=&_tmpFormat library=WORK;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  %if not %sysfunc( cexist(work.formats.&_tmpFormat..infmt) ) %then %do;
     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=11,
              msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
     %goto exit;
  %end;

  /* FMT_DTYPE_RK */
  %let _tmpDSN=WORK._TMPALLDIMS;
  %let _tmpFormat=FMT_DTYPE_RK;
  %let _tmpStart=DIMENSION_CD;
  %let _tmpLabel=DIM_TYPE_ID;
  data work.&_tmpFormat;
     length START $ 64;
     length LABEL 8;
     retain FMTNAME "&_tmpFormat";
     retain TYPE "I";
     retain HLO "";
     set &_tmpDSN;
     START = &_tmpStart;
     LABEL = &_tmpLabel;
     keep START LABEL FMTNAME TYPE HLO;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  proc format cntlin=&_tmpFormat library=WORK;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;


  %if not %sysfunc( cexist(work.formats.&_tmpFormat..infmt) ) %then %do;
     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=11,
              msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
     %goto exit;
  %end;

  /* FMT_MEMTYPE -- format to be used for data validation */
  %let _tmpFormat=FMT_MEMTYPE;
  %let _tmpStart=MEMBER_ID;
  %let _tmpLabel=DIM_TYPE_CD;
  %let _tmpWhere=where=(VALID_FROM_DTTM <= &fmsetl_processedDTTM_To and
                        VALID_TO_DTTM >= &fmsetl_processedDTTM_To);

  %let _tmpDSN=WORK._TMPALLDIMS;
  %let _tmpSortVar=INTERNAL_DIMENSION_ID;
  proc sort data = &_tmpDSN nodupkey;
     by &_tmpSortVar;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  %let _tmpDSN=&cSDMLib..SAS_MEMBER;

  data WORK._TMPMEMBER;
     set &_tmpDSN(&_tmpWhere);
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  proc sort data = WORK._TMPMEMBER;
     by &_tmpSortVar;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  data work.&_tmpFormat;
     length START 8;
     length LABEL $ 32;
     retain FMTNAME "&_tmpFormat";
     retain HLO "";
     retain TYPE "N";
     merge WORK._TMPALLDIMS (in=inA)
           WORK._TMPMEMBER (in=inB);
        by &_tmpSortVar;
     START = &_tmpStart;
     LABEL = &_tmpLabel;
     keep START LABEL FMTNAME TYPE HLO;
     if (inA and inB) and (DIM_TYPE_CD ^= 'TRADER') then
        output;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  data WORK.&_tmpFormat;
     set WORK.&_tmpFormat end=last;
     output;
     if (last) then do;
        HLO = 'O';
        START = .;
        LABEL = '*** ERROR ***';
        FMTNAME = "&_tmpFormat";
        TYPE = "N";
        output;
     end;
  run;

  proc format cntlin=&_tmpFormat library=WORK;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  %if not %sysfunc( cexist(work.formats.&_tmpFormat..format) ) %then %do;
     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=11,
              msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
     %goto exit;
  %end;

  /* Create the formats for all of the dimensions */

  %macro fmtdims/parmbuff;

  %let _tmpDSN=&cSDMLib..SAS_MEMBER;
  %let _tmpStart=MEMBER_CD;
  %let _tmpLabel=MEMBER_ID;

  %let fmt=fmt_dtype;
  %let x=_x;

  %do i = 1 %to &dimcnt;
     %let dtype_rk=%sysfunc(kscan(&syspbuff,&i,'(,)'));
     %let _tmpWhere=where=(INTERNAL_DIMENSION_ID = input(symget("dtype_rk"), FMT_DIMS.) and
                           VALID_FROM_DTTM <= &fmsetl_processedDTTM_To and
                           VALID_TO_DTTM >= &fmsetl_processedDTTM_To);
     proc sort data = &_tmpDSN(&_tmpWhere
                      keep= &_tmpStart &_tmpLabel INTERNAL_DIMENSION_ID VALID_FROM_DTTM VALID_TO_DTTM)
                out = work._tmpSort(drop= INTERNAL_DIMENSION_ID VALID_FROM_DTTM VALID_TO_DTTM);
        by &_tmpStart;
     run;
     %_chkError;
     %if &_tmpRC %then %goto fmtdims_exit;

     %let _tmpFormat=&fmt&dtype_rk&x;

     data work.&_tmpFormat;
        length START $ 64;
        length LABEL 8;
        retain FMTNAME "&_tmpFormat";
        retain TYPE "I";
        retain HLO "";
        set WORK._TMPSORT end=last;
        START = strip(&_tmpStart);
        LABEL = &_tmpLabel;
        output;
        if (last) then do;
           HLO = 'O';
           START = .;
           LABEL = .;
           output;
        end;
        keep START LABEL FMTNAME TYPE HLO;
     run;
     %_chkError;
     %if &_tmpRC %then %goto fmtdims_exit;

     proc format cntlin=&_tmpFormat library=WORK;
     run;
     %_chkError;
     %if &_tmpRC %then %goto fmtdims_exit;

     %if not %sysfunc( cexist(work.formats.&_tmpFormat..infmt) ) %then %do;
        %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=11,
                 msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
        %let format_missing = 1;
        %goto fmtdims_exit;
     %end;
  %end;

  %fmtdims_exit:

  %mend fmtdims;

  %fmtdims(&dtype_rks);

  %if (&format_missing) %then
      %goto exit;


  /* Check transaction records for validity */

  %macro datawork/parmbuff;

  %let err=_ERROR;
  %let fmt=fmt_dtype;
  %let x=_x.;
  %let cd=_CD;
  %let code=_CODE;

  /* Check if transaction table exists */
  %let _tmpDSN = &gl_trans_sum_table;
  %if not %sysfunc( exist(&_tmpDSN) ) %then %do;
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=21
             , msgPutlog=N);
     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=7
            , msgTxt1=&_tmpDSN, msgTxt2=&sesgmsg_msgReturn
            , msgPutlog=&putlog);
     %goto datawork_exit;
  %end ;

  /* ANALYSIS TIME INTORG */
  /* ACCOUNT CURRENCY ANALYSIS TIME INTORG SOURCE */

  data &update_tablename.(keep=CONCAT_MEMBER_IDS MEASURE OBJECT_TYPE_ID OBJECT_ID ANALYSIS TIME INTORG rename=(MEASURE=FACT_VALUE))
       WORK.BADRECS(drop=CONCAT_MEMBER_IDS OBJECT_TYPE_ID OBJECT_ID)
       WORK.ORG_EQ_TRADER(drop=CONCAT_MEMBER_IDS OBJECT_TYPE_ID OBJECT_ID)
       WORK.ICRECS(drop=CONCAT_MEMBER_IDS OBJECT_TYPE_ID OBJECT_ID)
       WORK.NONICRECS(drop=CONCAT_MEMBER_IDS OBJECT_TYPE_ID OBJECT_ID)
     ;
     length MEASURE 8;
     format MEASURE best32.;
     length BAD_RECORD 8;
     length MEM_DIMTYPE $ 8;
     length DIMTYPE_HOLD $ 8;
     length CONCAT_MEMBER_IDS $ 255;
     length OBJECT_TYPE_ID 8 ;
     length OBJECT_ID 8 ;
     set &_tmpDSN;
     MEASURE = TRANSACTION_AMT;
     BAD_RECORD = 0;
     /* Validate the members of the dimension types existing in the basefact table */
     %let dtypeString=%sysfunc(ksubstr(&syspbuff, 2), %eval(%sysfunc(klength(&syspbuff)) -2 ));
     %do i = 1 %to &dimcnt;
         %let dimtype=%sysfunc(kscan(%bquote(&dtypeString),&i,|));
         %let dimname=%sysfunc(putc(&dimtype, $FMT_TO_DIM.));
         %let dimtype_rk=%sysfunc(inputn(&dimname, FMT_DTYPE_RK.));
         %let dimtype1=%sysfunc(putc(%sysfunc(strip(&dimtype)), $DTPCOL.));
         %let dimtypecd=%sysfunc(strip(&dimtype1))&cd;
         %let dimtypecode=%sysfunc(strip(&dimtype1))&code;
         %let dimtypeerr=%sysfunc(strip(&dimtype1))&err;

         DIMTYPE_HOLD = "&dimtype";
         "&dimtype"n=input(strip("&dimtypecd"n), &fmt&dimtype_rk.&x);
         checkId = "&dimtype"n ;
         MEM_DIMTYPE = strip(put(checkId, FMT_MEMTYPE.));

         if (MEM_DIMTYPE = DIMTYPE_HOLD) then do;
            "&dimtypeerr"n = 0;
         end;
         else do;
            BAD_RECORD = 1;
            "&dimtypeerr"n = 1;
         end;

        "&dimtypecode"n = "&dimtypecd"n;
     %end;
     if (BAD_RECORD = 1) then do;
        keep MEASURE CONCAT_MEMBER_IDS OBJECT_TYPE_ID OBJECT_ID
             %do i = 1 %to &dimcnt;
                 %let dimtype=%sysfunc(kscan(%bquote(&dtypeString),&i,|));
                 %let dimtype1=%sysfunc(putc(%sysfunc(strip(&dimtype)), $DTPCOL.));
                 %let dimtypecode=%sysfunc(strip(&dimtype1))&code;
                 %let dimtypeerr=%sysfunc(strip(&dimtype1))&err;
                 "&dimtype"n "&dimtypecode"n "&dimtypeerr"n
             %end;
             ;
        output WORK.BADRECS;
     end;
     else do;
        keep MEASURE
             %do i = 1 %to &dimcnt;
                 %let dimtype=%sysfunc(kscan(%bquote(&dtypeString),&i,|));
                 %let dimtype1=%sysfunc(putc(%sysfunc(strip(&dimtype)), $DTPCOL.));
                 %let dimtypecode=%sysfunc(strip(&dimtype1))&code;
                 "&dimtype"n "&dimtypecode"n
             %end;
             ;
             CONCAT_MEMBER_IDS = &fmsetl_concatMbrStmt ;
             OBJECT_TYPE_ID = &dataLoadObjTypeId ;

             OBJECT_ID = &dataLoadObjId;
             output &update_tablename;
     end;
  run;

  %_chkError;
  %if &_tmpRC %then %goto datawork_exit;

  %datawork_exit:

  %mend datawork;

  %datawork(&bfdtypes);


  %_chkError;
  %if &_tmpRC %then %goto exit;

  %let fmsetl_statusId = 0; /* Program ran successful */

  %exit:

  %if &fmsetl_statusID ne 0 %then %do;
     %let fmsetl_error = &sesgmsg_msgCodeReturn;
     %let fmsetl_errormsg = &sesgmsg_msgReturn;
  %end;

%mend sesFMS_imptdataWithoutTr;


/**********************************************************
 * sesFMS_imptdataWithTr.sas
 **********************************************************/

/* Copyright 2003 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved. */

%macro sesFMS_imptdataWithTr (gl_trans_sum_table=, update_tablename=, ic_check_ind=, non_ic_check_ind=,
                 cycle_dim_table=, fmsetl_processedDTTM_To=,
                 dataLoadObjTypeId=, dataLoadObjId=,
                 dimTypeWorkAreaDS= );

  %let fmsetl_error = ;
  %let fmsetl_errormsg = ;
  %let fmsetl_statusId = 1 ; /* Will be changed to 0 if program runs successful */
  %let format_missing = 0;

  %let entry = imptdata;
  %let putlog = Y;

  %let curr_rec_dttm = '01JAN5999:00:00'DT;

  /* options metarepository="&SlnRepos"; */

  %macro _chkError;
    %global _tmpRC;
    %let _tmpRC = 0;
    %if (&sysrc ne 0) or (&syserr ne 0 and &syserr ne 4) %then %do;
      %if (&sysrc ne 0) %then
          %let _tmpRC = &sysrc;
      %else
          %let _tmpRC = &syserr;
      %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=999
              , msgTxt1=&_tmpRC, msgTxt2=&sysmsg, msgPutlog=&putlog);
    %end;
  %mend _chkError;


  %macro mbrConcat ;
     %global fmsetl_concatMbrStmt ;
     %let fmsetl_concatMbrStmt= ;
     data _null_ ;
       length CONCATSTMT $32767 ;
       retain CONCATSTMT ;
       set &dimTypeWorkAreaDS end=nEnd ;
       if _n_ = 1 then do ;
         CONCATSTMT = strip(CONCATSTMT) ||
                      "strip(put(" || quote(strip(CODE)) || "n, 32.))"  ;
       end ;
       else do ;
         CONCATSTMT = strip(CONCATSTMT) || " || ',' || " ||
                      "strip(put(" || quote(strip(CODE)) || "n, 32.))"  ;
       end ;
       if nEnd then do ;
         call symput("fmsetl_concatMbrStmt", CONCATSTMT) ;
       end ;
     run ;
  %mend ;

  %mbrConcat ;

  /* Check if datetime passed in is valid */
  %if %datatyp(&fmsetl_processedDTTM_To) ne NUMERIC %then %do;
    %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=1,
              msgTxt1=<fmsetl_processedDTTM_To : &fmsetl_processedDTTM_To>, msgPutlog=&putlog);
    %goto exit;
  %end;


  /* Check if update_tablename macro variable is passed */

  %if (&update_tablename=.) %then %do;
     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=2,
              msgTxt1=<update_tablename : &update_tablename>, msgPutlog=&putlog);
     %goto exit;
  %end;

  /* Check if &cSDMLib..SAS_DIM_TYPE exists */
  %let _tmpDSN = &cSDMLib..SAS_DIM_TYPE;
  %if not %sysfunc( exist(&_tmpDSN) ) %then %do;
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=13
             , msgPutlog=N);
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=3
             , msgTxt1=dimension_type, msgTxt2=&sesgmsg_msgReturn
             , msgPutlog=&putlog);
     %goto exit ;
  %end;

  %let trader_dim_type_rk = 0;
  proc sql noprint;
    select DIM_TYPE_ID into :trader_dim_type_rk from &_tmpDSN
    where upcase(DIM_TYPE_CD) = 'TRADER' and VALID_TO_DTTM  >= &fmsetl_processedDTTM_To;
  quit;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  %if &trader_dim_type_rk = 0 %then %do;
     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=4,
              msgTxt1=<TRADER>, msgPutlog=&putlog);
     %goto exit;
  %end;

  /* Check if &cSDMLib..SAS_DIMENSION exists */
  %let _tmpDSN = &cSDMLib..SAS_DIMENSION;
  %if not %sysfunc( exist(&_tmpDSN) ) %then %do;
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=14
             , msgPutlog=N);
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=3
             , msgTxt1=dimension, msgTxt2=&sesgmsg_msgReturn
             , msgPutlog=&putlog);
     %goto exit ;
  %end;

  /* Check for the existence of active records in the dimension table */
  proc sql noprint;
     select count(*) INTO :_rec_cnt
     from &_tmpDSN
     ;
  quit;
  %_chkError;
  %if &_tmpRC %then %goto exit ;

  %if (&_rec_cnt < 1) %then %do;
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=15
             , msgPutlog=N);
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=5
             , msgTxt1=dimension, msgTxt2=&sesgmsg_msgReturn
             , msgPutlog=&putlog);
     %goto exit ;
  %end;

  %let _tmpWhere = where=(DIM_TYPE_ID = &trader_dim_type_rk and
                          VALID_TO_DTTM >= &fmsetl_processedDTTM_To);
  data WORK._TMPTRADERDIMS;
     set &_tmpDSN (&_tmpWhere);
     CODE = DIMENSION_CD;
     keep CODE INTERNAL_DIMENSION_ID;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  /* Check for the existence of active trader dimensions in the dimension table */
  proc sql noprint;
     select count(*) INTO :_rec_cnt
     from WORK._TMPTRADERDIMS
     ;
  quit;
  %_chkError;
  %if &_tmpRC %then %goto exit ;

  %if (&_rec_cnt < 1) %then %do;
    %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=6
            , msgTxt1=<TRADER>, msgPutlog=&putlog);
    %goto exit ;
  %end;

  proc sort data=WORK._TMPTRADERDIMS;
     by CODE;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  /* Check if cycle's dimension table exists */
  %let _tmpDSN = &cycle_dim_table;
  %if not %sysfunc( exist(&_tmpDSN) ) %then %do;
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=16
             , msgPutlog=N);
     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=7
            , msgTxt1=&_tmpDSN, msgTxt2=&sesgmsg_msgReturn
            , msgPutlog=&putlog);
     %goto exit;
  %end;

  proc sort data=&_tmpDSN;
     by CODE;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  %let trader_int_rk = 0;
  data _null_;
     merge WORK._TMPTRADERDIMS (in = inA)
           &_tmpDSN (in = inB);
        by CODE;
     if (inA and inB) then
        call symput("trader_int_rk",INTERNAL_DIMENSION_ID);
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  %if &trader_int_rk = 0 %then %do;
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=8
             , msgTxt1=<TRADER>, msgPutlog=&putlog);
     %goto exit ;
  %end ;

  /* Check if &cSDMLib..SAS_MEMBER exists */
  %let _tmpDSN = &cSDMLib..SAS_MEMBER;
  %if not %sysfunc( exist(&_tmpDSN) ) %then %do;
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=17
             , msgPutlog=N);
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=3
             , msgTxt1=member, msgTxt2=&sesgmsg_msgReturn
             , msgPutlog=&putlog);
     %goto exit ;
  %end ;

  /* Check for the existence of active records in the member table */
  proc sql noprint;
     select count(*) INTO :_rec_cnt
     from &_tmpDSN
     ;
  quit;
  %_chkError;
  %if &_tmpRC %then %goto exit ;

  %if (&_rec_cnt < 1) %then %do;
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=18
             , msgPutlog=N);
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=5
             , msgTxt1=dimension, msgTxt2=&sesgmsg_msgReturn
             , msgPutlog=&putlog);
     %goto exit ;
  %end;

  %if (&ic_check_ind ne Y) and (&ic_check_ind ne N) %then %do;
     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=2,
              msgTxt1=<ic_check_ind : &ic_check_ind>, msgPutlog=&putlog);
     %goto exit;
  %end;

  %if (&non_ic_check_ind ne Y) and (&non_ic_check_ind ne N) %then %do;
     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=2,
              msgTxt1=<non_ic_check_ind : &non_ic_check_ind>, msgPutlog=&putlog);
     %goto exit;
  %end;

  /*
   * build a list of the basefact dimension types in a macro var
   *
   * Name: bfdtypes & dtype_rks
   * Contents: list of dimensions and dimension types separated by commas
   * Example: TIME, INTORG, ANALYSIS, CURRENCY, ....
   *
   * Name: dimcnt
   * Contents: number of items in above lists
   */

  %let _tmpSortVar = DIMENSION_CD;
  data WORK._TMPALLDIMS;
     set &cycle_dim_table;
     &_tmpSortVar = CODE;
     keep &_tmpSortVar;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  proc sort data = WORK._TMPALLDIMS;
     by &_tmpSortVar;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  %let _tmpWhere = where=(VALID_FROM_DTTM <= &fmsetl_processedDTTM_To and
                          VALID_TO_DTTM >= &fmsetl_processedDTTM_To);

  data WORK._TMPSDMDIM;
     set &cSDMLib..SAS_DIMENSION(&_tmpWhere);
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  proc sort data = WORK._TMPSDMDIM;
     by &_tmpSortVar;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  data WORK._TMPALLDIMS
       WORK._TMPMISSINGDIMS;
     merge WORK._TMPALLDIMS (in = inA)
           WORK._TMPSDMDIM (in = inB);
        by &_tmpSortVar;
     keep DIMENSION_CD INTERNAL_DIMENSION_ID DIM_TYPE_ID;
     if (inA and ^inB) then
        output WORK._TMPMISSINGDIMS;
     else if (inA) then
        output WORK._TMPALLDIMS;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  proc sql noprint;
     select count(*) INTO :missingCnt FROM WORK._TMPMISSINGDIMS;
  quit;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  %if (&missingCnt > 0) %then %do;
     data _null_;
        set WORK._TMPMISSINGDIMS (obs=1);
        call symput("missing_dim", strip(DIMENSION_CD));
     run;
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=6
             , msgTxt1=<&missing_dim>, msgPutlog=&putlog);
     %goto exit ;
  %end;

  %let _tmpSortVar = DIM_TYPE_ID;
  proc sort data = WORK._TMPALLDIMS;
     by &_tmpSortVar;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  %let _tmpWhere = where=(VALID_FROM_DTTM <= &fmsetl_processedDTTM_To and
                          VALID_TO_DTTM >= &fmsetl_processedDTTM_To);

  data WORK._TMPSDMDIMTYPE;
     set &cSDMLib..SAS_DIM_TYPE (&_tmpWhere);
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  proc sort data = WORK._TMPSDMDIMTYPE;
     by &_tmpSortVar;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  data WORK._TMPALLDIMS;
     merge WORK._TMPALLDIMS (in = inA)
           WORK._TMPSDMDIMTYPE;
        by &_tmpSortVar;
     keep DIMENSION_CD INTERNAL_DIMENSION_ID DIM_TYPE_ID DIM_TYPE_CD;
     if (inA) then
        output;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  proc sql noprint;
     select count(*) INTO :dimcnt FROM WORK._TMPALLDIMS;
  quit;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  data _null_;
     set WORK._TMPALLDIMS;
     if (_n_ = 1) then do;
        call symput("bfdtypes",DIM_TYPE_CD);
        call symput("dtype_rks",DIM_TYPE_ID);
     end;
     else do;
        call symput("bfdtypes",strip(symget("bfdtypes"))||"|"||DIM_TYPE_CD);
        call symput("dtype_rks",strip(symget("dtype_rks"))||","||DIM_TYPE_ID);
     end;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  /* Check if &cSDMLib..SAS_PROPERTY exists */
  %let _tmpDSN = &cSDMLib..SAS_PROPERTY;
  %if not %sysfunc( exist(&_tmpDSN) ) %then %do;
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=19
             , msgPutlog=N);
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=3
             , msgTxt1=property, msgTxt2=&sesgmsg_msgReturn
             , msgPutlog=&putlog);
     %goto exit ;
  %end ;

  %let ic_prop_rk = 0;
  proc sql noprint;
    select PROPERTY_ID into :ic_prop_rk from &_tmpDSN
    where upcase(PROPERTY_CD) = 'INTERCOMPANY';
  quit;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  %if &ic_prop_rk = 0 %then %do;
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=9
             , msgTxt1=<INTERCOMPANY>, msgPutlog=&putlog);
     %goto exit ;
  %end;

  /* Retrieve INTERNAL_DIMENSION_ID for the ACCOUNT dimension in the cycle_dim_table */
  %let _tmpDSN = &cSDMLib..SAS_DIM_TYPE;
  %let acct_dim_type_rk = 0;
  proc sql noprint;
    select DIM_TYPE_ID into :acct_dim_type_rk from &_tmpDSN
    where DIM_TYPE_CD = 'ACCOUNT' and VALID_TO_DTTM  >= &fmsetl_processedDTTM_To;
  quit;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  %if &acct_dim_type_rk = 0 %then %do;
     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=4,
              msgTxt1=<ACCOUNT>, msgPutlog=&putlog);
     %goto exit;
  %end;

  /* Create table containing codes and dimension internal rk's for the account dimension */
  %let _tmpDSN = &cSDMLib..SAS_DIMENSION;
  %let _tmpSortVar = DIMENSION_CD;
  %let _tmpWhere=where=(DIM_TYPE_ID = &acct_dim_type_rk and
                        VALID_TO_DTTM >= &fmsetl_processedDTTM_To);

  data WORK._TMPSORT;
     set &_tmpDSN(&_tmpWhere);
     keep &_tmpSortVar INTERNAL_DIMENSION_ID;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  proc sort data=WORK._TMPSORT;
     by &_tmpSortVar;
  run;

  data WORK._TMPACCTDIMS;
     set WORK._TMPSORT;
     CODE = &_tmpSortVar;
     keep CODE INTERNAL_DIMENSION_ID;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  /* Check for the existence of active account dimensions in the dimension table */
  proc sql noprint;
     select count(*) INTO :_rec_cnt
     from WORK._TMPACCTDIMS
     ;
  quit;
  %_chkError;
  %if &_tmpRC %then %goto exit ;

  %if (&_rec_cnt < 1) %then %do;
    %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=6
            , msgTxt1=<ACCOUNT>, msgPutlog=&putlog);
    %goto exit ;
  %end;

  proc sort data=WORK._TMPACCTDIMS;
     by CODE;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  /* Retrieve the Dimension Internal Id for the account dimension in cycle dimension table */
  %let acct_int_rk = 0;
  data _null_;
     merge WORK._TMPACCTDIMS (in = inA)
           &cycle_dim_table (in = inB);
        by CODE;
     if (inA and inB) then
        call symput("acct_int_rk", INTERNAL_DIMENSION_ID);
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  %if &acct_int_rk = 0 %then %do;
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=8
             , msgTxt1=<ACCOUNT>, msgPutlog=&putlog);
     %goto exit ;
  %end ;

  /* Create FMT_INTERCOMPANY -- format needed for determining intercompany accounts */
  %let _tmpDSN = &cSDMLib..SAS_MEMBER_PROPERTY;
  %if not %sysfunc( exist(&_tmpDSN) ) %then %do;
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=20
             , msgPutlog=N);
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=3
             , msgTxt1=member_property, msgTxt2=&sesgmsg_msgReturn
             , msgPutlog=&putlog);
     %goto exit ;
  %end ;

  %let _tmpFormat=FMT_INTERCOMPANY;
  %let _tmpStart=MEMBER_ID;
  %let _tmpLabel=PROPERTY_VALUE;
  %let _tmpWhere=where=(PROPERTY_ID = &ic_prop_rk and
                        INTERNAL_DIMENSION_ID = &acct_int_rk and
                        VALID_FROM_DTTM <= &fmsetl_processedDTTM_To and
                        VALID_TO_DTTM >= &fmsetl_processedDTTM_To);
  data work.&_tmpFormat;
     length START 8;
     length LABEL $ 32;
     retain FMTNAME "&_tmpFormat";
     retain TYPE "N";
     retain HLO "";
     set &_tmpDSN (&_tmpWhere);
     START = &_tmpStart;
     if (&_tmpLabel = 'True') then
        LABEL = 'Y';
     else
        LABEL = 'N';
     keep START LABEL FMTNAME TYPE HLO;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  /* Check for the existence of values for the "INTERCOMPANY" property */
  proc sql noprint;
     select count(*) INTO :_rec_cnt
     from WORK.&_tmpFormat
     ;
  quit;
  %_chkError;
  %if &_tmpRC %then %goto exit ;

  %if (&_rec_cnt < 1) %then %do;
    %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=10
            , msgPutlog=&putlog);
    %goto exit ;
  %end;

  proc format cntlin=&_tmpFormat library=WORK;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  %if not %sysfunc( cexist(work.formats.&_tmpFormat..format) ) %then %do;
     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=11,
              msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
     %goto exit;
  %end;

  /* FMT_TO_DIM */
  %let _tmpDSN=WORK._TMPALLDIMS;
  %let _tmpFormat=FMT_TO_DIM;
  %let _tmpStart=DIM_TYPE_CD;
  %let _tmpLabel=DIMENSION_CD;
  data WORK.&_tmpFormat;
     length START $ 64;
     length LABEL $ 32;
     retain FMTNAME "&_tmpFormat";
     retain TYPE "C";
     retain HLO "";
     set &_tmpDSN;
     START = &_tmpStart;
     LABEL = &_tmpLabel;
     keep START LABEL FMTNAME TYPE HLO;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  proc format cntlin=&_tmpFormat library=WORK;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  %if not %sysfunc( cexist(work.formats.&_tmpFormat..formatc) ) %then %do;
     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=11,
              msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
     %goto exit;
  %end;

  /* FMT_DIMS */
  %let _tmpDSN=WORK._TMPALLDIMS;
  %let _tmpFormat=FMT_DIMS;
  %let _tmpStart=DIM_TYPE_ID;
  %let _tmpLabel=INTERNAL_DIMENSION_ID;
  data work.&_tmpFormat;
     length START 8;
     length LABEL 8;
     retain FMTNAME "&_tmpFormat";
     retain TYPE "I";
     retain HLO "";
     set &_tmpDSN;
     START = &_tmpStart;
     LABEL = &_tmpLabel;
     keep START LABEL FMTNAME TYPE HLO;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  proc format cntlin=&_tmpFormat library=WORK;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  %if not %sysfunc( cexist(work.formats.&_tmpFormat..infmt) ) %then %do;
     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=11,
              msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
     %goto exit;
  %end;

  /* FMT_DTYPE_RK */
  %let _tmpDSN=WORK._TMPALLDIMS;
  %let _tmpFormat=FMT_DTYPE_RK;
  %let _tmpStart=DIMENSION_CD;
  %let _tmpLabel=DIM_TYPE_ID;
  data work.&_tmpFormat;
     length START $ 64;
     length LABEL 8;
     retain FMTNAME "&_tmpFormat";
     retain TYPE "I";
     retain HLO "";
     set &_tmpDSN;
     START = &_tmpStart;
     LABEL = &_tmpLabel;
     keep START LABEL FMTNAME TYPE HLO;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  proc format cntlin=&_tmpFormat library=WORK;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;


  %if not %sysfunc( cexist(work.formats.&_tmpFormat..infmt) ) %then %do;
     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=11,
              msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
     %goto exit;
  %end;

  /* FMT_MEMTYPE -- format to be used for data validation */
  %let _tmpFormat=FMT_MEMTYPE;
  %let _tmpStart=MEMBER_ID;
  %let _tmpLabel=DIM_TYPE_CD;
  %let _tmpWhere=where=(VALID_FROM_DTTM <= &fmsetl_processedDTTM_To and
                        VALID_TO_DTTM >= &fmsetl_processedDTTM_To);

  %let _tmpDSN=WORK._TMPALLDIMS;
  %let _tmpSortVar=INTERNAL_DIMENSION_ID;
  proc sort data = &_tmpDSN nodupkey;
     by &_tmpSortVar;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  %let _tmpDSN=&cSDMLib..SAS_MEMBER;

  data WORK._TMPMEMBER;
     set &_tmpDSN(&_tmpWhere);
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  proc sort data = WORK._TMPMEMBER;
     by &_tmpSortVar;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  data work.&_tmpFormat;
     length START 8;
     length LABEL $ 32;
     retain FMTNAME "&_tmpFormat";
     retain HLO "";
     retain TYPE "N";
     merge WORK._TMPALLDIMS (in=inA)
           WORK._TMPMEMBER (in=inB);
        by &_tmpSortVar;
     START = &_tmpStart;
     LABEL = &_tmpLabel;
     keep START LABEL FMTNAME TYPE HLO;
     if (inA and inB) and (DIM_TYPE_CD ^= 'TRADER') then
        output;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  data WORK.&_tmpFormat;
     set WORK.&_tmpFormat end=last;
     output;
     if (last) then do;
        HLO = 'O';
        START = .;
        LABEL = '*** ERROR ***';
        FMTNAME = "&_tmpFormat";
        TYPE = "N";
        output;
     end;
  run;

  proc format cntlin=&_tmpFormat library=WORK;
  run;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  %if not %sysfunc( cexist(work.formats.&_tmpFormat..format) ) %then %do;
     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=11,
              msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
     %goto exit;
  %end;

  /* Create the formats for all of the dimensions */

  %macro fmtdims/parmbuff;

  %let _tmpDSN=&cSDMLib..SAS_MEMBER;
  %let _tmpStart=MEMBER_CD;
  %let _tmpLabel=MEMBER_ID;

  %let fmt=fmt_dtype;
  %let x=_x;

  %do i = 1 %to &dimcnt;
     %let dtype_rk=%sysfunc(kscan(&syspbuff,&i,'(,)'));
     %let _tmpWhere=where=(INTERNAL_DIMENSION_ID = input(symget("dtype_rk"), FMT_DIMS.) and
                           VALID_FROM_DTTM <= &fmsetl_processedDTTM_To and
                           VALID_TO_DTTM >= &fmsetl_processedDTTM_To);
     proc sort data = &_tmpDSN(&_tmpWhere
                      keep= &_tmpStart &_tmpLabel INTERNAL_DIMENSION_ID VALID_FROM_DTTM VALID_TO_DTTM)
                out = work._tmpSort(drop= INTERNAL_DIMENSION_ID VALID_FROM_DTTM VALID_TO_DTTM);
        by &_tmpStart;
     run;
     %_chkError;
     %if &_tmpRC %then %goto fmtdims_exit;

     %let _tmpFormat=&fmt&dtype_rk&x;

     data work.&_tmpFormat;
        length START $ 64;
        length LABEL 8;
        retain FMTNAME "&_tmpFormat";
        retain TYPE "I";
        retain HLO "";
        set WORK._TMPSORT end=last;
        START = strip(&_tmpStart);
        LABEL = &_tmpLabel;
        output;
        if (last) then do;
           HLO = 'O';
           START = .;
           LABEL = .;
           output;
        end;
        keep START LABEL FMTNAME TYPE HLO;
     run;
     %_chkError;
     %if &_tmpRC %then %goto fmtdims_exit;

     proc format cntlin=&_tmpFormat library=WORK;
     run;
     %_chkError;
     %if &_tmpRC %then %goto fmtdims_exit;

     %if not %sysfunc( cexist(work.formats.&_tmpFormat..infmt) ) %then %do;
        %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=11,
                 msgTxt1=<&_tmpFormat>, msgTxt2=&syserr, msgPutlog=&putlog);
        %let format_missing = 1;
        %goto fmtdims_exit;
     %end;
  %end;

  %fmtdims_exit:

  %mend fmtdims;

  %fmtdims(&dtype_rks);

  %if (&format_missing) %then
      %goto exit;

  /* Determine the member_id of the EXT member of the TRADER dimension */

  %let ext_mem = 0;
  proc sql noprint;
    select MEMBER_ID into :ext_mem from &_tmpDSN
    where INTERNAL_DIMENSION_ID = &trader_int_rk and MEMBER_CD = 'EXT';
  quit;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  %if &ext_mem = 0 %then %do;
     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=12,
              msgTxt1=<EXT>, msgPutlog=&putlog);
     %goto exit;
  %end;

  /* Determine the member_id of the ALL member of the TRADER dimension */

  %let all_mem = 0;
  proc sql noprint;
    select MEMBER_ID into :all_mem from &_tmpDSN
    where INTERNAL_DIMENSION_ID = &trader_int_rk and MEMBER_CD = 'ALL';
  quit;
  %_chkError;
  %if &_tmpRC %then %goto exit;

  %if &all_mem = 0 %then %do;
     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=12,
              msgTxt1=<ALL>, msgPutlog=&putlog);
     %goto exit;
  %end;

  /* Check transaction records for validity */

  %macro datawork/parmbuff;

  %let err=_ERROR;
  %let fmt=fmt_dtype;
  %let x=_x.;
  %let cd=_CD;
  %let code=_CODE;

  /* Check if transaction table exists */
  %let _tmpDSN = &gl_trans_sum_table;
  %if not %sysfunc( exist(&_tmpDSN) ) %then %do;
     %sesgmsg( msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=21
             , msgPutlog=N);
     %sesgmsg(msgDsn=SASHELP.SFMMSGS, msgEntry=&entry, msgCode=7
            , msgTxt1=&_tmpDSN, msgTxt2=&sesgmsg_msgReturn
            , msgPutlog=&putlog);
     %goto datawork_exit;
  %end ;

  /* ANALYSIS TIME INTORG */
  /* ACCOUNT CURRENCY ANALYSIS TIME INTORG SOURCE */

  data &update_tablename.(keep=CONCAT_MEMBER_IDS MEASURE OBJECT_TYPE_ID OBJECT_ID ANALYSIS TIME INTORG rename=(MEASURE=FACT_VALUE))
       WORK.BADRECS(drop=CONCAT_MEMBER_IDS OBJECT_TYPE_ID OBJECT_ID)
       WORK.ORG_EQ_TRADER(drop=CONCAT_MEMBER_IDS OBJECT_TYPE_ID OBJECT_ID)
       WORK.ICRECS(drop=CONCAT_MEMBER_IDS OBJECT_TYPE_ID OBJECT_ID)
       WORK.NONICRECS(drop=CONCAT_MEMBER_IDS OBJECT_TYPE_ID OBJECT_ID)
       ;
     length MEASURE 8;
     format MEASURE best32.;
     length BAD_RECORD 8;
     length MEM_DIMTYPE $ 8;
     length DIMTYPE_HOLD $ 8;
     length INVALID_IC_RECORD 8;
     length INVALID_NON_IC_RECORD 8;
     length INTERCOMPANY_ACCOUNT $ 1;
     length CONCAT_MEMBER_IDS $ 255;
     length OBJECT_TYPE_ID 8 ;
     length OBJECT_ID 8 ;
     set &_tmpDSN;
     MEASURE = TRANSACTION_AMT;
     BAD_RECORD = 0;
     /* Validate the members of the dimension types existing in the basefact table */
     %let dtypeString=%sysfunc(ksubstr(&syspbuff, 2), %eval(%sysfunc(klength(&syspbuff)) -2 ));
     %do i = 1 %to &dimcnt;
         %let dimtype=%sysfunc(kscan(%bquote(&dtypeString),&i,|));
         %let dimname=%sysfunc(putc(&dimtype, $FMT_TO_DIM.));
         %let dimtype_rk=%sysfunc(inputn(&dimname, FMT_DTYPE_RK.));
         %let dimtype1=%sysfunc(putc(%sysfunc(strip(&dimtype)), $DTPCOL.));
         %let dimtypecd=%sysfunc(strip(&dimtype1))&cd;
         %let dimtypecode=%sysfunc(strip(&dimtype1))&code;
         %let dimtypeerr=%sysfunc(strip(&dimtype1))&err;

         DIMTYPE_HOLD = "&dimtype";
         "&dimtype"n=input(strip("&dimtypecd"n), &fmt&dimtype_rk.&x);
         checkId = "&dimtype"n ;
         MEM_DIMTYPE = strip(put(checkId, FMT_MEMTYPE.));
         if (DIMTYPE_HOLD = 'TRADER') then do;
            if (MEM_DIMTYPE = 'INTORG') then do;
               "&dimtypeerr"n = 0;
            end;
            else do;
               bad_record = 1;
               "&dimtypeerr"n = 1;
            end;
         end;
         else do;
            if (DIMTYPE_HOLD = 'INTORG' and (INTORG = &ext_mem or INTORG = &all_mem)) then do;
               BAD_RECORD = 1;
               "&dimtypeerr"n = 1;
            end;
            else do;
               if (MEM_DIMTYPE = DIMTYPE_HOLD) then do;
                  "&dimtypeerr"n = 0;
               end;
               else do;
                  BAD_RECORD = 1;
                  "&dimtypeerr"n = 1;
               end;
            end;
         end;
         "&dimtypecode"n = "&dimtypecd"n;
     %end;
     /* Perform intercompany account check if indicator set to 'Y' */
     INVALID_IC_RECORD = 0;
     %if &ic_check_ind eq Y %then %do;
        INTERCOMPANY_ACCOUNT = put(ACCOUNT, FMT_INTERCOMPANY.);
        if ((INTERCOMPANY_ACCOUNT = 'Y') and (TRADER = INTORG or TRADER = &ext_mem)) then
           INVALID_IC_RECORD = 1;
     %end;

     /* Perform non-intercompany account check if indicator set to 'Y' */
     INVALID_NON_IC_RECORD = 0;
     %if &non_ic_check_ind eq Y %then %do;
        INTERCOMPANY_ACCOUNT = put(ACCOUNT, FMT_INTERCOMPANY.);
        if ((INTERCOMPANY_ACCOUNT = 'N') and (TRADER ^= INTORG) and (TRADER ^= &ext_mem)) then
           INVALID_NON_IC_RECORD = 1;
     %end ;

     if (BAD_RECORD = 1) then do;
        keep MEASURE CONCAT_MEMBER_IDS OBJECT_TYPE_ID OBJECT_ID
             %do i = 1 %to &dimcnt;
                 %let dimtype=%sysfunc(kscan(%bquote(&dtypeString),&i,|));
                 %let dimtype1=%sysfunc(putc(%sysfunc(strip(&dimtype)), $DTPCOL.));
                 %let dimtypecode=%sysfunc(strip(&dimtype1))&code;
                 %let dimtypeerr=%sysfunc(strip(&dimtype1))&err;
                 "&dimtype"n "&dimtypecode"n "&dimtypeerr"n
             %end;
             ;
        output WORK.BADRECS;
     end;
     else do;
        keep MEASURE
             %do i = 1 %to &dimcnt;
                 %let dimtype=%sysfunc(kscan(%bquote(&dtypeString),&i,|));
                 %let dimtype1=%sysfunc(putc(%sysfunc(strip(&dimtype)), $DTPCOL.));
                 %let dimtypecode=%sysfunc(strip(&dimtype1))&code;
                 "&dimtype"n "&dimtypecode"n
             %end;
             ;
        if (INTORG = TRADER) then
           output WORK.ORG_EQ_TRADER;
        else
           if (INVALID_IC_RECORD = 1) then
              output WORK.ICRECS;
           else
              if (INVALID_NON_IC_RECORD = 1) then
                 output WORK.NONICRECS;
              else do ;
                 CONCAT_MEMBER_IDS = &fmsetl_concatMbrStmt ;
                 OBJECT_TYPE_ID = &dataLoadObjTypeId ;

                 OBJECT_ID = &dataLoadObjId;

                 output &update_tablename;
              end ;
     end;
  run;

  %_chkError;
  %if &_tmpRC %then %goto datawork_exit;

  %datawork_exit:

  %mend datawork;

  %datawork(&bfdtypes);


  %_chkError;
  %if &_tmpRC %then %goto exit;

  %let fmsetl_statusId = 0; /* Program ran successful */

  %exit:

  %if &fmsetl_statusID ne 0 %then %do;
     %let fmsetl_error = &sesgmsg_msgCodeReturn;
     %let fmsetl_errormsg = &sesgmsg_msgReturn;
  %end;

%mend sesFMS_imptdataWithTr;


/**********************************************************
 * sesFMS_init.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : YTDConv.sas                                                                     */
/*  Purpose: 1) All macros %sesSOL_* should be compiled before it                           */
/*              YTDConv is using macros of FMSUtil                                           */
/*           2) %FMSUtil should be run before YTDConv                                        */
/*              %FMSUtil includes globals used in YTDConv and YTDConv                        */
/*                                                                                           */
/*  History: 23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.            */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_init;

  /* Initialize list (dataset) */
  proc sql noprint;
    create table lDDSTranCols (code varchar(50));
    insert into lDDSTranCols
        values ('PROCESSED_DTTM')
        values ('TRANSACTION_AMT')
        values ('TRANSACTION_AMT_YTD_FLG')
        values ('AFFECTED_TIME_PERIOD_RK')
        values ('INITIATING_INTERNAL_ORG_RK')
        values ('AFFECTED_INTERNAL_ORG_RK')
        values ('GL_ACCOUNT_RK')
        values ('ANALYSIS_RK')
        values ('CURRENCY_CD')
     ;
  quit;

  proc sql noprint;
    create table lDDSTranOPCols (code varchar(50));
    insert into lDDSTranOPCols
        values ('PROCESSED_DTTM')
        values ('TRANSACTION_AMT')
        values ('TRANSACTION_AMT_YTD_FLG')
        values ('TIME_PERIOD_RK')
        values ('INTERNAL_ORG_RK')
        values ('GL_ACCOUNT_RK')
        values ('ANALYSIS_RK')
        values ('CURRENCY_CD')
     ;
  quit;

%mend sesFMS_init;


/**********************************************************
 * sesFMS_LoadCycleBaseFact.sas
 **********************************************************/

/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved. */
/* ----------------------------------------------------------------------------------------- */
/*  Name   : LoadCycleBaseFact.sas                                                           */
/*  PURPOSE:   Loads basefacts from the DDS into a Cycles basefact table.  Process includes: */
/*                                                                                           */
/*             Convert YTD transaction amount to Period Activity values.  This conversion    */
/*             occurs when transactions are imported into a Cycle from the DDS transaction   */
/*             table.                                                                        */
/*                                                                                           */
/*             If a libref to the data set does not exist, this process will attempt to      */
/*             submit one.  The libref is not removed when this code completes.              */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_LoadCycleBaseFact;

  /* Initialize global macros to run macros from FMSUtil and YTDConv */

  %global _FMErrorCode;
  %let _FMErrorCode=1; /* 0: error, 1:normal execution */

  %let nTrackTime = %qsysfunc(datetime()) ;

  /*  Seed initial return values  */
  %global fmsetl_statusId;
  %let fmsetl_statusId=1; /* success=0, critical error=1, validation error=2, warning=3  */

  %sesSOL_init ;
  %sesFMS_init ;

  /*  Turn logging information on/off based on fmsetl_YtdDebug macro value  */

  /*
   *  We need to create a empty or blank report and dump to DAV, otherwise we get a package exception
   *  when the StoredProcess completes
   */

  %sesSOL_blankReport ;

  /*
   *  Pick up parameters to this job.  This job is intended to be launched via a store process
   *  from a Java client.  Parameters are passed either by macro variable or by streamed input.
   */

  %let cOMRServer                 = %sesSOL_removeQuotes(&fmsetl_omrServer) ;
  %let nOMRPort                   = &fmsetl_omrPort;

  %let cCONFORMLib                = &fmsetl_CONFORMLibrary;
  %let cDDSTransTableNm           = &fmsetl_DDSTransactionTableNm;
  %let cDDSDimTypeTableNm         = &fmsetl_DDSDimensionTypeTableNm;
  %let cSDMLib                    = &fmsetl_SDMLibrary;
  %let cSource_BaseCode           = &fmsetl_Source_BaseCode;
  %let cSource_BaseJournCode      = &fmsetl_Source_BaseJournCode;
  %let nBaseMbrSDM_Id             = &fmsetl_Source_BaseId;
  %let nBaseJournMbrSDM_Id        = &fmsetl_Source_BaseJournId;
  %let cSourceDimTypeCode         = &fmsetl_sourceDimTypeCode;
  %let nProcessedDTTM_From        = &fmsetl_processedDTTM_From;
  %let nProcessedDTTM_To          = &fmsetl_processedDTTM_To;
  %let cAcctTypeRevenue           = %qsysfunc(strip(&fmsetl_acctTypeRevenue));
  %let cAcctTypeExpense           = %qsysfunc(strip(&fmsetl_acctTypeExpense));
  %let cAcctTypeStatFlow          = %qsysfunc(strip(&fmsetl_acctTypeStatFlow));
  %let cAcctTypeNonFrequency      = %qsysfunc(strip(&fmsetl_acctTypeNonFrequency));
  %let cTmpFactSDM_Nm             = &fmsetl_tempCreateFactTable;
  %let cTmpFactWork_Nm            = WORK.%qsysfunc(strip(%qsysfunc(kscan(&cTmpFactSDM_Nm, 2, .)))) ;
  %let cTmpFactWork_Nm            = WORK.sas_etl_base_fact;
  %let cTmpRemoveROFactSDM_Nm     = &fmsetl_tempRemoveROFactTable;
  %let cTmpRemoveRWFactSDM_Nm     = &fmsetl_tempRemoveRWFactTable;
  %let cSDMStatusDS               = &fmsetl_tempStatusTable;
  %let cCkIcTradingPartners       = %upcase(&fmsetl_ckIcTradingPartners);
  %let cCkNonIcTradingMembers     = %upcase(&fmsetl_ckNonIcTradingMembers);
  %let nAccountDimId              = &fmsetl_accountDimId;
  %let nAcctTypePropertyId        = &fmsetl_acctTypePropertyId;
  %let nActionType                = &fmsetl_actionType;
  %let cDLObjTypeId               = &fmsetl_dataLoadObjectTypeId;
  %let cDLObjId                   = &fmsetl_dataLoadObjectIds;
  %let nSdmROFactSetId            = &fmsetl_readOnlyFactSetId;
  %let nSdmRWFactSetId            = &fmsetl_readWriteFactSetId;

  /* Added for more than time and analysis */
  %let cETL_instanceID            = &fmsetl_instanceId;
  %let cTempFactDimMbr            = &cSdmLib..SAS_ETL_FACT_DIM_MBR;
  %let csubTempFactDimMbr         = WORK.SAS_ETL_FACT_DIM_MBR;

  /*  Assign constants  */
  %let cSCLEntryNm           =sesFMS_LoadCycleBaseFact;
  %let cLogEntry             =LoadCycleBasefact;
  %let cProduct              =BASE;

  %let cCurrentTransYtdDS    =WORK._CURRENTTRANYTD;
  %let cCurrentTransPaDS     =WORK._CURRENTTRANPA;
  %let cPriorTransYtdDS      =WORK._PRIORTRANYTD;
  %let cConvertedGlTransDS   =WORK._CONVERTEDGLTRANS;
  %let cCurrencyWorkAreaDS   =WORK._CYCLECURRENCY;
  %let cAccountWorkAreaDS    =WORK._ACCOUNT;
  %let cDimTypeWorkAreaDS    =WORK._CYCLEDIMTYPE;
  %let cDimTypeWorkAreaDS1   =WORK._CYCLEDIMTYPE1;
  %let cDimWorkAreaDS        =WORK._CYCLEDIM;
  %let cDimDimTypeDS         =WORK._DIMDIMTYPE;

  %let cAnalysisWorkAreaDS   =WORK._ANALYSIS;
  %let cAnalysisWorkAreaDS_T =WORK._ANALYSIS_T;
  %let cAnalysisDS_SDM_RKs   =WORK._ANALYSIS_IDS;

  %let cDDSDimTypeWorkAreaDS =WORK._DDSDIMTYPE;
  %let cRKtoCodeLinkDS       =WORK._DDSDIMTYPETABLENM;
  %let cDimTypeRKColDS       =WORK._DIMTYPERKCOL;
  %let cOutputYtdColBlankDS  =WORK._BLANKYTDFLAGTRANS;
  %let cStatusDS             =WORK._FACTSTATUS;
  %let cDLObjIdDS            =WORK._DLOBJIDS;

  %let cCurrTimeMbrId_Map_PrdId_DS      =WORK._CURRTIME_SDM_IDS;
  %let cCurrTimeMbrCode_Map_PrdId_DS    =WORK._CURRTIME_SDM_CODES;
  %let cCurrTimeMbrDDS_RKs_DS           =WORK._CURRTIME_DDS_RKS;
  %let cCurrTimeDdsRK_To_Code_Fmt       =CURTME;
  %let cCurrTimeMbr_DDSRKs_Map_PrdId_DS =WORK._CURRTIME_DDS_RKS_PRDID;
  %let cCurrTimeDdsRK_To_PrdId_Fmt      =CURTMEP;

  %let cPriorTimeMbrId_Map_PrdId_DS     =WORK._PRIORTIME_SDM_IDS;
  %let cPriorTimeMbrCode_Map_PrdId_DS   =WORK._PRIORTIME_SDM_CODES;
  %let cPriorTimeMbrDDS_RKs_DS          =WORK._PRIORTIME_DDS_RKS;
  %let cPriorTimeDdsRK_To_Code_Fmt      =PRITME;
  %let cPriTimeMbr_DDSRKs_Map_PrdId_DS  =WORK._PRIORTIME_DDS_RKS_PRDID;
  %let cPriorTimeDdsRK_To_PrdId_Fmt     =PRITMEP;

  %let cIsFirstPrd_Map_PrdId_DS       =WORK._ISFIRSTPRD;
  %let cIsFirstPrd_Map_DDSTimeRKs_DS  =WORK._ISFIRSTPRD_MAP_DDS_RK;
  %let cIsFirstPrd_Fmt                =FRSTPRD;

  %let cRunYTDToPA_Map_PrdId_DS       =WORK._RUN_YTD_TO_PA;
  %let cRunYTDToPA_Map_DDSTimeRKs_DS  =WORK._YTDTOPA_MAP_DDS_RK;
  %let cRunYTDToPA_Fmt                =RYTDPA;

  %let cCyclePrdIdsDS                 =WORK._CYCLE_PRD_IDS;

  %let cCurrencyFormat       =$CURR;
  %let cAccountFormat        =ACCT;
  %let cDimTypeFormat        =$DTYPE;
  %let cDimTypeColFormat     =$DTPCOL;
  %let cAnalysisFormat       =ANLYS;
  %let cDDSDimTypeFormat     =$DDTYPE;
  %let cRKtoCodeLinkFormat   =$RKLINK;
  %let cDimTypeRKColFormat   =$RKCOL;
  %let cDLObjIdFormat        =$DLOBJID;
  %let cCurrentAmtColNm      =convert_amt_CurrentYtd;
  %let cPriorAmtColNm        =convert_amt_PriorYtd;
  %let cTransAmtColNm        =transaction_amt;
  %let cDDSJrnlDS            =GL_JRNL;
  %let cDDSJrnlDetailDS      =GL_JRNL_DETAILS;
  %let cJournalDS            =WORK._DDSJRNLS;
  %let cSourceColNm          =SOURCE_CD;
  %let cTempPrdColNm         =TEMP__PERIOD__ID__FOR__MERGE;

  /*  Make sure required values have been supplied  */
  %if &cOMRServer eq   %then %do;
    %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_omrServer, &cSCLEntryNm);
    %goto ERROR;
  %end;

  %if &nOMRPort eq . or &nOMRPort eq 0 %then %do;
    %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_omrPort, &cSCLEntryNm);
    %goto ERROR;
  %end;

  %if &cCONFORMLib eq   %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_CONFORMLibrary, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if &cDDSTransTableNm eq   %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_DDSTransactionTableNm, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if &cSourceDimTypeCode eq   %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_sourceDimTypeCode, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if &cSource_BaseCode eq   %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_Source_BaseCode, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if &cSource_BaseJournCode eq   %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_Source_BaseJournCode, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if &nProcessedDTTM_From eq . %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_processedDTTM_From, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if &nProcessedDTTM_To eq . %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_processedDTTM_To, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if &cAcctTypeRevenue eq   %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_acctTypeRevenue, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if &cAcctTypeExpense eq   %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_acctTypeExpense, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if &cAcctTypeStatFlow eq   %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_acctTypeStatFlow, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if &cAcctTypeNonFrequency eq   %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_AcctTypeNonFrequency, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if &cTmpFactSDM_Nm eq   %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_tempCreateFactTable, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if &cTmpRemoveROFactSDM_Nm eq   %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_tempRemoveROFactTable, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if &cTmpRemoveRWFactSDM_Nm eq   %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_tempRemoveRWFactTable, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if &cCkIcTradingPartners eq   %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_ckIcTradingPartners, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if &cCkNonIcTradingMembers eq   %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_ckNonIcTradingMembers, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if &cSDMLib eq   %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_SDMLibrary, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if &nAccountDimId eq . %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_accountDimId, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if &nAcctTypePropertyId eq . %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_acctTypePropertyId, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if &nActionType eq . %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_actionType, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if &cDLObjTypeId eq   %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_dataLoadObjectTypeId, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if &cDLObjId eq   %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_dataLoadObjectIds, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if &nSdmROFactSetId eq . %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_readOnlyFactSetId, &cSCLEntryNm);
      %goto ERROR;
  %end;
  %if &nSdmRWFactSetId eq . %then %do;
      %sesSOL_logError( &cLogEntry, L, 1, &fmsetl_readWriteFactSetId, &cSCLEntryNm);
      %goto ERROR;
  %end;


  /*
   *  DDS Libref
   */

  options validvarname=any ;

  %bldLibNm(sesetl_sasLibRef=&cCONFORMLib
          , sesetl_OMRSASLibraryObjName=&Soldatatier_folder./&CONFLibObj./&CONFLibObj.
          , sesetl_useType=META
          , sesetl_validateLib=Y
          );

  /*
   *  Create libref connections
   */
  %bldLibNm(sesetl_sasLibRef=&cSDMLib
          , sesetl_OMRSASLibraryObjName=&Solsystem_folder./&SdmLibObj./&SdmLibObj.
          , sesetl_useType=BASE
          , sesetl_validateLib=Y
           ) ;

  %bldLibNm(sesetl_OMRSASLibraryObjName=&Solsystem_folder./&SdmLibObj./&SdmLibObj.,
               sesetl_useType=CON
               ) ;

  /**********************************************************************************************************/
  /* subsetting target dimensions by given ETL instance ID and store it in a separate table in WORK library */
  /**********************************************************************************************************/
  %sesFMS_getDimMbrSubset(&cETL_instanceID, &cTempFactDimMbr, &cSubTempFactDimMbr);
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 58, &cSCLEntryNm);
      %goto ERROR;
  %end ;


  /*
   *  We send end the Currency Codes, Dimension Type Codes and Dimension
   *  Codes in as macros.  Retrieve macro values and populate temporary
   *  work area data sets.
   */

  %sesFMS_macrosToDataSets( &cCurrencyWorkAreaDS,
                                      &cDimTypeWorkAreaDS,
                                      &cDimTypeWorkAreaDS1,
                                      &cDimWorkAreaDS,
                                      &cDimDimTypeDS,
                                      &cAnalysisWorkAreaDS_T,
                                      &cAnalysisDS_SDM_RKs,
                                      &cDLObjIdDS,
                                      &cCurrTimeMbrId_Map_PrdId_DS,
                                      &cCurrTimeMbrCode_Map_PrdId_DS,
                                      &cPriorTimeMbrId_Map_PrdId_DS,
                                      &cPriorTimeMbrCode_Map_PrdId_DS,
                                      &cCyclePrdIdsDS,
                                      &cIsFirstPrd_Map_PrdId_DS,
                                      &cRunYTDToPA_Map_PrdId_DS
                                      );

  %if &_FMErrorCode = 0 %then %do;
    %sesSOL_logError( &cLogEntry, L, 4, &cSCLEntryNm);
    %goto Error ;
  %end ;


  /*********************************
         Analysis
    ********************************/

  /*
   *  Match list of Analysis Codes from mid tier to import with DDS RK values
   */

  %sesFMS_getAnalysisDDSRKs( &cAnalysisWorkAreaDS_T, &cAnalysisWorkAreaDS, &cCONFORMLib);
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 4, &cSCLEntryNm);
      %goto ERROR;
  %end ;

   /*
    *  For Analysis, build a SAS Format and return a named list.
    *
    */
  %sesFMS_createFormat( &cAnalysisWorkAreaDS, &cAnalysisFormat, 1, lAnalysis, 1);

  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 9, &cAnalysisWorkAreaDS, &cSCLEntryNm);
      %goto ERROR;
   %end ;



    /*********************************
         Current Time Mbr
     ********************************/
    /*
     *  Match list of Current Time Mbr Codes from mid tier to import with DDS RK values
     */
  %sesFMS_getTimeDDSRKs(&cCurrTimeMbrCode_Map_PrdId_DS, &cCurrTimeMbrDDS_RKs_DS, &cCurrTimeMbr_DDSRKs_Map_PrdId_DS,
  &cCONFORMLib);
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 39, &cSCLEntryNm);
      %goto ERROR;
  %end ;

    /*
     *  For Current Time Mbr, build a SAS Format and return a named list (named on code).
     *      CODE = DDS RK (numeric)
     *      VALUE = Code
     */
  %sesFMS_createFormat(&cCurrTimeMbrDDS_RKs_DS, &cCurrTimeDdsRK_To_Code_Fmt, 1, lCurrTime_DdsRK_To_Code, 1 );
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 9, cCurrTimeMbrDDS_RKs_DS, &cSCLEntryNm);
      %goto ERROR;
  %end ;

  %sesFMS_createFormat(&cCurrTimeMbr_DDSRKs_Map_PrdId_DS, &cCurrTimeDdsRK_To_PrdId_Fmt, 1, 0, 0);
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 9, &cCurrTimeMbr_DDSRKs_Map_PrdId_DS, &cSCLEntryNm);
      %goto ERROR;
  %end ;


    /*********************************
         Prior Time Mbr
     ********************************/

    /*
     *  Match list of Prior Time Mbr Codes from mid tier to import with DDS RK values
     */
  %sesFMS_getTimeDDSRKs(&cPriorTimeMbrCode_Map_PrdId_DS, &cPriorTimeMbrDDS_RKs_DS, &cPriTimeMbr_DDSRKs_Map_PrdId_DS,
  &cCONFORMLib);
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 40, &cSCLEntryNm);
      %goto ERROR;
  %end ;

    /*
     *  For Prior Time Mbr, build a SAS Format and return a named list (named on code).
     *      CODE = DDS RK (numeric)
     *      VALUE = Code
     */
  %sesFMS_createFormat(&cPriorTimeMbrDDS_RKs_DS, &cPriorTimeDdsRK_To_Code_Fmt, 1, lPriorTime_DdsRK_To_Code, 1 );
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 9, cPriorTimeMbrDDS_RKs_DS, &cSCLEntryNm);
      %goto ERROR;
  %end;

  %sesFMS_createFormat(&cPriTimeMbr_DDSRKs_Map_PrdId_DS, &cPriorTimeDdsRK_To_PrdId_Fmt, 1,0, 0);
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 9, &cPriTimeMbr_DDSRKs_Map_PrdId_DS, &cSCLEntryNm);
      %goto ERROR;
  %end;

    /**********************************************************
         Build Map between DDS Time Mbr RK and Run YTDtoPA
     **********************************************************/
  %sesFMS_buildRunYTDToPAFormatDS(&cCurrTimeMbrCode_Map_PrdId_DS, &cCurrTimeMbrDDS_RKs_DS, &cRunYTDToPA_Map_PrdId_DS,
  &cRunYTDToPA_Map_DDSTimeRKs_DS, &cConformLib);
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 54, &cSCLEntryNm);
      %goto ERROR;
  %end ;

  %sesFMS_createFormat(&cRunYTDToPA_Map_DDSTimeRKs_DS, &cRunYTDToPA_Fmt, 1, 0, 0);
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 9, cRunYTDToPA_Map_DDSTimeRKs_DS, &cSCLEntryNm);
      %goto ERROR;
  %end ;

    /**********************************************************
         Build Map between DDS Time Mbr RK and IsFirstPrd
     **********************************************************/
  %sesFMS_buildIsFirstPrdFormatDS(&cCurrTimeMbrCode_Map_PrdId_DS, &cCurrTimeMbrDDS_RKs_DS, &cIsFirstPrd_Map_PrdId_DS,
  &cIsFirstPrd_Map_DDSTimeRKs_DS, &cConformLib);
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 55, &cSCLEntryNm);
      %goto ERROR;
  %end ;

  %sesFMS_createFormat(&cIsFirstPrd_Map_DDSTimeRKs_DS, &cIsFirstPrd_Fmt, 1, 0, 0);
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 9, &cIsFirstPrd_Map_DDSTimeRKs_DS, &cSCLEntryNm);
      %goto ERROR;
  %end ;


  /*********************************
   *********************************
      (All or) Other dimension
   *********************************
   *********************************/
  /* get DDS RKS for the "Other Dimension"  and create SAS Format*/
  %sesFMS_getAllDDSRKsFmt(&cSubTempFactDimMbr, &cCONFORMLib, &cDimDimTypeDS);
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 57, &cSCLEntryNm);
      %goto ERROR;
  %end ;


  /*
   *  Retrieve a list of distinct accounts with their associated account types
   */
  %sesFMS_getAccountTypes( &cAccountWorkAreaDS, &cSDMLib, &nAccountDimId, &nAcctTypePropertyId, &cCONFORMLib,
  &nProcessedDTTM_To);
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 3, &cSCLEntryNm);
      %goto ERROR;
  %end ;


    /*
     *  Build format of DataLoad Object Ids
     */
  %sesFMS_createFormat( &cDLObjIdDS, &cDLObjIdFormat, 0, lDLObjId, 0 );
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 51, &cSCLEntryNm);
      %goto ERROR;
  %end ;

    /*
     *  For Currency leaf nodes, build a SAS Format and return a named list.  There should be a spec%ific
     *  currency hierarchy associated to the Cycle, we are importing to.
     *
     */
  %sesFMS_createFormat( &cCurrencyWorkAreaDS, &cCurrencyFormat, 0, lCurrency, 0 );
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 5, &cSCLEntryNm);
      %goto ERROR;
  %end ;


    /*
     *  For Account and account types, build a SAS Format and return a named list.
     */
  %sesFMS_createFormat( &cAccountWorkAreaDS, &cAccountFormat, 1, lAccount, 0 );
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 6, &cSCLEntryNm);
      %goto ERROR;
  %end ;


    /*
     *  For the Cycle's Dimension Types, build a SAS Format and return a named list.
     */
  %sesFMS_createFormat( &cDimTypeWorkAreaDS, &cDimTypeFormat, 0, lCycleDimTypes, 1 );
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 7, &cSCLEntryNm);
      %goto ERROR;
  %end ;

    /*
     *  For the Cycle's Dimension Types, build a SAS Format and return a named list.
     */
  %sesFMS_createFormat( &cDimTypeWorkAreaDS1, &cDimTypeColFormat, 0, lCycleDimTypes1, 0 );
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 7, &cSCLEntryNm);
      %goto ERROR;
  %end ;

    /*
     *  Build a list of dim types in fact set order.
     */
  %sesFMS_createDimTypeFactSetOrd( &cDimTypeWorkAreaDS, lDimTypeFactSetOrder );
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 7, &cSCLEntryNm);
      %goto ERROR;
  %end ;

    /*
     *  Note:  the SOURCE dimension type does not exist in the DDS, but will in the mart, as such,
     *         we need to remove the SOURCE dimension type from this list
     */

  proc sql noprint;
    create table lOrigCycleDimTypes
      as select *
         from lCycleDimTypes
        where CODE NE "&cSourceDimTypeCode";
    quit;


    /*
     *  Finds the currently active records from the DDS Dimension Type table and loads into a work area data set
     */
  %sesFMS_getDDSDimensionTypes( &cCONFORMLib, &cDDSDimTypeTableNm,
                                 &cDDSDimTypeWorkAreaDS, &cRKtoCodeLinkDS,
                                 &cDimTypeRKColDS );
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 8, &cSCLEntryNm);
      %goto ERROR;
  %end ;


    /*
     *  For the DDS Dimension Types, build a SAS Format and return a named list.  This gives us an association
     *  between the Mart's dimension type name and the column name of the DDS gl_transaction_sum table.
     *
     *  It is worth noting that this list is/should be a comprehensive list of dimension types in the DDS, and
     *  not necessarily the list of dimension types in the Mart's Cycle.
     */
  %sesFMS_createFormat( &cDDSDimTypeWorkAreaDS, &cDDSDimTypeFormat, 0, lDDSDimTypes, 1 );
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 9, &cDDSDimTypeWorkAreaDS, &cSCLEntryNm);
      %goto ERROR;
  %end ;


    /*
     *  We need a list that associations the DimType Code to each dimensions
     *  corresponding member tables in the DDS.  This is so we can convert
     *  DDS RK(s) to Codes near the end of this process.
     */
  %sesFMS_createFormat( &cRKtoCodeLinkDS, &cRKtoCodeLinkFormat, 0, lRKtoCode, 1 );
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 9, cRKtoCodeLinkDS, &cSCLEntryNm);
      %goto ERROR;
  %end ;


    /*
     *  We need a list that associations the DimType Code to each dimensions
     *  corresponding member tables RK Column in the DDS.  This is so we can convert
     *  DDS RK(s) to Codes near the end of this process.
     */
  %sesFMS_createFormat( &cDimTypeRKColDS, &cDimTypeRKColFormat, 0, lDimTypeRKCol, 1 );
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 9, &cDimTypeRKColDS, &cSCLEntryNm);
      %goto ERROR;
  %end ;

    /*
     *  There will be a specific list of Dimension Types associated to the Mart's Cycle.  We need to make
     *  sure that those dimension types also exist in the DDS Dimension Type table.  Otherwise, we have no
     *  hope of matching Cycle dimension types to columns in the DDS gl_transaction_sum table.
     *
     */

  %sesFMS_verifyNgetDimTypeRK( lCycleDimTypes, lDDSDimTypes, lCycleDimTypesRKs );
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 10, &cSCLEntryNm);
      %goto ERROR;
  %end ;

    /*
     *  Verify that the DDS GL Transaction table contains the stardard model columns.  Customer can add columns
     *  to the model without any problems.  However if columns are removed or renamed, we can expect to have
     *  errors in this and other processes.
     */
  %sesFMS_verifyDDSTransModel( &cCONFORMLib, &cDDSTransTableNm, lCycleDimTypesRKs, lDDSTranCols );
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 11, &cSCLEntryNm);
      %goto ERROR;
  %end ;


  /*
  *  Create a column list to sort, summarization and merging transaction tables.  We need to remove
  *  the time period column from the list because we will be sorting and merging columns between
  *  current and prior period and we want the transactions to match.
  */
  proc sql noprint;
    create table lTranCols as
    select value
      from lCycleDimTypesRKs
     where upcase(value) NE "AFFECTED_TIME_PERIOD_RK";
  quit;

  /*
   *  Get a subset of transaction from the DDS and dump into two work area data set.  One data set with
   *  transactions flagged as YTD amounts, other flagged as either (Period activity amounts or Statistical
   *  accounts).
   *
   *  Subset by:
   *     > range of Process_dttm
   *     > list of current time period(s)
   *     > list of analysis(s)
   *     > currency hierarchy leaf nodes
   *
   *  Where the following is true, Transactions are dumped to a work area YTD data set:
   *     > transaction_amt_ytd_flg is 0 (false)
   *     > account type is statistical
   *     > current period is the first period of the fiscal year and account type is I/S
   *  Otherwise transactions are dumped to a work area Period Activity data set
   *     (i.e. transactions are imported but not converted)
   *
   */

  %sesFMS_getTransFromDDS( &cCONFORMLib, &cDDSTransTableNm,
                                     &nProcessedDTTM_From, &nProcessedDTTM_To,
                                     &cCurrTimeDdsRK_To_Code_Fmt, &cAnalysisFormat, &cCurrencyFormat,
                                     &cCurrentTransYtdDS, &cCurrentTransPaDS,
                                     &cOutputYtdColBlankDS,
                                     &cAccountFormat, &cCurrentAmtColNm,
                                     &cIsFirstPrd_Fmt, lCycleDimTypesRKs,
                                     &cAcctTypeRevenue, &cAcctTypeExpense,
                                     &cAcctTypeStatFlow, &cAcctTypeNonFrequency,
                                     &cRunYTDToPA_Fmt, &cCurrTimeDdsRK_To_PrdId_Fmt,
                                     &cTempPrdColNm);
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 12, &cSCLEntryNm);
      %goto ERROR;
  %end ;


    /*
     *  Get a subset of the prior period GL transactions from the DDS and dump into a work area data set.
     *  We only want those transactions where the YTD amount flag is true.
     *
     *  Subset by:
     *     > prior time period
     *     > transaction_amt_ytd_flg eq true
     *     > analysis hierarchy leaf nodes
     *     > currency hierarchy leaf nodes
     */
  %sesFMS_getPriorTransFromDDS( &cCONFORMLib, &cDDSTransTableNm,
                                          &cPriorTimeDdsRK_To_Code_Fmt, &cAnalysisFormat, &cCurrencyFormat,
                                          &cPriorTransYtdDS, &cPriorAmtColNm,
                                          &cPriorTimeDdsRK_To_PrdId_Fmt,
                                          &cTempPrdColNm);
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 16, &cSCLEntryNm);
      %goto ERROR;
  %end ;

  /*  Build a list that include PERIOD_ID Column  */
  /* Copy */
  data lTranCols_PrdID;
    set lTranCols;
  run;

  /* and add */
  proc sql noprint;
    insert into  lTranCols_PrdId values ("&cTempPrdColNm");
  quit;

  /*  Sort and summarize the Prior Period YTD transactions  */
  %sesFMS_sortDataSet( &cPriorTransYtdDS, lTranCols_PrdId);
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 18, &cSCLEntryNm);
      %goto ERROR;
  %end ;

  %sesFMS_summarizeDataSet( &cPriorTransYtdDS, lTranCols_PrdId, &cPriorAmtColNm, 1);
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 19, &cSCLEntryNm);
      %goto ERROR;
  %end ;

   /*  Sort and summarize the Current Period YTD subsetted work area data set  */
  %sesFMS_sortDataSet( &cCurrentTransYtdDS, lTranCols_PrdId);
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 20, &cSCLEntryNm);
      %goto ERROR;
  %end ;

  %sesFMS_summarizeDataSet( &cCurrentTransYtdDS, lTranCols_PrdId, &cCurrentAmtColNm, 1);
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 21, &cSCLEntryNm);
      %goto ERROR;
  %end ;

  /***************************************************************
  *
  *  Do the math on YTD transactions (current - prior period)
  *
  ***************************************************************/

  %sesFMS_calcPeriodActivity( &cCurrentTransYtdDS, &cPriorTransYtdDS,
                                        lTranCols_PrdId, &cCurrentAmtColNm,
                                        &cPriorAmtColNm, &cConvertedGlTransDS,
                                        &cTempPrdColNm);
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 22, &cSCLEntryNm);
      %goto ERROR;
  %end ;


    /*
     *  Append newly derived Period Activity trans. to non-derived Period Activity trans.
     */
  %sesFMS_appendDS(&cConvertedGlTransDS, &cCurrentTransPaDS);
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 15, &cCurrentTransPaDS, &cConvertedGlTransDS, &cSCLEntryNm);
      %goto ERROR;
  %end ;

    /*
     *  Add the Source column with the Base Code value.  We add this here because this dimension
     *  only exists in the FMS Mart.
     */
  %sesFMS_addSourceCode( &cConvertedGLTransDS, &cSourceColNm, &cSource_BaseCode );
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 29, &cConvertedGlTransDS, SOURCE_CD, &cSCLEntryNm);
      %goto ERROR;
  %end ;

    /*
     *  Fetch Journal Transactions
     *
     *  Subset by:
     *     > range of Process_dttm
     *     > list of current time period(s)
     *     > list of analysis(es)
     *     > part of listed currencies
     *
     */

  %global nJrnlCnt;
  %let nJrnlCnt=0;
  %sesFMS_getJournsFromDDS( &cCONFORMLib, &nProcessedDTTM_From,
                                      &nProcessedDTTM_To, &cCurrTimeDdsRK_To_Code_Fmt,
                                      &cAnalysisFormat, &cCurrencyFormat,
                                      &cJournalDS, &cAccountFormat,
                                      lCycleDimTypesRKs, &cDDSJrnlDS,
                                      &cDDSJrnlDetailDS, &cSourceColNm,
                                      &cSource_BaseJournCode
                                      );
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 44, &cSCLEntryNm);
      %goto ERROR;
  %end ;

    /*
     *  Append Journal DS to Basefact DS, only if we have found journals
     */
  %if &nJrnlCnt ne 0 %then %do;
    %sesFMS_appendDS( &cConvertedGLTransDS, &cJournalDS );
    %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 15, &cJournalDS, &cConvertedGLTransDS, &cSCLEntryNm);
      %goto ERROR;
    %end ;
  %end ;

    /*  Does our resulting table have any records in it?  */

  %if %sysfunc(exist(&cConvertedGLTransDS)) %then %do;
    proc sql noprint;
      select count(*)
        into :nCnt
        from  &cConvertedGLTransDS;
      quit;
  %end;
  %else %do;
    %sesSOL_logError( &cLogEntry, L, 24, &cConvertedGlTransDS, &cSCLEntryNm);
      %goto ERROR;
  %end ;

  %if &nCnt eq 0 %then %do;
      /*  Create a table of all Warnings */
    %sesFMS_buildStatusDS(&cCurrTimeMbrId_Map_PrdId_DS, &cAnalysisDS_SDM_RKs, &cStatusDS, 3);
    %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 47, &cSCLEntryNm);
      %goto ERROR;
    %end ;

      /*  No records found to update, set status to warning  */
    %let fmsetl_statusId=3;  /* success=0, critical error=1, validation error=2, warning=3  */
    %sesSOL_logError( &cLogEntry, L, 25, &cSCLEntryNm);
  %end;
  %else %do; /* Very long else */
      /*  For this set of sort and summaries, we want to include the SOURCE_CD column  */
    proc sql noprint;
      create table lTranCols_with_SOURCE as select * from lTranCols;
      insert into  lTranCols_with_SOURCE values( "AFFECTED_TIME_PERIOD_RK" );
      insert into  lTranCols_with_SOURCE values( "&cSourceColNm" );
    quit;

      /*  Sort and summarize work area data set  */
    %sesFMS_sortDataSet( &cConvertedGLTransDS, lTranCols_with_SOURCE );
    %if &_FMErrorCode = 0 %then %do;
          %sesSOL_logError( &cLogEntry, L, 26, &cConvertedGlTransDS, &cSCLEntryNm);
          %goto ERROR;
      %end ;
      %sesFMS_summarizeDataSet( &cConvertedGLTransDS, lTranCols_with_SOURCE, &cTransAmtColNm, 0 );
    %if &_FMErrorCode = 0 %then %do;
          %sesSOL_logError( &cLogEntry, L, 27, &cConvertedGlTransDS, &cSCLEntryNm);
          %goto ERROR;
      %end ;

      /*  Convert DDS RK's to DDS Codes  */
      %sesFMS_convertDdsRksToCodes( &cConvertedGLTransDS, &cCONFORMLib, lRKtoCode, lDimTypeRKCol,
    &cCurrTimeDdsRK_To_Code_Fmt, lCycleDimTypesRKs);
    %if &_FMErrorCode = 0 %then %do;
          %sesSOL_logError( &cLogEntry, L, 41, &cConvertedGlTransDS, &cSCLEntryNm);
          %goto ERROR;
      %end ;

    %put BEGIN imptdata **************** ************* ******************;

    /*
     *  Run validation and import into the Cycles basefact table
     *  depending on TRADER in cycle input
     */

    /* Check if Trader exsits */
    %let TRADER_EXIST = 0;
    proc sql noprint;
         select count(*)
         into :TRADER_EXIST
         from &cDimDimTypeDS
         where kupcase(strip(dim_type_cd)) = 'TRADER'
         ;
    quit;

    %if  &TRADER_EXIST > 0 %then %do;
      %sesFMS_imptdataWithTr (gl_trans_sum_table=&cConvertedGLTransDS,
                    update_tablename=&cTmpFactWork_Nm,
                    ic_check_ind=&cCkIcTradingPartners,
                    non_ic_check_ind=&cCkNonIcTradingMembers,
                    cycle_dim_table=&cDimWorkAreaDS,
                    fmsetl_ProcessedDTTM_To=&nProcessedDTTM_To,
                    dataLoadObjTypeId=&cDLObjTypeId,
                    dataLoadObjId=&cDLObjId,
                    dimTypeWorkAreaDS=&cDimTypeWorkAreaDS);
    %end;
    %else %do;
      %sesFMS_imptdataWithoutTr (gl_trans_sum_table=&cConvertedGLTransDS,
                    update_tablename=&cTmpFactWork_Nm,
                    cycle_dim_table=&cDimWorkAreaDS,
                    fmsetl_ProcessedDTTM_To=&nProcessedDTTM_To,
                    dataLoadObjTypeId=&cDLObjTypeId,
                    dataLoadObjId=&cDLObjId,
                    dimTypeWorkAreaDS=&cDimTypeWorkAreaDS);
    %end;

    /********************************************************************************
     *
     *  Gather some stats on the outcome of the job, to send back to calling code
     *
     ********************************************************************************/

      /*  Check for a Critical Error Occurance  */
    %if &fmsetl_statusId = 1 %then %do;
        %goto ERROR;
    %end ;


      /*  Create a table of all Successful imports */
    %sesFMS_buildStatusDS(&cCurrTimeMbrId_Map_PrdId_DS, &cAnalysisDS_SDM_RKs, &cStatusDS, 0);
    %if &_FMErrorCode = 0 %then %do;
        %sesSOL_logError( &cLogEntry, L, 47, &cSCLEntryNm);
        %goto ERROR;
    %end ;


      /*  Count BadRecs */
    %sesFMS_updateInvalidStatusCnt( &cStatusDS, work.badrecs, INVALID_ERROR_CNT );
    %if &_FMErrorCode = 0 %then %do;
        %sesSOL_logError( &cLogEntry, L, 48, work.badrecs, &cSCLEntryNm);
        %goto ERROR;
    %end ;


      /*  Count Org eq Trader errors */
    %sesFMS_updateInvalidStatusCnt( &cStatusDS, work.org_eq_trader, ORG_EQ_TRADER_CNT );
    %if &_FMErrorCode = 0 %then %do;
        %sesSOL_logError( &cLogEntry, L, 48, work.org_eq_trader, &cSCLEntryNm);
        %goto ERROR;
    %end ;


      /*  Count IC errors */
    %sesFMS_updateInvalidStatusCnt( &cStatusDS, work.icrecs, IC_ERROR_CNT );
    %if &_FMErrorCode = 0 %then %do;
        %sesSOL_logError( &cLogEntry, L, 48, work.icrecs, &cSCLEntryNm);
        %goto ERROR;
    %end ;


      /*  Count Non IC errors */
    %sesFMS_updateInvalidStatusCnt( &cStatusDS, work.nonicrecs, NON_IC_ERROR_CNT );
    %if &_FMErrorCode = 0 %then %do;
        %sesSOL_logError( &cLogEntry, L, 48, work.nonicrecs, &cSCLEntryNm);
        %goto ERROR;
    %end ;


      /*  Count successful Imports  */
    %sesFMS_updateValidStatusCnt( &cStatusDS, &cTmpFactWork_Nm, IMPORTED_TRANS_CNT );
    %if &_FMErrorCode = 0 %then %do;
        %sesSOL_logError( &cLogEntry, L, 49, &cTmpFactWork_Nm, &cSCLEntryNm);
        %goto ERROR;
    %end ;


      /*  Count Zero values  */
    %sesFMS_updateZeroStatusCnt( &cStatusDS, &cTmpFactWork_Nm, ZERO_TRANS_CNT);
    %if &_FMErrorCode = 0 %then %do;
        %sesSOL_logError( &cLogEntry, L, 56, &cTmpFactWork_Nm, &cSCLEntryNm);
        %goto ERROR;
    %end ;


    /*  Print Error Report(s)  */
    %sesFMS_generateErrorReports( &cStatusDS, lDimTypeFactSetOrder, WORK.BADRECS,
                                            WORK.ORG_EQ_TRADER, WORK.ICRECS,
                                            WORK.NONICRECS, &cDLObjId);

    %if &_FMErrorCode = 0 %then %do;
        %sesSOL_logError( &cLogEntry, L, 36, &cSCLEntryNm);
        %goto ERROR;
    %end ;


      /*  Create temporary REMOVE table for matching actions  */
    %sesFMS_replaceMatching( &cTmpRemoveROFactSDM_Nm, &cTmpRemoveRWFactSDM_Nm,
                    &cStatusDS, &nActionType, lDimTypeFactSetOrder,
                    &nBaseMbrSDM_Id, &nBaseJournMbrSDM_Id,
                    &cSdmLib, &nSdmROFactSetId, &nSdmRWFactSetId, &cDLObjTypeId,
                    &cTmpFactWork_Nm );
    %if &_FMErrorCode = 0 %then %do;
        %sesSOL_logError( &cLogEntry, L, 53, &cSCLEntryNm);
        %goto ERROR;
    %end ;

      /*  Upload Facts to SASSDM  */
    %sesFMS_uploadTempFactToSDM( &cTmpFactSDM_Nm, &cTmpFactWork_Nm, &cStatusDS );
    %if &_FMErrorCode = 0 %then %do;
        %sesSOL_logError( &cLogEntry, L, 52, &cTmpFactWork_Nm, &cTmpFactSDM_Nm, &cSCLEntryNm);
        %goto ERROR;
    %end ;

  %end;


    /*  Upload Status Report to SASSDM  */
  %sesFMS_uploadStatusTableToSDM(&cStatusDS, &cSDMStatusDS);
  %if &_FMErrorCode = 0 %then %do;
      %sesSOL_logError( &cLogEntry, L, 52, &cSDMStatusDS, &cStatusDS, &cSCLEntryNm);
      %goto ERROR;
  %end ;


  /*  Dump the Time  */
  %let nTrackTime=%sysevalf(%qsysfunc(datetime()) - &nTrackTime); ;
  %let TimeData=%sesFMS_getTimer;
  %put TimeData:&TimeData;

  %if &TimeData > 0  %then %do;
      %put TIMER: total time to run YTD to Period Activity conversion: &nTrackTime ;
  %end ;

  %goto THEEND;

  %ERROR:
    %let fmsetl_statusId=1;  /* success=0, critical error=1, validation error=2, warning=3  */

  %THEEND:

%mend sesFMS_LoadCycleBaseFact;


/**********************************************************
 * sesFMS_macrosToDataSets.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_macrosToDataSets.sas                                                           */
/*  Purpose: Parse information from macro variables into data sets.                          */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */
%macro sesFMS_macrosToDataSets (cCurrencyDS,
                                 cDimTypeDS,
                                 cDimTypeDS1,
                                 cDimDS,
                                 cDimDimTypeDS,
                                 cAnalysisDS,
                                 cAnalysisDS_SDM_RKs,
                                 cDLObjIdDS,
                                 cCurrTimeMbrIdsSDM,
                                 cCurrTimeMbrCodes,
                                 cPriorTimeMbrIdsSDM,
                                 cPriorTimeMbrCodes,
                                 cCyclePrdIds,
                                 cIsFirstPrdYr,
                                 cRunYTDToPA
                                 );

  %let cMethodNm = sesFMS_macrosToDataSets ;
  %let nTrackTime= %sysfunc(datetime());

  data &cCurrencyDS       (keep=CODE VALUE)
     &cDimTypeDS          (keep=CODE VALUE)
     &cDimTypeDS1         (keep=CODE VALUE)
     &cDimDS              (keep=CODE VALUE)
     &cDimDimTypeDS       (keep=DIM_TYPE_CD DIM_CD)
     &cAnalysisDS         (keep=CODE VALUE)
     &cAnalysisDS_SDM_RKs (keep=CODE VALUE)
     &cDLObjIdDS          (keep=CODE VALUE)
     &cCurrTimeMbrIdsSDM  (keep=CODE VALUE)
     &cCurrTimeMbrCodes   (keep=CODE VALUE)
     &cPriorTimeMbrIdsSDM (keep=CODE VALUE)
     &cPriorTimeMbrCodes  (keep=CODE VALUE)
     &cCyclePrdIds        (keep=CODE VALUE)
     &cIsFirstPrdYr       (keep=CODE VALUE)
     &cRunYTDToPA         (keep=CODE VALUE)
     ;

     length CODE $64;
     length DIM_TYPE_CD $64;
     length DIM_CD $64;
     length NUMCODE 8;
     length VALUE $64;
     length TMPCODE $254;
     length TIME_MBR_SDM_ID 8;
     length ANALYSIS_MBR_SDM_ID 8;


      do nXX = 1 to 9999999999;
        CODE = strip( kscan("&fmsetl_dimensionTypeCodes", nXX, "09"x) );
        if CODE eq "" then leave;
        NUMCODE = .; VALUE = "OK";
        output &cDimTypeDS;
        VALUE = CODE;
        CODE = VALUE;
        if (klength(VALUE) gt 26) then do;
          VALUE = kcompress("_" !! put(nXX, 12.) !! "_" !! strip(VALUE));
          VALUE = ksubstr(VALUE, 1, min(klength(VALUE), 26));
        end;
        output &cDimTypeDS1;
      end;

      do nXX = 1 to 9999999999;
        CODE = strip( kscan("&fmsetl_dimensionCodes", nXX, "09"x) );
        if CODE eq "" then leave;
        NUMCODE = .;
        VALUE = "OK";
        output &cDimDS;
      end;

      do nXX = 1 to 9999999999;
        DIM_CD      = strip( kscan("&fmsetl_dimensionCodes", nXX, "09"x) );
        DIM_TYPE_CD = strip( kscan("&fmsetl_dimensionTypeCodes", nXX, "09"x) );
        if DIM_TYPE_CD eq "" then leave;
      if DIM_CD eq "" then leave;
        output &cDimDimTypeDS;
       end;

      do nXX = 1 to 9999999999;
          CODE = strip( kscan("&fmsetl_currencyCodes", nXX, "09"x) );
          if CODE eq "" then leave;
          NUMCODE = .; VALUE = "OK";
          output &cCurrencyDS;
      end;


      do nXX = 1 to 9999999999;
          CODE = strip( kscan("&fmsetl_analysisCodes", nXX, "09"x) );
          if CODE eq "" then leave;
          NUMCODE = .;
          VALUE = "OK";
          output &cAnalysisDS;
      end;

      do nXX = 1 to 9999999999;
        CODE = strip( kscan("&fmsetl_analysisIds_SDM", nXX, "09"x) );  /* tab delimiter */
        if CODE eq "" then leave; NUMCODE = .;
        VALUE = "OK";
        output &cAnalysisDS_SDM_RKs;
      end;

      do nXX = 1 to 9999999999; TMPCODE = strip( kscan("&fmsetl_dataLoadObjectIds", nXX, "09"x) );
        if TMPCODE eq "" then leave;
        TIME_MBR_SDM_ID = input(kscan(TMPCODE, 1, ":"), 32.);
        ANALYSIS_MBR_SDM_ID = input(kscan(TMPCODE, 2, ":"), 32.);
        VALUE = input(kscan(TMPCODE, 3, ":"), 32.);
        CODE = strip(put(TIME_MBR_SDM_ID, Z30.)) || "+" || strip(put(ANALYSIS_MBR_SDM_ID, Z30.));
        output &cDLObjIdDS;
      end;

      do nXX = 1 to 9999999999;
        TMPCODE = strip( kscan("&fmsetl_currentTimeMbrIds_SDM", nXX, "09"x) );
        if TMPCODE eq "" then leave; /*  Format = PeriodID:TimeMbrID  */
        CODE = input(kscan(TMPCODE, 1, ":"), 32.);
        VALUE = input(kscan(TMPCODE, 2, ":"), 32.);
        output &cCurrTimeMbrIdsSDM;
    end
       ;

      do nXX = 1 to 9999999999;
        TMPCODE = strip( kscan("&fmsetl_currentTimeMbrCodes", nXX, "09"x) );
        if TMPCODE eq "" then leave; /*  Format = PeriodID:TimeMbrCodes  */
        CODE = input(kscan(TMPCODE, 1, ":"), 32.);
        VALUE = put(kscan(TMPCODE, 2, ":"), 32.);
        output &cCurrTimeMbrCodes;
      end
       ;

      do nXX = 1 to 9999999999;
        TMPCODE = strip( kscan("&fmsetl_priorTimeMbrIds_SDM", nXX, "09"x) );
        if TMPCODE eq "" then leave; /*  Format = PeriodID:TimeMbrID  */
        CODE = input(kscan(TMPCODE, 1, ":"), 32.);
        VALUE = input(kscan(TMPCODE, 2, ":"), 32.);
        output &cPriorTimeMbrIdsSDM;
    end;

      do nXX = 1 to 9999999999;
        TMPCODE = strip( kscan("&fmsetl_priorTimeMbrCodes", nXX, "09"x) );
        if TMPCODE eq "" then leave; /*  Format =
        PeriodID:TimeMbrCodes  */ CODE = input(kscan(TMPCODE, 1, ":"), 32.);
        VALUE = put(kscan(TMPCODE, 2, ":"), 32.);
        output &cPriorTimeMbrCodes;
    end;

      do nXX = 1 to 9999999999;
        TMPCODE = strip( kscan("&fmsetl_cyclePrdIds", nXX, "09"x) );
        if TMPCODE eq "" then leave; /*  Format = PeriodID:PeriodID  */
        CODE = input(kscan(TMPCODE, 1, ":"), 32.);
        VALUE = input(kscan(TMPCODE, 2, ":"), 32.);
        output &cCyclePrdIds;
    end;

      do nXX = 1 to 9999999999;
        TMPCODE = strip( kscan("&fmsetl_isFirstPeriodOfFiscalYear", nXX, "09"x) );
        if TMPCODE eq "" then leave; /*  Format = PeriodID:Y or N  */
        CODE = input(kscan(TMPCODE, 1, ":"), 32.);
      VALUE = put(kscan(TMPCODE, 2, ":"), 32.);
      output &cIsFirstPrdYr;
    end;

      do nXX = 1 to 9999999999;
        TMPCODE = strip( kscan("&fmsetl_runYTDtoPeriodActivity", nXX, "09"x) );
        if TMPCODE eq "" then leave; /*  Format = PeriodID:Y or N  */
        CODE = input(kscan(TMPCODE, 1, ":"), 32.);
        VALUE = put(kscan(TMPCODE, 2, ":"), 32.);
        output &cRunYTDToPA;
      end;

  run;


  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %let error_number=1;
    %goto ERRORRETURN;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );

  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %let _FMErrorCode=0;
    %sesSOL_logError( YTDConv, L, &error_number, &cMethodNm);

  %THEEND:

%mend sesFMS_macrosToDataSets;


/**********************************************************
 * sesFMS_replaceMatching.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_replaceMatching.sas                                                           */
/*  Purpose: Replace matching process.                                                       */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */
%macro sesFMS_replaceMatching(cTmpROFactSDM_Nm,
                                cTmpRWFactSDM_Nm,
                                cStatusDS,
                                nActionType,
                                lCycleDimTypes,
                                nBaseMbrSDM_Id,
                                nBaseJournMbrSDM_Id,
                                cSdmLib,
                                nSdmROFactSetId,
                                nSdmRWFactSetId,
                                cDLObjTypeId,
                                cNewFactsConCatDS);

  %let cMethodNm = sesFMS_replaceMatching;
  %let cSdmROFactW_BaseDS = WORK._SdmROFactW_Base;
  %let cSdmROFactNoBaseDS = WORK._SdmROFactNoBase;
  %let cSdmRWFactW_BaseDS = WORK._SdmRWFactW_Base;
  %let cSdmRWFactNoBaseDS = WORK._SdmRWFactNoBase;
  %let cNewFactDS = WORK._DDSFactsToImp ;
  %let cNewFactDS1 = WORK._DDSFactsToImp1 ;
  %let cNewFactDS2 = WORK._DDSFactsToImp2 ;
  %let cMatchRO_W_Base = WORK._matchRO_W_Base ;
  %let cMatchRO_NoBase = WORK._matchRO_NoBase ;
  %let cMatchRW_W_Base = WORK._matchRW_W_Base ;
  %let cMatchRW_NoBase = WORK._matchRW_NoBase ;
  %let cSdmFactSetNm = %qsysfunc(strip(&cSdmLib)).SAS_FACT;
  %let nowDTTM = %qsysfunc(datetime());
  %let nTrackTime = %qsysfunc(datetime());

  /*  If this is not a Replace Matching action, return  */
  %if &nActionType NE 4 and &nActionType NE 5 %then %do;
    %goto THEEND;
  %end;

  /*  Clean up, in case we had a previously aborted attempt  */
  %if %sysfunc(exist(&cSdmROFactW_BaseDS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cSdmROFactW_BaseDS, 2, '.');
    quit;
  %end;

  %if %sysfunc(exist(&cSdmROFactNoBaseDS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cSdmROFactNoBaseDS, 2, '.');
    quit;
  %end;

  %if %sysfunc(exist(&cSdmRWFactW_BaseDS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cSdmRWFactW_BaseDS, 2, '.');
    quit;
  %end;

  %if %sysfunc(exist(&cSdmRWFactNoBaseDS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cSdmRWFactNoBaseDS, 2, '.');
    quit;
  %end;

  %if %sysfunc(exist(&cNewFactDS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cNewFactDS, 2, '.');
    quit;
  %end;

  %if %sysfunc(exist(&cNewFactDS1)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cNewFactDS1, 2, '.');
    quit;
  %end;


  %if %sysfunc(exist(&cNewFactDS2)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cNewFactDS2, 2, '.');
    quit;
  %end;

  %if %sysfunc(exist(&cMatchRO_W_Base)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cMatchRO_W_Base, 2, '.');
    quit;
  %end;

  %if %sysfunc(exist(&cMatchRO_NoBase)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cMatchRO_NoBase, 2, '.');
    quit;
  %end;

  %if %sysfunc(exist(&cMatchRW_W_Base)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cMatchRW_W_Base, 2, '.');
    quit;
  %end;

  %if %sysfunc(exist(&cMatchRW_NoBase)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cMatchRW_NoBase, 2, '.');
    quit;
  %end;

  /*  Get the list of Time/Analysis Ids that have okay import status   */
  proc sql noprint;
    select count(*)
      into :num_data
      from &cStatusDS
      where STATUS_ID = 0;
  quit;

  proc sql noprint;
      create table lTimeAnalysisIds as
      select "(TIME = " || strip(put(TIME_ID, 32.)) || " and ANALYSIS = " || strip(put(ANALYSIS_ID, 32.)) || ")" as code
        from &cStatusDS
        where STATUS_ID = 0;
  quit;

  proc sql noprint;
       connect to &sesetl_EngineType ( &sesetl_BestLogin &sesetl_ConnectionOptions );
       execute(CREATE TABLE &cTmpROFactSDM_Nm (
               FACT_IDX INT(11) NOT NULL,
               PRIMARY KEY (FACT_IDX)
               )engine=myISAM
               ) by &sesetl_EngineType ;
       execute(CREATE TABLE &cTmpRWFactSDM_Nm (
               FACT_IDX INT(11) NOT NULL,
               PRIMARY KEY (FACT_IDX)
               )engine=myISAM
               ) by &sesetl_EngineType ;
       disconnect from &sesetl_EngineType ;
  quit;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %sesSOL_logError( YTDConv, L, 50, &cMethodNm) ;
      %goto RETURNZERO;
  %end;

  /*
   *  Extract appropriate facts from SDM into a temporary data set.
   *  Where criteria is for a RO fact set, RW fact set, Analysis list,
   *  and time period
   */
  %sesFMS_replaceMatchingExtract( lTimeAnalysisIds, &lCycleDimTypes, &cSdmFactSetNm,
         &cSdmROFactW_BaseDS, &cSdmROFactNoBaseDS,
         &cSdmRWFactW_BaseDS, &cSdmRWFactNoBaseDS,
         &nSdmROFactSetId, &nSdmRWFactSetId, &cDLObjTypeId,
         &nBaseMbrSDM_Id, &nBaseJournMbrSDM_Id );
  %if  &_FMErrorCode = 0 %then %do;
    %sesSOL_logError( YTDConv, L, 46, &cMethodNm) ;
        %goto RETURNZERO ;
  %end;

  /*  Split work area fact DS member concat rks  */
  %sesFMS_replaceMatSplitNewFacts( &lCycleDimTypes, &cNewFactDS, &cNewFactsConCatDS);
  %if  &_FMErrorCode = 0 %then %do;
    %goto RETURNZERO ;
  %end;

  /*  Sort New Facts on all Dim Types  */
  %sesFMS_replaceMatchingSort(&lCycleDimTypes, &cNewFactDS, &cNewFactDS1, 1, &nActionType );
  %if  &_FMErrorCode = 0 %then %do;
    %goto RETURNZERO ;
  %end;

  /*  Sort New Facts on all Dim Types, except possibly CURRENCY and SOURCE  */
  %sesFMS_replaceMatchingSort(&lCycleDimTypes, &cNewFactDS, &cNewFactDS2, 0, &nActionType );
  %if  &_FMErrorCode = 0 %then %do;
    %goto RETURNZERO ;
  %end;

  /*  Sort SDM Fact extracts  */
  %sesFMS_replaceMatchingSort(&lCycleDimTypes, &cSdmROFactW_BaseDS, &cSdmROFactW_BaseDS, 1, &nActionType );
  %if  &_FMErrorCode = 0 %then %do;
    %goto RETURNZERO ;
  %end;

  %sesFMS_replaceMatchingSort(&lCycleDimTypes, &cSdmROFactNoBaseDS, &cSdmROFactNoBaseDS, 0, &nActionType );
  %if  &_FMErrorCode = 0 %then %do;
    %goto RETURNZERO ;
  %end;

  %sesFMS_replaceMatchingSort(&lCycleDimTypes, &cSdmRWFactW_BaseDS, &cSdmRWFactW_BaseDS, 1, &nActionType );
  %if  &_FMErrorCode = 0 %then %do;
    %goto RETURNZERO ;
  %end;

  %sesFMS_replaceMatchingSort(&lCycleDimTypes, &cSdmRWFactNoBaseDS, &cSdmRWFactNoBaseDS, 0, &nActionType );
  %if  &_FMErrorCode = 0 %then %do;
    %goto RETURNZERO ;
  %end;


  %*put ******************************ERROR *****************;
  /*  Match merge RO w/Base Match  */
  %sesFMS_replaceMatchingMerge(&lCycleDimTypes, &cMatchRO_W_Base, &cSdmROFactW_BaseDS, &cNewFactDS1, 1, &nActionType );
  %if  &_FMErrorCode = 0 %then %do;
    %goto RETURNZERO ;
  %end;

  /*  Match merge RO No Base Match  */
  %sesFMS_replaceMatchingMerge(&lCycleDimTypes, &cMatchRO_NoBase, &cSdmROFactNoBaseDS, &cNewFactDS2, 0, &nActionType );
  %if  &_FMErrorCode = 0 %then %do;
    %goto RETURNZERO ;
  %end;

  /*  Match merge RW w/Base Match  */
  %sesFMS_replaceMatchingMerge(&lCycleDimTypes, &cMatchRW_W_Base, &cSdmRWFactW_BaseDS, &cNewFactDS1, 1, &nActionType );
  %if  &_FMErrorCode = 0 %then %do;
    %goto RETURNZERO ;
  %end;


  /*  Match merge RW No Base Match  */

  %sesFMS_replaceMatchingMerge(&lCycleDimTypes, &cMatchRW_NoBase, &cSdmRWFactNoBaseDS, &cNewFactDS2, 0, &nActionType );
  %if  &_FMErrorCode = 0 %then %do;
    %goto RETURNZERO ;
  %end;

  /*  Could have resulting dupes on matches, need to get rid of  */
  proc sort in=&cMatchRO_W_Base out=&cMatchRO_W_Base nodupkey;
       by FACT_IDX ;
  run;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %sesSOL_logError( YTDConv, L, 48, &cMatchRO_W_Base, &cMethodNm) ;
      %goto RETURNZERO;
  %end;

  proc sort in=&cMatchRO_NoBase out=&cMatchRO_NoBase nodupkey;
       by FACT_IDX ;
  run;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %sesSOL_logError( YTDConv, L, 48, &cMatchRO_NoBase, &cMethodNm) ;
      %goto RETURNZERO;
  %end;

  proc sort in=&cMatchRW_W_Base out=&cMatchRW_W_Base nodupkey;
       by FACT_IDX ;
  run;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %sesSOL_logError( YTDConv, L, 48, &cMatchRW_W_Base, &cMethodNm) ;
      %goto RETURNZERO;
  %end;

  proc sort in=&cMatchRW_NoBase out=&cMatchRW_NoBase nodupkey;
       by FACT_IDX ;
  run;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %sesSOL_logError( YTDConv, L, 48, &cMatchRW_NoBase, &cMethodNm) ;
      %goto RETURNZERO;
  %end;


  /*  Upload RO w/Base matches to temporary Remove RO table in SDM  */

  %sesFMS_uploadDataToSDM(&cMatchRO_W_Base, &cTmpROFactSDM_Nm)
  %if &_FMErrorCode = 0 %then %do;
    %sesSOL_logError( YTDConv, L, 51, &cMatchRO_W_Base, &cTmpROFactSDM_Nm, &cMethodNm) ;
    %goto RETURNZERO;
  %end;

  /*  Append RO No Base matches to temporary Remove RO table in SDM  */
  %sesFMS_uploadDataToSDM(&cMatchRO_NoBase, &cTmpROFactSDM_Nm)
  %if &_FMErrorCode = 0 %then %do;
    %sesSOL_logError( YTDConv, L, 51, &cMatchRO_NoBase, &cTmpROFactSDM_Nm, &cMethodNm) ;
    %goto RETURNZERO;
  %end ;

  /*  Append RW w/Base matches to temporary Remove RW table in SDM  */
  %sesFMS_uploadDataToSDM(&cMatchRW_W_Base, &cTmpRWFactSDM_Nm)
  %if &_FMErrorCode = 0 %then %do;
    %sesSOL_logError( YTDConv, L, 51, &cMatchRW_W_Base, &cTmpRWFactSDM_Nm, &cMethodNm) ;
    %goto RETURNZERO;
  %end;

  /*  Append RW No Base matches to temporary Remove RW table in SDM  */
  %sesFMS_uploadDataToSDM(&cMatchRW_NoBase, &cTmpRWFactSDM_Nm)
  %if &_FMErrorCode = 0 %then %do;
    %sesSOL_logError( YTDConv, L, 51, &cMatchRW_NoBase, &cTmpRWFactSDM_Nm, &cMethodNm) ;
    %goto RETURNZERO;
  %end ;


  /*  Count facts to REMOVE Imports  */
  %sesFMS_updateRemoveStatusCnt( &cStatusDS, &cMatchRO_W_Base, REMOVE_CNT );
  %if &_FMErrorCode = 0 %then %do;
    %sesSOL_logError( LoadCycleBaseFact, L, 59, &cMatchRO_W_Base, &cMethodNm);
    %goto RETURNZERO;
  %end ;

  %sesFMS_updateRemoveStatusCnt( &cStatusDS, &cMatchRO_NoBase, REMOVE_CNT );
  %if &_FMErrorCode = 0 %then %do;
    %sesSOL_logError( LoadCycleBaseFact, L, 59, &cMatchRO_NoBase, &cMethodNm);
    %goto RETURNZERO;
  %end ;

  %sesFMS_updateRemoveStatusCnt( &cStatusDS, &cMatchRW_W_Base, REMOVE_CNT );
  %if &_FMErrorCode = 0 %then %do;
    %sesSOL_logError( LoadCycleBaseFact, L, 59, &cTmpFactWork_Nm, &cMethodNm);
    %goto RETURNZERO;
  %end ;

  %sesFMS_updateRemoveStatusCnt( &cStatusDS, &cMatchRW_NoBase, REMOVE_CNT );
  %if &_FMErrorCode = 0 %then %do;
    %sesSOL_logError( LoadCycleBaseFact, L, 59, &cMatchRW_NoBase, &cMethodNm);
    %goto RETURNZERO;
  %end ;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_replaceMatching;


/**********************************************************
 * sesFMS_replaceMatchingExtract.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_replaceMatchingExtract.sas                                              */
/*  Purpose: Extract RO & RW Facts from SDM.                                                 */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_replaceMatchingExtract(lTimeAnalysisIds,
                                       lCycleDimTypes,
                                       cSdmFactSetNm,
                                       cSdmROFactW_BaseDS,
                                       cSdmROFactNoBaseDS,
                                       cSdmRWFactW_BaseDS,
                                       cSdmRWFactNoBaseDS,
                                       nSdmROFactSetId,
                                       nSdmRWFactSetId,
                                       cDLObjTypeId,
                                       nBaseMbrSDM_Id,
                                       nBaseJournMbrSDM_Id);

  %let cMethodNm = sesFMS_replaceMatchingExtract;
  %let nowDTTM = %qsysfunc(datetime());
  %let nTrackTime = %qsysfunc(datetime());

  /* ------ _stmt1 --------*/
  filename _stmt1 temp;
  data _null_;
    file _stmt1;
    set &lCycleDimTypes end=eof;
    code = strip(code);
    put '"' code +(-1) '"n ' ;
    if eof then
      put ' FACT_IDX FACT_SET_ID';
  run;

  /* ------ _stmt2 --------*/
  filename _stmt2 temp;
  data _null_;
    file _stmt2;
    set &lCycleDimTypes end=eof;
    code = strip(code);
    put '"' code +(-1) '"n = input(kscan(CONCAT_MEMBER_IDS, ' _N_ ', ","), 32.);';
  run;

  /* ------ _stmt3 --------*/
  filename _stmt3 temp;
  data _null_;
    file _stmt3;
    set &lTimeAnalysisIds end=eof;
    code = strip(code);
    _time_str='"TIME"n';
    code = tranwrd(code, "TIME", _time_str);
    _analysis_Str='"ANALYSIS"n';
    code = tranwrd(code, "ANALYSIS", _analysis_str);
    if _N_ = 1 then do;
      put  'if ' code  ;
    end;
    else do;
      put ' or ' code  ;
    end;
    if eof then
      put ';';
  run;

  data &cSdmROFactW_BaseDS
       &cSdmROFactNoBaseDS
       &cSdmRWFactW_BaseDS
       &cSdmRWFactNoBaseDS;

       keep
          %inc _stmt1;;

       length
          %inc _stmt1; 8;

       set &cSdmFactSetNm(where=(FACT_SET_ID=&nSdmROFactSetId or
                                  FACT_SET_ID=&nSdmRWFactSetId )) ;
       %inc _stmt2;

     /*  Time/Analysis subsetting  */
     /*  %inc _stmt3; */

       /*  Split SDM Facts to appropriate work area Tables  */
     if FACT_SET_ID = &nSdmROFactSetId then do ;
         if OBJECT_TYPE_ID=&cDLObjTypeId then do ;
            output &cSdmROFactW_BaseDS ;
           end ;
           else do ;
            output &cSdmROFactNoBaseDS ;
           end ;
     end ;
       else do ;
        if OBJECT_TYPE_ID=&cDLObjTypeId then do ;
            output &cSdmRWFactW_BaseDS ;
          end ;
          else do ;
            output &cSdmRWFactNoBaseDS ;
          end ;
     end ;
  run;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %sesSOL_logError( YTDConv, L, 46, &cMethodNm);
    %goto RETURNZERO;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_replaceMatchingExtract;


/**********************************************************
 * sesFMS_replaceMatchingMerge.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_replaceMatchingMerge.sas                                                */
/*  Purpose: Merge sorted tables for matches.                                                */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_replaceMatchingMerge(lCycleDimTypes, cMatchDS, cSDMFactDS, cDDSFactDS, isBaseFactSet, nActionType);

  %let cMethodNm = sesFMS_replaceMatchingMerge;
  %let nowDTTM = %qsysfunc(datetime());
  %let nTrackTime = %qsysfunc(datetime());

  /* count num of rows in lCycleDimTypes */
  filename _stmt temp;
  data _null_;
    file _stmt;
    set &lCycleDimTypes end=eof;
    code = strip(code);
    if (&nActionType eq 4 and not &isBaseFactSet)  OR
        (&nActionType eq 5) then do;
      if  (code NE 'CURRENCY') and
          (code NE 'SOURCE') then do;
          put '"' code +(-1) '"n ' ;
      end;
    end;
    else do;
       put '"' code +(-1) '"n ' ;
    end;
    if eof then
      put ';';

  run;

  data &cMatchDS(keep=FACT_IDX);
       merge &cSDMFactDS(in=sdmIn) &cDDSFactDS(in=ddsIn) ;
          by
            %inc _stmt;
          if sdmIn and ddsIn ;
  run ;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %sesSOL_logError( YTDConv, L, 49, &cSDMFactDS, &cDDSFactDS, &cMethodNm) ;
    %goto RETURNZERO;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend  sesFMS_replaceMatchingMerge;


/**********************************************************
 * sesFMS_replaceMatchingSort.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_replaceMatchingSort.sas                                                 */
/*  Purpose: Sort work area table.                                                           */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_replaceMatchingSort(lCycleDimTypes, cInFactDS, cOutFactDS, isBaseFactSet, nActionType);

  %let cMethodNm = sesFMS_replaceMatchingSort;
  %let nowDTTM = %qsysfunc(datetime());
  %let nTrackTime = %qsysfunc(datetime());

  filename _stmt temp;
  data _null_;
    file _stmt;
    set &lCycleDimTypes end=eof;
    code = strip(code);
    if (&nActionType eq 4 and not &isBaseFactSet)  OR
        (&nActionType eq 5) then do;
      if  (code NE 'CURRENCY') and
          (code NE 'SOURCE') then do;
            put '"' code +(-1) '"n ' ;
      end;
    end;
    else do;
     put '"' code +(-1) '"n ' ;
    end;
    if eof then
      put ';';
  run;

  proc sort in=&cInFactDS out=&cOutFactDS ;
       by
        %inc _stmt;
  run;


  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %let error_number=48;
    %let errObj=&cInFactDS;
    %goto ERRORRETURN;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesSOL_logError( YTDConv, L, &error_number, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_replaceMatchingSort;


/**********************************************************
 * sesFMS_replaceMatSplitNewFacts.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_replaceMatSplitNewFacts.sas                                             */
/*  Purpose: Split concat member RKS work area new fact DS.                                  */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */
%macro sesFMS_replaceMatSplitNewFacts(lCycleDimTypes, cNewFactDS, cNewFactsConCatDS);

  %let cMethodNm =sesFMS_replaceMatchingSplitNewFacts;
  %let nowDTTM = %qsysfunc(datetime());
  %let nTrackTime = %qsysfunc(datetime());

  /* ------ _stmt1 --------*/
  filename _stmt1 temp;
  data _null_;
    file _stmt1;
    set &lCycleDimTypes end=eof;
    code = strip(code);
    put '"' code +(-1) '"n ' ;
  run;

  /* ------ _stmt2 --------*/
  filename _stmt2 temp;
  data _null_;
    file _stmt2;
    set &lCycleDimTypes end=eof;
    code = strip(code);
    put '"' code +(-1) '"n = input(kscan(CONCAT_MEMBER_IDS, ' _N_ ', ","), 32.);';
  run;


  data &cNewFactDS;
     keep
        %inc _stmt1;;
     length
        %inc _stmt1 ; 8;
     set  &cNewFactsConCatDS;
       %inc _stmt2;
  run;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %let error_number=47;
    %let errObj=&cNewFactsConCatDS;
    %goto ERRORRETURN;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesSOL_logError( YTDConv, L, &error_number, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_replaceMatSplitNewFacts;


/**********************************************************
 * sesFMS_sortDataSet.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_sortDataSet.sas                                                         */
/*  Purpose: Sort data set.                                                                  */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */
%macro sesFMS_sortDataSet(cDSName, sortKey);

  %let cMethodNm = sesFMS_sortDataSet;
  %let nTrackTime = %qsysfunc(datetime());

  /*  Make sure we have a source transaction table in the DDS  */
  %if not %sysfunc(exist(&cDSName)) %then %do;
    %let error_number=15;
    %let errObj=&cInputDS;
    %goto ERRORRETURN;
  %end ;

  /*  Build the column sort by list  */
  filename _col temp;
  data _null_;
      file _col;
    set &sortKey end=eof;
    put '"' value + (-1) '"n ';
      if eof then put ';';
  run;

  /* Sort using cCols */
  proc sort in=&cDSName out=&cDSName ;
       by %inc _col ;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %let error_number=17;
    %let errObj=&cDSName;
    %goto ERRORRETURN;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesSOL_logError( YTDConv, L, &error_number, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_sortDataSet;


/**********************************************************
 * sesFMS_summarizeDataSet.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_summarizeDataSet.sas                                                    */
/*  Purpose: Summarize data set.                                                             */
/*                                                                                           */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_summarizeDataSet(cDSName, lTranCols, cAmtColNm, nIDTimePeriod);

  %let cMethodNm = sesFMS_summarizeDataSet;
  %let nTrackTime = %qsysfunc(datetime());

  /*  Make sure we have a source transaction table in the DDS  */
  %if not %sysfunc(exist(&cDSName))  %then %do;
    %let error_number=15;
    %let errObj=&cDSName;
    %goto ERRORRETURN;
  %end ;

  /*  Build the column summary by list  */
  filename _col temp;
  data _null_;
      file _col;
    set &lTranCols end=eof;
    put '"' value + (-1) '"n ';
      if eof then put ';';
  run;

  /* conditional code */
  %let subset_org_code=;
  %if &nIDTimePeriod %then %do;
    %if &cProduct eq BASE %then %do;
       %let subset_org_code= %str(id AFFECTED_TIME_PERIOD_RK;);
    %end;
    %else %do;
       %let subset_org_code= %str(id TIME_PERIOD_RK;);
    %end;
  %end ;

  /*  Run the summarization  */
  proc summary data=&cDSName nway ;
       by %inc _col ;
      &subset_org_code
          var &cAmtColNm ;
          output out=&cDSName (drop=_type_ _freq_) sum= ;
  run ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %let error_number=18;
    %let errObj=&cDSName;
    %goto ERRORRETURN;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesSOL_logError( YTDConv, L, &error_number, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_summarizeDataSet;


/**********************************************************
 * sesFMS_updateInvalidStatusCnt.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_updateInvalidStatusCnt.sas                                              */
/*  Purpose: Update Invalid Status Count.                                                    */
/*                                                                                           */
/*  History    23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_updateInvalidStatusCnt(cStatusDS, cErrorDS, cErrCntColNm);

  %let cMethodNm = sesFMS_updateInvalidStatusCnt;
  %let cSummaryDS = work._tempFact1;
  %let nowDTTM = %qsysfunc(datetime());
  %let nTrackTime = %qsysfunc(datetime());

  %if %sysfunc(exist(&cSummaryDS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cSummaryDS, 2, '.');
    quit;
  %end;

  %let _nobs = 0;
  proc sql noprint;
    select count(*) into :_nobs
    from &cErrorDS
    ;
  quit;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  %if &_nobs gt 0 %then %do;
    proc sql noprint;
      update &cStatusDS
      set &cErrCntColNm = &_nobs
        , status_id = 2
      ;
    quit;
  %end;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_updateInvalidStatusCnt;


/**********************************************************
 * sesFMS_updateRemoveStatusCnt.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_updateRemoveStatusCnt.sas                                                */
/*  Purpose: Update Valid Status Count.                                                      */
/*                                                                                           */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_updateRemoveStatusCnt(cStatusDS, cDS, cImpCntColNm);

  %let cMethodNm = sesFMS_updateRemoveStatusCnt;
  %let nowDTTM = %qsysfunc(datetime());
  %let nTrackTime = %qsysfunc(datetime());

  %let _nobs = 0;
  data _null_;
    set &cDS nobs=_nobs;
    call symput('_nobs', _nobs);
    stop;
  run;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  proc sql noprint;
    update &cStatusDS
    set &cImpCntColNm = &cImpCntColNm +
       case when status_id ne 0
         then 0
         else &_nobs
       end
    ;
  quit;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_updateRemoveStatusCnt;


/**********************************************************
 * sesFMS_updateValidStatusCnt.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_updateValidStatusCnt.sas                                                */
/*  Purpose: Update Valid Status Count.                                                      */
/*                                                                                           */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_updateValidStatusCnt(cStatusDS, cValidDS, cImpCntColNm);

  %let cMethodNm = sesFMS_updateValidStatusCnt;
  %let cSummaryDS = work._tempFact1;
  %let nowDTTM = %qsysfunc(datetime());
  %let nTrackTime = %qsysfunc(datetime());

  %if %sysfunc(exist(&cSummaryDS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cSummaryDS, 2, '.');
    quit;
  %end;

  %let _nobs = 0;
  proc sql noprint;
    select count(*) into :_nobs
    from &cValidDS
    where FACT_VALUE ne 0
    ;
  quit;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  proc sql noprint;
    update &cStatusDS
    set &cImpCntColNm =
       case when status_id ne 0
         then 0
         else &_nobs
       end
     , status_id =
       case when status_id eq 0 and &_nobs eq 0
         then 3
         else status_id
       end
    ;
  quit;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_updateValidStatusCnt;



/**********************************************************
 * sesFMS_updateZeroStatusCnt.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_updateZeroStatusCnt.sas                                                           */
/*  Purpose: Update Zero Status Count.                                                       */
/*                                                                                           */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_updateZeroStatusCnt(cStatusDS, cValidDS, cImpCntColNm);

  %let cMethodNm = sesFMS_updateZeroStatusCnt;
  %let cSummaryDS = WORK._TEMPFACT1;
  %let nowDTTM = %qsysfunc(datetime());
  %let nTrackTime = %qsysfunc(datetime());

  %if %sysfunc(exist(&cSummaryDS)) %then  %do;
    proc datasets library=work nolist;
      delete %kscan(&cSummaryDS, 2, '.');
    quit;
  %end;

  %let _nobs = 0;
  proc sql noprint;
    select count(*) into :_nobs
    from &cValidDS
    where FACT_VALUE eq 0
    ;
  quit;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  proc sql noprint;
    update &cStatusDS
    set &cImpCntColNm =
       case when status_id eq 2
         then 0
         else &_nobs
       end
     , status_id =
       case when status_id ne 2 and &_nobs gt 0 and IMPORTED_TRANS_CNT = 0
         then 0
         else status_id
       end
    ;
  quit;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_updateZeroStatusCnt;


/**********************************************************
 * sesFMS_uploadDataToSDM.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_uploadDataToSDM.sas                                                     */
/*  Purpose: Upload Table to SDM.                                                            */
/*                                                                                           */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */
%macro sesFMS_uploadDataToSDM(cSrcNm, cTrgNm);

  %let cMethodNm = sesFMS_uploadDataToSDM;
  %let nTrackTime = %qsysfunc(datetime());

  %let fmsetl_tempCheck1 = ;
  proc sql noprint ;
       select count(*)
         into :fmsetl_tempCheck1
         from &cSrcNm ;
  quit ;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  %let nCnt = &fmsetl_tempCheck1;
  %if &nCnt eq . %then %let nCnt = 0 ;
  %if &nCnt EQ 0 %then %do;
      /*  No records found, skip the rest  */
    %goto THEEND;
  %end ;

  /*  Create output file name  */
  %let cExportCSV = %qsysfunc(strip(%qsysfunc(tranwrd(%qsysfunc(pathname(WORK)), \, /))))/upload.txt;

  /* %let cSrcNm=%qsysfunc(&cSrcNm); */
  proc export data=&cSrcNm
       outfile = "&cExportCSV"
       dbms = DLM REPLACE;
       delimiter='~';
  run;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  proc sql noprint;
       connect to &sesetl_EngineType ( &sesetl_BestLogin &sesetl_ConnectionOptions );

       execute( load data local infile "&cExportCSV"
                into table &cTrgNm
                fields terminated by '~' optionally enclosed by '"'
                lines terminated by '\n' starting by ''
                ignore 1 lines
              ) by &sesetl_EngineType ;

       disconnect from &sesetl_EngineType ;
  quit;
  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;


      /*  Delete text file  */
  %let nRC = %qsysfunc(filename(_tmpExp, &cExportCSV)) ;
  %if %qsysfunc(fexist(_tmpExp)) %then %do;
    %qsysfunc(fdelete(_tmpExp)) ;
  %end;
  %let nRC = %qsysfunc(filename(_tmpExp, '')) ;


  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_uploadDataToSDM;


/**********************************************************
 * sesFMS_uploadStatusTableToSDM.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_uploadStatusTableToSDM.sas                                              */
/*  Putpose: Upload Status Table to SDM.                                                     */
/*                                                                                           */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_uploadStatusTableToSDM( cSrcTable, cTrgTable);

  %let cMethodNm = sesFMS_uploadStatusTable;
  %let nowDTTM = %qsysfunc(datetime());
  %let nTrackTime = %qsysfunc(datetime());

  proc sql noprint;
   connect to &sesetl_EngineType ( &sesetl_BestLogin &sesetl_ConnectionOptions );
   execute(CREATE TABLE &cTrgTable (
           TIME_ID INT(11),
           ANALYSIS_ID INT(11),
           PERIOD_ID INT(11),
           STATUS_ID INT(11),
           IMPORTED_TRANS_CNT INT(11),
           ZERO_TRANS_CNT INT(11),
           INVALID_ERROR_CNT INT(11),
           ORG_EQ_TRADER_CNT INT(11),
           IC_ERROR_CNT INT(11),
           NON_IC_ERROR_CNT INT(11),
           REMOVE_CNT INT(11)
           )engine=myISAM
           ) by &sesetl_EngineType ;
   disconnect from &sesetl_EngineType ;
  quit;

  %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
    %goto RETURNZERO;
  %end;

  %sesFMS_uploadDataToSDM(&cSrcTable, &cTrgTable)
  %if &_FMErrorCode = 0 %then %do;
    %goto RETURNZERO;
  %end ;


  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_uploadStatusTableToSDM;


/**********************************************************
 * sesFMS_uploadTempFactToSDM.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_uploadTempFactToSDM.sas                                                 */
/*  Purpose: Upload Temporary Fact Table to SDM.                                             */
/*                                                                                           */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */
%macro sesFMS_uploadTempFactToSDM (cTmpFactSDM_Nm, cTmpFactWork_Nm, cStatusDS);

  %let cMethodNm = sesFMS_uploadTempFactToSDM;
  %let nowDTTM = %qsysfunc(datetime());
  %let nTrackTime = %qsysfunc(datetime());

  /*  Get the list of Time and Analysis Ids that have okay import status   */
  proc sql noprint;
    select count(*)
      into :num_data
      from &cStatusDS
      where STATUS_ID = 0;
  quit;

  %let num_data = %eval(&num_data);
  %let where_cond_stmt=;

  %if &num_data > 0 %then %do;

    data &cTmpFactWork_Nm (drop=TIME ANALYSIS INTORG IDXCNT);
        length FACT_IDX 8 ;
        length CONCAT_MEMBER_IDS $255 ;
        length FACT_VALUE 8 ;
        length OBJECT_TYPE_ID 8 ;
        length OBJECT_ID 8 ;
        retain IDXCNT 0 ;
        set &cTmpFactWork_Nm(where=(FACT_VALUE ne 0));
        IDXCNT = IDXCNT + 1 ;
        FACT_IDX = IDXCNT ;
    run ;
    %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
      %goto RETURNZERO;
    %end;

    /*  Upload facts to SDM  */
    proc sql noprint;
        connect to &sesetl_EngineType ( &sesetl_BestLogin &sesetl_ConnectionOptions );
        execute(CREATE TABLE &cTmpFactSDM_Nm (
               FACT_IDX INT(11),
               CONCAT_MEMBER_IDS varchar(255) NOT NULL default '',
               FACT_VALUE double NOT NULL default 0,
               OBJECT_TYPE_ID int(11) NOT NULL default 0,
               OBJECT_ID int(11) NOT NULL default 0
               )engine=myISAM
               ) by &sesetl_EngineType ;
        disconnect from &sesetl_EngineType ;
    quit;
    %if (&SYSERR ne 0 and &SYSERR ne 4) %then %do;
      %goto RETURNZERO;
    %end;

    %sesFMS_uploadDataToSDM(&cTmpFactWork_Nm, &cTmpFactSDM_Nm);
    %if &_FMErrorCode = 0 %then %do;
      %goto RETURNZERO;
    %end ;

  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;
  %THEEND:

%mend sesFMS_uploadTempFactToSDM;


/**********************************************************
 * sesFMS_verifyDDSTransModel.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_verifyDDSTransModel.sas                                                 */
/*  Purpose: Verify DDS trans. table contains standard model columns.                        */
/*                                                                                           */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */

%macro sesFMS_verifyDDSTransModel(cDDSLibRef, cDDSTransTableNm, lCycleDimTypes, cDDSModel);

  %let cMethodNm = sesFMS_verifyDDSTransModel;
  %let cDDSLibRef = %qsysfunc(strip(&cDDSLibRef));
  %let cDDSTransTableNm = %qsysfunc(strip(&cDDSTransTableNm));
  %let cInputDS =&cDDSLibRef.&cDDSTransTableNm;
  %let nTrackTime = %qsysfunc(datetime());

  /*  Build a list of columns from the physical DDS table  */
  proc sql noprint;
    create table _lTranCols as
      select upcase(name) as code
        from dictionary.columns
       where libname="&cDDSLibRef" and
           memname="&cDDSTransTableNm"
      order by code;
  quit;

  /* Does the DDS.gl_transaction_sum table contain the standard list of columns ? */
  /* Does the Exclude list of columns exist in the physical table  */
  /* lDDSTranCols and lDDSTranOPCols have been created by YTDConv */

  proc sort in=&cDDSModel ;
    by code;
  run;

  data _tmp_a_minus_b;
     merge  &cDDSModel(in=a) _lTranCols(in=b);
     by code;
     if a and not b then do;
      output;
     end;
  run;

  /* count # of rows in the temp table */
  %if (%qsysfunc(exist(_tmp_a_minus_b))) %then %do;
    %let numRows=0;
    proc sql noprint;
      select count(*)
        into :numRows
          from _tmp_a_minus_b;
     quit;
      %let numRows=%eval(&numRows);

    %if &numRows > 0 %then %do;
      proc sql noprint;
        select code
          into :_tmpRow1 - :_tmpRow&numRows
          from _tmp_a_minus_b;

      %do ii=1 %to &numRows;
        %sesSOL_logError(YTDConv, L, 13,  &cDDSTransTableNm, &&_tmpRow&ii, &cMethodNm);
      %end;
      %goto RETURNZERO;
    %end;
  %end;
  %else %do;
    &errorObj= _tmp_a_minus_b;
    %goto RETURNZERO;
  %end;

  /*  Does the DDS.gl_transaction_sum table contain the dimension type columns the Cycle is expecting ?  */
  proc sql noprint;
    create table work._lcycle_ldds_dim as
    select  d.value as code
      from  lCycleDimTypes c, lDdsDimtypes d
       where c.code=d.code;

  proc sort in=_lcycle_ldds_dim out=_lcycle_ldds_dim;
    by code;
  run;

  data _tmp_a_minus_b;
       merge  _lcycle_ldds_dim(in=a) _lTranCols(in=b);
     by code;
     if a and not b then do;
      output;
     end;
  run;

  /* count # of rows in the temp table */
  %if (%qsysfunc(exist(_tmp_a_minus_b))) %then %do;
    %let numRows=0;
    proc sql noprint;
      select count(*)
        into :numRows
          from _tmp_a_minus_b;
     quit;
      %let numRows=%eval(&numRows);

    %if &numRows > 0 %then %do;
      proc sql noprint;
        select code
          into :_tmpRow1 - :_tmpRow&numRows
          from _tmp_a_minus_b;

      %do ii=1 %to &numRows;
        %sesSOL_logError(YTDConv, L, 14,  &cDDSTransTableNm, &&_tmpRow&ii, &cMethodNm);
      %end;
      %goto  RETURNZERO;
    %end;
    /* clean the temp work table */
    /*
    proc datasets library=work nolist;
      delete _tmp_a_minus_b;
    quit;
    */

  %end;
  %else %do;
    &errorObj= _tmp_a_minus_b;
    %goto RETURNZERO;
  %end;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend sesFMS_verifyDDSTransModel;


/**********************************************************
 * sesFMS_verifyDimensionTypes.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_verifyDimensionTypes.sas                                                */
/*  Purpose: Verify Marts Cycle dimension types with DDS Dimension types.                    */
/*           There will be a specific list of Dimension Types associated to the Mart's Cycle.*/
/*           We need to make sure that those dimension types also exist in the DDS Dimension */
/*           Type table.  Otherwise, we have no hope of matching Cycle dimension types to    */
/*           columns in the DDS gl_transaction_sum table.                                    */
/*           One exception:  The SOURCE Dimension type does not exist in the DDS, but will   */
/*                           in the mart.                                                    */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */
%macro sesFMS_verifyDimensionTypes(lCycleDimTypes, lDDSDimTypes);

  %let cMethodNm = sesFMS_verifyDimensionTypes;
  %let nTrackTime = %qsysfunc(datetime());

  /*  Does the Exclude list of columns exist in the physical table  */
  data _tmp_a_minus_b;
       merge  &lCycleDimTypes(in=a) &lDDSDimTypes(in=b);
     by code;
     if a and not b then do;
      output;
     end;
  run;

  /* count # of rows in the temp table */
  %if (%qsysfunc(exist(work._tmp_a_minus_b))) %then %do;
    proc sql noprint;
      select count(*)
        into :numRows
          from _tmp_a_minus_b;
     quit;
      %let numRows=%eval(&numRows);

    %if &numRows > 0 %then %do;
      proc sql noprint;
        select code
          into :_tmpRow1 - :_tmpRow&numRows
          from _tmp_a_minus_b;
      quit;
      %do ii=1 %to &numRows;
        %sesSOL_logError(YTDConv, L, 12, &&_tmpRow&ii, &cMethodNm);
      %end;
      %goto  RETURNZERO;
    %end;
    /* clean the temp work table */
    proc datasets library=work nolist;
      delete _tmp_a_minus_b;
    quit;

  %end;
  %else %do;
    &errorObj= _tmp_a_minus_b;
    %goto ERRORRETURN;
  %end;


  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesSOL_logError( YTDConv, L, 12, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend  sesFMS_verifyDimensionTypes;


/**********************************************************
 * sesFMS_verifyNgetDimTypeRK.sas
 **********************************************************/

/* ----------------------------------------------------------------------------------------- */
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.             */
/*  Name   : sesFMS_verifyNgetDimTypeRK.sas                                                 */
/*  Purpose: Verify Marts Cycle dimension types with DDS Dimension types.                    */
/*           It also stores the corresponding Fact table Dimension Type RKs                  */
/*           in the new table, lCycleDimTypesRKs                                             */
/*                                                                                           */
/*           There will be a specific list of Dimension Types associated to the Mart's Cycle.*/
/*           We need to make sure that those dimension types also exist in the DDS Dimension */
/*           Type table.  Otherwise, we have no hope of matching Cycle dimension types to    */
/*           columns in the DDS gl_transaction_sum table.                                    */
/*           One exception:  The SOURCE Dimension type does not exist in the DDS, but will   */
/*                           in the mart.                                                    */
/*  History:   23June2008 by Tae Sik Han : Converted from SCL to SAS macro program.          */
/* ----------------------------------------------------------------------------------------- */
%macro sesFMS_verifyNgetDimTypeRK(lCycleDimTypes, lDDSDimTypes, lCycleDimTypesRKs);

  %let cMethodNm = sesFMS_verifyDimensionTypes;
  %let nTrackTime = %qsysfunc(datetime());

  /*  Does the Exclude list of columns exist in the physical table  */
  data _tmp_a_minus_b;
       merge  &lCycleDimTypes(in=a where=(code ne 'SOURCE')) &lDDSDimTypes(in=b);
     by code;
     if a and not b then do;
      output;
     end;
  run;

  /* count # of rows in the temp table */
  %if (%qsysfunc(exist(work._tmp_a_minus_b))) %then %do;
    proc sql noprint;
      select count(*)
        into :numRows
          from _tmp_a_minus_b;
     quit;
      %let numRows=%eval(&numRows);

    %if &numRows > 0 %then %do;
      proc sql noprint;
        select code
          into :_tmpRow1 - :_tmpRow&numRows
          from _tmp_a_minus_b;
      quit;
      %do ii=1 %to &numRows;
        %sesSOL_logError(YTDConv, L, 12, &&_tmpRow&ii, &cMethodNm);
      %end;
      %goto  RETURNZERO;
    %end;
    /* clean the temp work table */
    proc datasets library=work nolist;
      delete _tmp_a_minus_b;
    quit;

  %end;
  %else %do;
    &errorObj= _tmp_a_minus_b;
    %goto ERRORRETURN;
  %end;

  /*Get corresponding dimension type RK using lDDSDimTypes tables extracted from DDS */
  %if %sysfunc(exist(&lCycleDimTypesRKs)) %then  %do;
    proc datasets library=work nolist;
      delete &lCycleDimTypesRKs;
    quit;
  %end;

  proc sql noprint;
    create table &lCycleDimTypesRKs as
    select b.code, b.value
        from &lCycleDimTypes a, &lDDSDimTypes b
     where a.code=b.code;
  quit;

  /*  Dump the Time  */
  %sesSOL_dumpTimer( &nTrackTime, &cMethodNm );
  %goto THEEND; /* if successful, set global _FMErrorCode to 1 and quit this macro */

  %ERRORRETURN:
    %sesSOL_logError( YTDConv, L, 12, &errObj, &cMethodNm);

  %RETURNZERO:
    %let _FMErrorCode=0;

  %THEEND:

%mend  sesFMS_verifyNgetDimTypeRK;

%MEND sesfms;
